EnableExplicit

;{ STRUCTURES

Structure T_LIST_ROOT
 *current
 *first
 *last
 *fpFreeMem
 iSize.i
 iItems.i
EndStructure

Structure T_LIST_ITEM
 *next
 *prev
EndStructure

;}

;{ DECLARES

Declare.i   LinkedList_New (iItemSize, *fpFreeMem)
Declare.i   LinkedList_Select (*list.T_LIST_ROOT, *item.T_LIST_ITEM)
Declare.i   LinkedList_Delete (*list.T_LIST_ROOT)
Declare.i   LinkedList_Insert (*list.T_LIST_ROOT)
Declare.i   LinkedList_Add (*list.T_LIST_ROOT)
Declare.i   LinkedList_Current (*list.T_LIST_ROOT)
Declare.i   LinkedList_First (*list.T_LIST_ROOT)
Declare.i   LinkedList_Last (*list.T_LIST_ROOT)
Declare.i   LinkedList_Count (*list.T_LIST_ROOT)
Declare.i   LinkedList_Next (*list.T_LIST_ROOT)
Declare.i   LinkedList_Previous (*list.T_LIST_ROOT)
Declare     LinkedList_Clear (*list.T_LIST_ROOT)
Declare     LinkedList_Free (*list.T_LIST_ROOT)
Declare     LinkedList_Sort (*list.T_LIST_ROOT, *fpCompare)

;}

; DEPS
XIncludeFile "SortStructuredEx.pb"
   
Procedure.i LinkedList_New (iItemSize, *fpFreeMem)
 Protected *list.T_LIST_ROOT = AllocateMemory(SizeOf(T_LIST_ROOT))
 
 If *list
    *list\first = 0
    *list\last = 0
    *list\current = 0
    *list\fpFreeMem = *fpFreeMem
    *list\iSize = iItemSize
    *list\iItems = 0
 EndIf
 
 ProcedureReturn *list
EndProcedure

Procedure.i LinkedList_Select (*list.T_LIST_ROOT, *item.T_LIST_ITEM)
 If *list
    *list\current = *item
    ProcedureReturn *list\current
 EndIf
 ProcedureReturn 0
EndProcedure

Procedure.i LinkedList_Delete (*list.T_LIST_ROOT)
; the previous element become the new current element
; if there is not a previous element the next element (if any) becomes the current

 Protected *item.T_LIST_ITEM 
 Protected *item_prev.T_LIST_ITEM 
 Protected *item_next.T_LIST_ITEM  
 Protected *current.T_LIST_ITEM 
    
 If *list
    *item = *list\current
    
    If *item
    
        *item_prev = *item\prev
        *item_next = *item\next
    
    
        If *item_prev
            *current = *item_prev                    
        ElseIf *item_next
            *current = *item_next
        Else 
            *current = 0
        EndIf
                
        
        If *item_prev And *item_next
            *item_prev\next = *item_next
            *item_next\prev = *item_prev    
        EndIf
        

        If *item_prev And *item_next = 0
            *item_prev\next = 0    
            *list\last = *current
        EndIf


        If *item_prev = 0 And *item_next
            *item_next\prev = 0
            *list\first = *current       
        EndIf
                      

        *list\current = *current 
        
        *list\iItems - 1    
        
        If *list\fpFreeMem 
            CallFunctionFast(*list\fpFreeMem, *item)
        EndIf
        
        FreeMemory(*item) 
    EndIf 
       
 EndIf   
 
 ProcedureReturn *current 
EndProcedure

Procedure.i LinkedList_Insert (*list.T_LIST_ROOT)

 Protected *item.T_LIST_ITEM = 0
 
 Protected *current.T_LIST_ITEM, *ptr.T_LIST_ITEM
 
 If *list

    *item = AllocateMemory (*list\iSize)
    
    If *item
    
        If *list\current
            *current = *list\current
            
            If *current\prev
                *ptr = *current\prev
                *current\prev = *item
                *item\prev = *ptr
                *item\next = *current
                *ptr\next = *item
            Else
                *current\prev = *item                
                *item\next = *current
                *item\prev = 0
                
                *list\first = *item
            EndIf            
        Else                       
            *list\first = *item
            *list\last = *item        
        EndIf
                                
        *list\current = *item
        
        *list\iItems + 1
    EndIf
    
 EndIf
 
 ProcedureReturn *item
EndProcedure

Procedure.i LinkedList_Add (*list.T_LIST_ROOT)

 Protected *item.T_LIST_ITEM = 0
 
 Protected *current.T_LIST_ITEM, *ptr.T_LIST_ITEM
 
 If *list

    *item = AllocateMemory (*list\iSize)
    
    If *item
    
        If *list\current
            *current = *list\current
            
            If *current\next
                *ptr = *current\next
                *current\next = *item
                *item\next = *ptr
                *item\prev = *current
            Else
                *current\next = *item                
                *item\prev = *current
                *item\next = 0
                
                *list\last = *item
            EndIf            
        Else                       
            *list\first = *item
            *list\last = *item        
        EndIf
                                
        *list\current = *item
        
        *list\iItems + 1
    EndIf
    
 EndIf
 
 ProcedureReturn *item
EndProcedure

Procedure.i LinkedList_Current(*list.T_LIST_ROOT)
 If *list
    ProcedureReturn *list\current
 EndIf 
 ProcedureReturn 0
EndProcedure

Procedure.i LinkedList_First (*list.T_LIST_ROOT)

 If *list
    *list\current = *list\first
    ProcedureReturn *list\first
 EndIf
 
 ProcedureReturn 0
EndProcedure

Procedure.i LinkedList_Last (*list.T_LIST_ROOT)

 If *list
    *list\current = *list\last
    ProcedureReturn *list\last
 EndIf
 
 ProcedureReturn 0
EndProcedure

Procedure.i LinkedList_Count (*list.T_LIST_ROOT)
 If *list
    ProcedureReturn *list\iItems
 EndIf
 
 ProcedureReturn 0
EndProcedure

Procedure.i LinkedList_Next (*list.T_LIST_ROOT)

 Protected *item.T_LIST_ITEM = 0
    
 If *list
    If *list\current
        *item = *list\current
        *item = *item\next
        If *item
            *list\current = *item    
        EndIf
    EndIf
 EndIf
 
 ProcedureReturn *item
 
EndProcedure

Procedure.i LinkedList_Previous (*list.T_LIST_ROOT)

 Protected *item.T_LIST_ITEM = 0

 If *list
    If *list\current
        *item = *list\current
        *item = *item\prev
        If *item
            *list\current = *item    
        EndIf
    EndIf
 EndIf
    
 ProcedureReturn *item
 
EndProcedure

Procedure LinkedList_Clear (*list.T_LIST_ROOT)

 Protected *item.T_LIST_ITEM = 0
 Protected *current.T_LIST_ITEM = 0
 
 If *list
    *item = *list\first    
    
    While *item
        
        *current = *item
        
        *item = *item\next

        If *list\fpFreeMem 
            CallFunctionFast(*list\fpFreeMem, *current)
        EndIf
        
        FreeMemory(*current) 
        
    Wend               
    
    *list\first = 0
    *list\last = 0
    *list\current = 0
    *list\iItems = 0
    
 EndIf
 
EndProcedure

Procedure LinkedList_Free (*list.T_LIST_ROOT)
 
 If *list 
    LinkedList_Clear(*list)
    
    FreeMemory(*list)     
 EndIf
 
EndProcedure

Procedure LinkedList_Sort (*list.T_LIST_ROOT, *fpCompare)
 SortStructuredListEx (LinkedList_First(*list), LinkedList_Last(*list), LinkedList_Count(*list), *list\iSize, *fpCompare, 0)
EndProcedure
; IDE Options = PureBasic 5.73 LTS (Windows - x86)
; Folding = ----
; EnableXP
; EnableUser
; CPU = 1
; DisableDebugger
; CompileSourceDirectory
; EnableBuildCount = 0