#MEGABYTE = 1048576.0 

Declare.s LogBuildFileName (PdfFileName$)
Declare.i LogStart (ProgramString$, PdfFileName$, LogFileName$, *Options.T_OPTIONS)
Declare   LogAddMissing (ImageFileName$)
Declare   LogAddFailed (iReason, ImageFileName$)
Declare   LogAddProcessed (ImageFileName$, iFileSize, iFileSizeInsidePdf)
Declare   LogClose ()
Declare   LogDelete()
Declare.s LogGetFileName()

;****************************
;- STRUCTURES / GLOBALS 
;****************************

Structure T_LOGFILE
   LogFileName$
   PdfFileName$
   nNumFile.i
   iStartTime.i
   iEndTime.i
   iCurrPage.i
   iTotFiles.i
   iMissingFiles.i
   iFailedFiles.i
   qTotBytesProcessed.q
EndStructure

; The following variable is shared only in procedures in this file,
; but not in procedures in other files.
Define s_tLogfile.T_LOGFILE

Procedure.s LogBuildFileName (PdfFileName$)
   ProcedureReturn GetPathPart(PdfFileName$) + GetFilePart(PdfFileName$) + ".log"
EndProcedure

Procedure.i LogStart (ProgramString$, PdfFileName$, LogFileName$, *Options.T_OPTIONS)
   Shared s_tLogfile
   Protected Text$
   
   s_tLogfile\PdfFileName$ = PdfFileName$
   s_tLogfile\LogFileName$ = LogFileName$
   s_tLogfile\nNumFile = CreateFile(#PB_Any, LogFileName$)
   
   s_tLogfile\iStartTime = Date() 
   s_tLogfile\iEndTime = 0
   
   s_tLogfile\iCurrPage = 0
   s_tLogfile\iTotFiles = 0
   s_tLogfile\iMissingFiles = 0
   s_tLogfile\iFailedFiles = 0
   s_tLogfile\qTotBytesProcessed = 0
   
   If s_tLogfile\nNumFile 
      Text$ + ProgramString$ + #CRLF$ + #CRLF$
      
      Text$ + "Image compression = "
      
      If (*Options\flgJpgCompression)
         Text$ + "JPG (quality: " + Str(*Options\iJpgQuality) + ")"
      Else
         Text$ + "PNG (lossless)"
      EndIf
      
      Text$ + #CRLF$
      
      
      Text$ + "Resizing = " + Str(*Options\iImageScaling) + " %" + #CRLF$
      
      If *Options\iResizeOnlyIfWider > 0
         Text$ + "Resize only if width > " + Str(*Options\iResizeOnlyIfWider) + #CRLF$
      EndIf
      
      If *Options\iPaperFormat <> #PAPER_FORMAT_PAPERLESS
         
         Text$ + "Orientation = "
         
         If *Options\iPaperOrientation = #PAPER_PORTRAIT
            Text$ + "Portrait" 
         Else
            Text$ + "Landscape" 
         EndIf
         
         Text$ + #CRLF$
         
      EndIf
      
      
      Text$ + "Paper format = "
      
      Select *Options\iPaperFormat
         Case #PAPER_FORMAT_A3         : Text$ + "A3"
         Case #PAPER_FORMAT_A4         : Text$ + "A4"
         Case #PAPER_FORMAT_LEGAL      : Text$ + "Legal"
         Case #PAPER_FORMAT_LETTER     : Text$ + "Letter"
         Case #PAPER_FORMAT_PAPERLESS  : Text$ + "Paperless"
      EndSelect 
      
      Text$ + #CRLF$
      
      Text$ + "Startup mode = "
      
      Select *Options\iPdfStartup
         Case #PDF_ZOOM_FULLPAGE     : Text$ + "Fit page"
         Case #PDF_ZOOM_FULLWIDTH    : Text$ + "Fit width"
         Case #PDF_ZOOM_REAL         : Text$ + "True size"
         Case #PDF_ZOOM_FULLSCREEN   : Text$ + "Full screen"    
      EndSelect 
      
      Text$ + #CRLF$
      
      If *Options\iPaperFormat <> #PAPER_FORMAT_PAPERLESS
         
         Text$ + "Enlarge to fit page if smaller = "
         
         If (*Options\flgEnlargeToFit)
            Text$ + "True"
         Else
            Text$ + "False"
         EndIf
         
         Text$ + #CRLF$
         
         Text$ + "Keep original image aspect ratio = "
         
         If (*Options\flgKeepAspect)
            Text$ + "True"
         Else
            Text$ + "False"
         EndIf
         
         Text$ + #CRLF$
         
      EndIf     
      
      If *Options\flgJpgNoRecompression     
         Text$ + "JPG storing mode is enabled"                  
         Text$ + #CRLF$   
      EndIf  
      
      Text$ + #CRLF$
      
      WriteString(s_tLogfile\nNumFile, Text$)
   EndIf
   
   ProcedureReturn s_tLogfile\nNumFile ; 0 if failed
EndProcedure


Procedure LogAddMissing (ImageFileName$)
   Shared s_tLogfile
   Protected Text$
   
   If IsFile(s_tLogfile\nNumFile) = 0 : ProcedureReturn : EndIf
   
   s_tLogfile\iMissingFiles + 1
   
   Text$ + "[ERR] " + ImageFileName$ + " is missing !" + #CRLF$
   
   WriteString(s_tLogfile\nNumFile, Text$)
EndProcedure


Procedure LogAddFailed (iReason, ImageFileName$)
   Shared s_tLogfile
   Protected Text$
   
   If IsFile(s_tLogfile\nNumFile) = 0 : ProcedureReturn : EndIf
   
   s_tLogfile\iFailedFiles + 1
   
   Select iReason
      Case #I2PDF_LOG_UNSUPPORTED
         Text$ + "[ERR] " + ImageFileName$ + " skipped, unsupported file type !" + #CRLF$
      Case #I2PDF_LOG_JPG_STORING        
         Text$ + "[ERR] " + ImageFileName$ + " skipped, unable to store this JPG file !" + #CRLF$
   EndSelect
   
   WriteString(s_tLogfile\nNumFile, Text$)
EndProcedure


Procedure LogAddProcessed (ImageFileName$, iFileSize, iFileSizeInsidePdf)
   Shared s_tLogfile
   Protected Text$
   
   If IsFile(s_tLogfile\nNumFile) = 0 : ProcedureReturn : EndIf
   
   s_tLogfile\iTotFiles + 1
   s_tLogfile\qTotBytesProcessed + iFileSize
   s_tLogfile\iCurrPage + 1
   
   Text$ + "[OK]" + " Page " + FormatQuad(s_tLogfile\iCurrPage) + ": " + ImageFileName$
   Text$ + " (" + FormatDouble((iFileSizeInsidePdf/iFileSize) * 100, 1) + " %)"  + #CRLF$
   
   WriteString(s_tLogfile\nNumFile, Text$)
EndProcedure


Procedure LogClose ()
   Shared s_tLogfile
   Protected Text$
   Protected iElapsed
   Protected fProcessingSpeed.f, fShrinked.f
   
   If IsFile(s_tLogfile\nNumFile) = 0 : ProcedureReturn : EndIf
   
   s_tLogfile\iEndTime = Date() 
   
   iElapsed = s_tLogfile\iEndTime - s_tLogfile\iStartTime 
   
   iElapsed + 1 ; against DIV by zero
   
   If s_tLogfile\qTotBytesProcessed
      fProcessingSpeed = ((s_tLogfile\qTotBytesProcessed * 1.0) / (iElapsed * 1.0)) / #MEGABYTE ; MB/sec
      fShrinked = (FileSize(s_tLogfile\PdfFileName$) / s_tLogfile\qTotBytesProcessed) * 100
   Else
      fProcessingSpeed = 0
      fShrinked = 0
   EndIf
   
   iElapsed - 1 ; against DIV by zero
   
   Text$ + #CRLF$
   Text$ + "Start time        : " + FormatDate(GetSystemDateFormat(), s_tLogfile\iStartTime) + " " + FormatDate("%hh:%ii:%ss", s_tLogfile\iStartTime) + #CRLF$
   Text$ + "End time          : " + FormatDate(GetSystemDateFormat(), s_tLogfile\iEndTime) + " " + FormatDate("%hh:%ii:%ss", s_tLogfile\iEndTime) + #CRLF$
   
   If iElapsed
      Text$ + "Elapsed time      : " + FormatDate("%hh:%ii:%ss", iElapsed) + #CRLF$
   Else
      Text$ + "Elapsed time      : Less than a second." + #CRLF$ 
   EndIf
   
   Text$ + #CRLF$
   If (s_tLogfile\iTotFiles = 0)
      Text$ + "Output file       : " + "Not available." + #CRLF$
   Else
      Text$ + "Output file       : " + s_tLogfile\PdfFileName$ + " (" + FormatQuad(FileSize(s_tLogfile\PdfFileName$)) + " bytes)" + #CRLF$
   EndIf
   
   If fShrinked <= 100.0
      Text$ + "Shrinked to       : " 
   Else
      Text$ + "Enlarged to       : " 
   EndIf
   Text$ + FormatDouble(fShrinked, 1) + " %" + #CRLF$
   
   If iElapsed 
      Text$ + "Speed             : " + FormatDouble(fProcessingSpeed, 2) + " MB/sec" +  #CRLF$
   EndIf
   
   Text$ + #CRLF$
   Text$ + "Files processed   : " + FormatQuad(s_tLogfile\iTotFiles) + " (" + FormatQuad(s_tLogfile\qTotBytesProcessed) + " bytes)" +  #CRLF$
   Text$ + "Files skipped     : " + FormatQuad(s_tLogfile\iFailedFiles) + #CRLF$
   Text$ + "Files missing     : " + FormatQuad(s_tLogfile\iMissingFiles) + #CRLF$ 
   
   Text$ + #CRLF$
   
   If s_tLogfile\iFailedFiles + s_tLogfile\iMissingFiles
      Text$ + "!!! COMPLETED WITH ERRORS !!!" + #CRLF$
   Else
      Text$ + "NO ERRORS" + #CRLF$
   EndIf
   
   WriteString(s_tLogfile\nNumFile, Text$)
   
   CloseFile(s_tLogfile\nNumFile)
EndProcedure


Procedure LogDelete()
   Shared s_tLogfile
   
   DeleteFile(s_tLogfile\LogFileName$)
EndProcedure


Procedure.s LogGetFileName()
   Shared s_tLogfile
   
   ProcedureReturn s_tLogfile\LogFileName$
EndProcedure

; IDE Options = PureBasic 5.73 LTS (Windows - x64)
; Folding = --
; EnableXP
; EnableUser
; CPU = 1
; CompileSourceDirectory
; EnableExeConstant