EnableExplicit

; PDF light 1.11 - LMG Software
; Luis, April 2009

; The font spacing data in the datasection of this include is taken from the PurePDF library by ABBKlaus. Thank you.

IncludeFile "LinkedList.pb"

Enumeration 
   #PDF_OK = 0
   #PDF_ERR_OUT_OF_MEMORY 
   #PDF_ERR_INVALID_PARAMETERS 
   #PDF_ERR_FILE_CREATION
   #PDF_ERR_FILE_IO
   #PDF_ERR_UNSUPPORTED_PNG
EndEnumeration

#PDF_LIB_DESCRIPTION    = "PDF Light 1.11 by LMG Software"

#PDF_HEADER_VERSION     = "%PDF-1.5"

#SPLF$ = Chr(32) + Chr(10)

#PDF_DEFAULT            = -1
#PDF_DEFAULT$           = ""

#PDF_PORTRAIT           = 1
#PDF_LANDSCAPE          = 2

#PDF_ZOOM_FULLPAGE      = 1
#PDF_ZOOM_FULLWIDTH     = 2
#PDF_ZOOM_REAL          = 3
#PDF_ZOOM_FULLSCREEN    = 4 

#PDF_LAYOUT_SINGLE      = 1 ; /SinglePage
#PDF_LAYOUT_CONTINUOUS  = 2 ; /OneColumn

#PDF_UNITS_PT           = 1
#PDF_UNITS_MM           = 2
#PDF_UNITS_INCH         = 3

; sizes in 72 DPI
#PDF_PAGE_FORMAT_A2     = "1190.55,1683.78"  ; 420 x 594 mm
#PDF_PAGE_FORMAT_A3     = "841.89,1190.55"   ; 297 x 420 mm
#PDF_PAGE_FORMAT_A4     = "595.28,841.89"    ; 210 x 297 mm
#PDF_PAGE_FORMAT_LEGAL  = "612.00,1008.00"   ; 216 x 356 mm
#PDF_PAGE_FORMAT_LETTER = "612.00,792.00"    ; 216 x 279 mm    

#PDF_PAGE_FORMAT_DEFAULT = #PDF_PAGE_FORMAT_A4

#PDF_BPP_24             = 1
#PDF_BPP_8              = 2
#PDF_BPP_8_GREY         = 3

#PDF_IMAGE_JPG          = 1
#PDF_IMAGE_PNG          = 2

#PDF_ALIGN_LEFT         = 1
#PDF_ALIGN_CENTER       = 2
#PDF_ALIGN_RIGHT        = 3

#PDF_FONT_COURIER       = "Courier"     ; monospaced
#PDF_FONT_HELVETICA     = "Helvetica"   ; proportional


;*****************************************************************************
; PDF DOCUMENT OBJECTS - MULTIPLE INSTANCES - HEAP ALLOCATED
;*****************************************************************************

Structure T_PDF_OBJ_PALETTE 
   iObjNum.i              ; object number
   iDataOffset.i          ; offset in the file
   iDataLen.i             ; length of the databuffer for this section (before + binary + after)
   DataBufferBefore$      ; pdf statements for the object (the part before the stream section)
   DataBufferAfter$       ; pdf statements for the object (the part after the stream section)
   iStreamLen.i           ; length of the stream section 
   iEntries.i             ; number of palette entries
   *Binary                ; memory buffer for the binary palette data
EndStructure

Structure T_PDF_OBJ_IMAGE_STREAM Extends T_LIST_ITEM
   
   ; define a binary image stream, it will be referred to through an alias 
   ; defined in the resource obj
   
   ; pointed by   : T_PDF_OBJ_RESOURCES                   /I(n) (obj) 0 R
   
   iObjNum.i              ; object number
   iDataOffset.i          ; offset in the file
   iDataLen.i             ; length of the databuffer for this section (before + binary + after)
   DataBufferBefore$      ; pdf statements for the object (the part before the stream section)
   DataBufferAfter$       ; pdf statements for the object (the part after the stream section)
    
   fWidth.f               ; always in pixels
   fHeight.f              ; always in pixels
   ImageName$             ; unique name (with path if needed) used to identify an image in memory
   iImageType.i           ; #PDF_IMAGE_JPG, #PDF_IMAGE_PNG
   iImageBPP.i            ; 24 for JPG, 8/24 for PNG
   iImageAlias.i          ; alias id
   iStreamLen.i           ; length of the stream section 
   *Binary                ; memory buffer for the binary image 
   *Palette.T_PDF_OBJ_PALETTE ; point to palette data for PNG indexed 8 bit images (#Null for 24 bit)
   
EndStructure

Structure T_PDF_OBJ_CONTENTS_STREAM Extends T_LIST_ITEM
   
   ; define the actual contents of the page, using aliases to refer to fonts 
   ; and to image streams
   
   ; pointed by   : T_PDF_OBJ_PAGE                        /Contents
   
   ; point to     : T_PDF_OBJ_RESOURCES                   /I(n) /F(n)
   
   iObjNum.i      ; object number
   iDataOffset.i  ; offset in the file
   iDataLen.i     ; length of the databuffer for this section 
   DataBuffer$    ; pdf statements for the object
   
   iStreamLen.i   ; length of the stream section 
   ImageBuffer$   ; pdf statements related to the image only
   FontBuffer$    ; pdf statements related to the font only
   TextBuffer$    ; pdf statements related to the text only    
   
EndStructure

Structure T_PDF_OBJ_PAGE Extends T_LIST_ITEM
   
   ; points to the page list and to the contents of this page
   
   ; pointed by   : T_PDF_OBJ_PAGES                       /Kids [ ] 
   
   ; point to     : T_PDF_OBJ_PAGES                       /Parent
   ; point to     : T_PDF_OBJ_RESOURCES                   /Resources
   ; point to     : T_PDF_OBJ_CONTENTS_STREAM (multiple)  /Contents
   
   iObjNum.i      ; object number
   iDataOffset.i  ; offset in the file
   iDataLen.i     ; length of the databuffer for this section 
   DataBuffer$    ; pdf statements for the object
   
   fWidth.f       ; always in pixels
   fHeight.f      ; always in pixels    
   
   *ToContents.T_PDF_OBJ_CONTENTS_STREAM ; point to a contents struct in memory
   
EndStructure


;*****************************************************************************
; PDF DOCUMENT OBJECTS - SINGLE INSTANCE
;*****************************************************************************

Structure T_PDF_OBJ_FONT  
   
   ; define a font, it will be referred to through an alias defined in
   ; the resource obj
   
   ; pointed by   : T_PDF_OBJ_RESOURCES                   /F(n) (n) 0 R
   
   iObjNum.i      ; object number
   iDataOffset.i  ; offset in the file
   iDataLen.i     ; length of the databuffer for this section 
   DataBuffer$    ; pdf statements for the object
   
   FamilyName$    ; #PDF_FONT_HELVETICA, ... etc
   iFontSize.i    ; font size 8, 10, 12, ... etc
   iFontAlias.i   ; alias id
   
   CharWidth.i [256] ; preloaded with char widths for the font
   
EndStructure

Structure T_PDF_OBJ_RESOURCES
   
   ; list the resource used by the various pages (images, fonts)
   
   ; pointed by   : T_PDF_OBJ_PAGE                        /Resources
   
   ; point to     : T_PDF_OBJ_FONT                        /F(n) (n) 0 R
   ; point to     : T_PDF_OBJ_IMAGE_STREAM (multiple)     /I(n) (n) 0 R
   
   iObjNum.i      ; object number
   iDataOffset.i  ; offset in the file
   iDataLen.i     ; length of the databuffer for this section 
   DataBuffer$    ; pdf statements for the object
   
EndStructure

Structure T_PDF_OBJ_PAGES 
   
   ; contains a list of all the kid pages composing the document
   
   ; point to     : T_PDF_OBJ_PAGE (multiple)             /Kids [ ] 
   
   iObjNum.i      ; object number
   iDataOffset.i  ; offset in the file
   iDataLen.i     ; length of the databuffer for this section 
   DataBuffer$    ; pdf statements for the object
   
   *PageList.T_PDF_OBJ_PAGE ; point to a list of pages in memory
   
EndStructure

Structure T_PDF_OBJ_INFO 
   
   ; the info section for the PDF document
   
   ; pointed by   : T_PDF_OBJ_TRAILER                     /Root
   
   iObjNum.i      ; object number
   iDataOffset.i  ; offset in the file
   iDataLen.i     ; length of the databuffer for this section 
   DataBuffer$    ; pdf statements for the object
   
   Producer$      ; software library used
   Title$         ; title of the document
   Author$        ; Author of the document
   Creator$       ; program used to generate the document     
   
EndStructure

Structure T_PDF_OBJ_CATALOG 
   
   ; the catalog dictionary for the PDF document
   
   ; pointed by   : T_PDF_OBJ_TRAILER                     /Root
   
   ; point to     : T_PDF_OBJ_PAGES                       /Pages
   
   iObjNum.i      ; object number
   iDataOffset.i  ; offset in the file
   iDataLen.i     ; length of the databuffer for this section 
   DataBuffer$    ; pdf statements for the object
   
EndStructure

;*****************************************************************************
; PDF DOCUMENT SPECIAL SECTIONS
;*****************************************************************************

Structure T_PDF_XREF Extends T_LIST_ITEM
   iObjNum.i      ; object number (the object pointed)
   iXrefOffs.i    ; offset in the file (to the start of the object pointed)   
EndStructure

Structure T_PDF_OBJ_XREF 
   iDataOffset.i  ; offset in the file
   iDataLen.i     ; length of the databuffer for this section 
   DataBuffer$    ; pdf statements for this special section
   *XrefList.T_PDF_XREF ; point to a list of xref items in memory
EndStructure

Structure T_PDF_OBJ_TRAILER
   iDataLen.i     ; length of the databuffer for this section 
   DataBuffer$    ; pdf statements for this special section
EndStructure



;*****************************************************************************
; PDF DOCUMENT MAIN OBJECT
;*****************************************************************************

Structure T_PDF_DOCUMENT
   
   iObjCount.i
   iAliasImagesCount.i
   
   fDefPageWidth.f   
   fDefPageHeight.f
   
   fCurrX.f
   fCurrY.f
   
   iUnits.i
   fUnitsK.f
   
   iPageZoom.i
   iPageLayout.i
   iPageOrientation.i
   
   tXref.T_PDF_OBJ_XREF
   tTrailer.T_PDF_OBJ_TRAILER
   tCatalog.T_PDF_OBJ_CATALOG
   tInfo.T_PDF_OBJ_INFO
   tPages.T_PDF_OBJ_PAGES
   tResources.T_PDF_OBJ_RESOURCES   
   tFont.T_PDF_OBJ_FONT
   
   *CurrPage.T_PDF_OBJ_PAGE  
   *ImageList.T_PDF_OBJ_IMAGE_STREAM      
   *ContentList.T_PDF_OBJ_CONTENTS_STREAM 
   
   iLastError.i
   LastError$
   iLineError.i
   
   UserComments$
EndStructure


DataSection 
   
   ;{ FONTS
   
   Helvetica:
   Data.i  278, 278, 278, 278, 278, 278, 278, 278, 278, 278, 278, 278, 278, 278, 278, 278, 278, 278
   Data.i  278, 278, 278, 278, 278, 278, 278, 278, 278, 278, 278, 278, 278, 278, 278, 278, 355, 556
   Data.i  556, 889, 667, 191, 333, 333, 389, 584, 278, 333, 278, 278, 556, 556, 556, 556, 556, 556
   Data.i  556, 556, 556, 556, 278, 278, 584, 584, 584, 556,1015, 667, 667, 722, 722, 667, 611, 778
   Data.i  722, 278, 500, 667, 556, 833, 722, 778, 667, 778, 722, 667, 611, 722, 667, 944, 667, 667
   Data.i  611, 278, 278, 278, 469, 556, 333, 556, 556, 500, 556, 556, 278, 556, 556, 222, 222, 500
   Data.i  222, 833, 556, 556, 556, 556, 333, 500, 278, 556, 500, 722, 500, 500, 500, 334, 260, 334
   Data.i  584, 350, 556, 350, 222, 556, 333,1000, 556, 556, 333,1000, 667, 333,1000, 350, 611, 350
   Data.i  350, 222, 222, 333, 333, 350, 556,1000, 333,1000, 500, 333, 944, 350, 500, 667, 278, 333
   Data.i  556, 556, 556, 556, 260, 556, 333, 737, 370, 556, 584, 333, 737, 333, 400, 584, 333, 333
   Data.i  333, 556, 537, 278, 333, 333, 365, 556, 834, 834, 834, 611, 667, 667, 667, 667, 667, 667
   Data.i 1000, 722, 667, 667, 667, 667, 278, 278, 278, 278, 722, 722, 778, 778, 778, 778, 778, 584
   Data.i  778, 722, 722, 722, 722, 667, 667, 611, 556, 556, 556, 556, 556, 556, 889, 500, 556, 556
   Data.i  556, 556, 278, 278, 278, 278, 556, 556, 556, 556, 556, 556, 556, 584, 611, 556, 556, 556
   Data.i  556, 500, 556, 500
   
   ;}
   
EndDataSection


;{ DECLARES

Declare 	ipf_FreePage (*item.T_PDF_OBJ_PAGE)
Declare 	ipf_FreeContent (*item.T_PDF_OBJ_CONTENTS_STREAM)
Declare 	ipf_FreeImage (*item.T_PDF_OBJ_IMAGE_STREAM)
Declare 	ipf_SortXref(*a.T_PDF_XREF, *b.T_PDF_XREF)
Declare.i 	ipf_ValidateOrientation (iOrientation)
Declare.i 	ipf_ValidateUnits (iUnits)
Declare.i 	ipf_ValidateZoom (iZoom)
Declare.i 	ipf_ValidateLayout (iLayout)
Declare 	ipf_SetUnits (iUnits)
Declare.s 	ipf_Escape (Buffer$)
Declare.i 	ipf_SetError (iErrCode, ProcName$, iLineNum = 0)
Declare.i 	ipf_NextObjID()
Declare.i 	ipf_NextImageAlias()
Declare.i 	ipf_WriteBuffer (nFileNum, *MemAddr, iMemSize)
Declare.i 	ipf_OpenFile (FileName$)
Declare 	ipf_CloseFile (nFileNum)
Declare 	ipf_Initialize()
Declare.s 	ipf_StartDoc()
Declare.s 	ipf_EndDoc()
Declare.i 	ipf_SavePdfToFile (nFileNum)
Declare.i 	ipf_LinkDataStructure()
Declare.f 	ipf_GetStringWidth (Text$)
Declare.i 	ipf_IsImageLoaded (ImageName$)
Declare.i 	ipf_Read32BIT (*mem)
Declare.i 	ipf_CalcRawDataSizePNG (*Binary, *iTotDataSize.Integer)
Declare.i 	ipf_ParsePNG (*Binary, iTotDataSize)

Declare.i 	pdf_GetErrorCode()
Declare.s 	pdf_GetErrorMessage()
Declare.i 	pdf_GetErrorLine()
Declare 	pdf_ClearError()
Declare.s 	pdf_MakeCustomFormatString (fWidth.f, fHeight.f, iOrientation = #PDF_PORTRAIT)
Declare.i 	pdf_SetFont (FamilyName$, iSize)
Declare 	pdf_SetXY (fX.f, fY.f)
Declare 	pdf_Cell (fCellWidth.f, fCellHeight.f, Text$, flgBorder = 0, iAlignment = #PDF_ALIGN_LEFT)
Declare.i 	pdf_Save (FileName$)
Declare 	pdf_End()
Declare.f 	pdf_MMtoPT (fMillimeters.f)
Declare.f 	pdf_PTtoMM (fPoints)
Declare 	pdf_PaperSizeToPT (PaperSize$, *fWidthPT.float, *fHeightPT.float)
Declare 	pdf_PaperSizeToMM (PaperSize$, *fWidthMM.float, *fHeightMM.float)
Declare 	pdf_Init()
Declare 	pdf_SetAuthor (Author$)
Declare 	pdf_SetTitle (Title$)
Declare 	pdf_SetCreator (Creator$)
Declare 	pdf_SetUserComments (UserComments$)
Declare 	pdf_SetDisplayMode (iZoom, iLayout)
Declare.i 	pdf_Create (iOrientation = #PDF_DEFAULT, PaperSize$ = #PDF_DEFAULT$, iUnits = #PDF_DEFAULT)
Declare.i 	pdf_ImageFromMem (ImageName$, iImageType, iImageBPP, *Binary, iSize, fWidth.f, fHeight.f, fX.f, fY.f, fDestWidth.f, fDestHeight.f)
Declare.f 	pdf_GetPageWidth()
Declare.f 	pdf_GetPageHeight()
Declare.i 	pdf_AddPage (iOrientation = #PDF_DEFAULT, PaperSize$ = #PDF_DEFAULT$)

;}


;{ MACROS

Macro IPF_CHECK_RET (retval, label)
   If retval : s_tPDF\iLineError = #PB_Compiler_Line : Goto label : EndIf
EndMacro

Macro IPF_CHECK_MEM (ptr, label)
   If ptr = 0 : s_tPDF\iLineError = #PB_Compiler_Line : Goto label : EndIf
EndMacro

Macro IPF_CHECK_HND (handle, label)
   If handle = 0: s_tPDF\iLineError = #PB_Compiler_Line : Goto label : EndIf
EndMacro

Macro IPF_2PIX (val)
   ((val) * s_tPDF\fUnitsK)
EndMacro

Macro IPF_2UNI (val)
   ((val) / s_tPDF\fUnitsK)
EndMacro

;}

; The following variable is shared only in procedures in this file,
; but not in procedures in other files.
Define s_tPDF.T_PDF_DOCUMENT

Procedure ipf_FreePage (*item.T_PDF_OBJ_PAGE)
   FreeString(@*item\DataBuffer$) 
EndProcedure

Procedure ipf_FreeContent (*item.T_PDF_OBJ_CONTENTS_STREAM)
   FreeString(@*item\DataBuffer$)
   
   FreeString(@*item\ImageBuffer$)
   FreeString(@*item\FontBuffer$)
   FreeString(@*item\TextBuffer$)
EndProcedure

Procedure ipf_FreeImage (*item.T_PDF_OBJ_IMAGE_STREAM)
   
   FreeString(@*item\DataBufferBefore$)
   FreeString(@*item\DataBufferAfter$)
   
   FreeString(@*item\ImageName$)
   
   If *item\Binary
      FreeMemory(*item\Binary) : *item\Binary = 0
   EndIf
   
   If *item\Palette
      FreeString(@*item\Palette\DataBufferBefore$)
      FreeString(@*item\Palette\DataBufferAfter$)
      
      FreeMemory(*item\Palette) : *item\Palette = 0
   EndIf
EndProcedure


Procedure ipf_SortXref (*a.T_PDF_XREF, *b.T_PDF_XREF)
   ; ascending by obj number
   
   If *a\iObjNum <= *b\iObjNum ; if iObjNum a < iObjNum b is ok 
      ProcedureReturn #True 
   EndIf
   
   ProcedureReturn #False ; this pair need to be swapped
   
EndProcedure

Procedure.i ipf_ValidateOrientation (iOrientation)
   Shared s_tPDF
   
   Select iOrientation             
      Case #PDF_PORTRAIT, #PDF_LANDSCAPE
         ; no changes required
      Default
         ; if #PDF_DEFAULT or invalid value
         ; we get the default stored in s_tPDF
         iOrientation = s_tPDF\iPageOrientation
   EndSelect
   
   ProcedureReturn iOrientation
   
EndProcedure

Procedure.i ipf_ValidateUnits (iUnits)
   Shared s_tPDF
   
   Select iUnits
      Case #PDF_UNITS_PT, #PDF_UNITS_MM, #PDF_UNITS_INCH
         ; no changes required
      Default:
         ; if #PDF_DEFAULT or invalid value
         ; we get the default stored in s_tPDF    
         iUnits = s_tPDF\iUnits
   EndSelect   
   
   ProcedureReturn iUnits 
EndProcedure

Procedure.i ipf_ValidateZoom (iZoom)
   Shared s_tPDF
   
   Select iZoom
      Case #PDF_ZOOM_FULLPAGE, #PDF_ZOOM_FULLWIDTH, #PDF_ZOOM_REAL, #PDF_ZOOM_FULLSCREEN
         ; no changes required
      Default
         ; if #PDF_DEFAULT or invalid value
         ; we get the default stored in s_tPDF        
         iZoom = s_tPDF\iPageZoom 
   EndSelect
   
   ProcedureReturn iZoom
   
EndProcedure

Procedure.i ipf_ValidateLayout (iLayout)
   Shared s_tPDF
   
   Select iLayout
      Case #PDF_LAYOUT_CONTINUOUS, #PDF_LAYOUT_SINGLE
         ; no changes required
      Default
         ; if #PDF_DEFAULT or invalid value
         ; we get the default stored in s_tPDF        
         iLayout = s_tPDF\iPageLayout
   EndSelect
   
   ProcedureReturn iLayout
   
EndProcedure

Procedure ipf_SetUnits (iUnits)
   Shared s_tPDF
   
   Select iUnits
      Case #PDF_UNITS_PT
         s_tPDF\iUnits = #PDF_UNITS_PT
         s_tPDF\fUnitsK = 1
      Case #PDF_UNITS_MM
         s_tPDF\iUnits = #PDF_UNITS_MM
         s_tPDF\fUnitsK = 72/25.4
      Case #PDF_UNITS_INCH    
         s_tPDF\iUnits = #PDF_UNITS_INCH
         s_tPDF\fUnitsK = 72
   EndSelect   
   
EndProcedure

Procedure.s ipf_Escape (Buffer$) 
   Protected RetVal$
   
   RetVal$ = ReplaceString(Buffer$, "\", "\\")
   RetVal$ = ReplaceString(RetVal$, "(", "\(") 
   RetVal$ = ReplaceString(RetVal$, ")", "\)")
   
   ProcedureReturn RetVal$ 
EndProcedure

Procedure.i ipf_SetError (iErrCode, ProcName$, iLineNum = 0)  
   Shared s_tPDF
   Protected ErrMsg$
   
   s_tPDF\iLastError = iErrCode
   
   If iLineNum
      s_tPDF\iLineError = iLineNum
   EndIf
   
   Select iErrCode
      Case #PDF_ERR_FILE_CREATION : ErrMsg$ = "File creation error"
      Case #PDF_ERR_FILE_IO : ErrMsg$ = "File I/O error"
      Case #PDF_ERR_INVALID_PARAMETERS : ErrMsg$ = "Invalid parameters"
      Case #PDF_ERR_OUT_OF_MEMORY : ErrMsg$ = "Out of memory"
      Case #PDF_ERR_FILE_CREATION : ErrMsg$ = "Error creating file"
      Case #PDF_ERR_UNSUPPORTED_PNG : ErrMsg$ = "Unsupported PNG header or data corrupted"
      Default 
         ErrMsg$ = "Unknown error"    
   EndSelect
   
   s_tPDF\LastError$ = ErrMsg$ + #CRLF$ + #CRLF$
   s_tPDF\LastError$ + "in " + ProcName$ + "()" + " on line " + Str(s_tPDF\iLineError) + "." + #CRLF$
   
   ProcedureReturn iErrCode
EndProcedure

Procedure.i ipf_NextObjID()
   Shared s_tPDF
   
   s_tPDF\iObjCount + 1
   ProcedureReturn s_tPDF\iObjCount 
EndProcedure

Procedure.i ipf_NextImageAlias()
   Shared s_tPDF
   
   s_tPDF\iAliasImagesCount + 1
   ProcedureReturn s_tPDF\iAliasImagesCount
EndProcedure

Procedure.i ipf_WriteBufferASCII (nFileNum, Buffer$, iMemSize) 
   Protected iRetCode
   
   Protected *ASCII = Ascii(Buffer$) ; Fred removed support for ASCII strings in PB 5.50
   iRetCode = ipf_WriteBuffer(nFileNum, *ASCII, iMemSize) 
   FreeMemory(*ASCII)
   
   ProcedureReturn iRetCode 
EndProcedure

Procedure.i ipf_WriteBuffer (nFileNum, *MemAddr, iMemSize) 
   Protected iCurrLen
   
   If IsFile(nFileNum)
      iCurrLen = Lof(nFileNum)
      WriteData (nFileNum , *MemAddr, iMemSize)                    
      FlushFileBuffers(nFileNum)
   Else
      ProcedureReturn ipf_SetError (#PDF_ERR_INVALID_PARAMETERS , #PB_Compiler_Procedure, #PB_Compiler_Line)    
   EndIf  
   
   If Lof(nFileNum) - iCurrLen <> iMemSize
      ProcedureReturn ipf_SetError (#PDF_ERR_FILE_IO, #PB_Compiler_Procedure, #PB_Compiler_Line)    
   EndIf
   
   ProcedureReturn #PDF_OK
EndProcedure

Procedure.i ipf_OpenFile (FileName$)  
   ProcedureReturn CreateFile (#PB_Any, FileName$) 
EndProcedure

Procedure ipf_CloseFile (nFileNum)
   If IsFile(nFileNum)
      CloseFile(nFileNum)
   EndIf
EndProcedure

Procedure ipf_Initialize()
   Shared s_tPDF
   
   With s_tPDF
      \iObjCount = 0
      \iAliasImagesCount = 0
      
      ; in the PDF structure the page size is stored always using the portrait orientation
      ; and always in points (pixels)
      \fDefPageWidth = ValF(StringField(#PDF_PAGE_FORMAT_DEFAULT, 1, ","))
      \fDefPageHeight = ValF(StringField(#PDF_PAGE_FORMAT_DEFAULT, 2, ","))    
      
      \fCurrX = 0
      \fCurrY = 0
      
      ; default to MM
      ipf_SetUnits (#PDF_UNITS_MM)
      
      \iPageZoom = #PDF_ZOOM_FULLPAGE
      \iPageLayout = #PDF_LAYOUT_CONTINUOUS
      
      ; this will tell us if we must swap page sizes when creating an actual page
      \iPageOrientation = #PDF_PORTRAIT
      
      \CurrPage = #Null
      
      \iLastError = 0
      \LastError$ = ""
      \iLineError = 0
      
      \UserComments$ = ""
      
   EndWith
   
   With s_tPDF\tXref
      \iDataOffset = 0
      \iDataLen = 0
      \DataBuffer$ = ""
      
      LinkedList_Free(\XrefList): \XrefList = 0
   EndWith
   
   With s_tPDF\tTrailer
      \iDataLen = 0
      \DataBuffer$ = ""
   EndWith
   
   With s_tPDF\tCatalog
      \iObjNum = 0
      \iDataOffset = 0
      \iDataLen = 0
      \DataBuffer$ = ""
   EndWith
   
   With s_tPDF\tInfo
      \iObjNum = 0
      \iDataOffset = 0
      \iDataLen = 0
      \DataBuffer$ = ""
      \Producer$ = #PDF_LIB_DESCRIPTION
      \Title$ = ""
      \Author$ = ""
      \Creator$ = ""    
   EndWith
   
   With s_tPDF\tPages
      \iObjNum = 0
      \iDataOffset = 0
      \iDataLen = 0
      \DataBuffer$ = ""
      
      LinkedList_Free(\PageList) : \PageList = 0
   EndWith
   
   With s_tPDF\tResources
      \iObjNum = 0
      \iDataOffset = 0
      \iDataLen = 0 
      \DataBuffer$ = ""
   EndWith
   
   With s_tPDF\tFont
      \iObjNum = 0
      \iDataOffset = 0
      \iDataLen = 0
      \DataBuffer$ = ""    
      \iFontSize = 0
      \iFontAlias = 0 
      \FamilyName$ = ""    
   EndWith
   
   LinkedList_Free(s_tPDF\ImageList) : s_tPDF\ImageList = 0
   LinkedList_Free(s_tPDF\ContentList) : s_tPDF\ContentList = 0
   
EndProcedure

Procedure.s ipf_StartDoc()
   ProcedureReturn #PDF_HEADER_VERSION + #LF$ + "%" + #LF$
EndProcedure

Procedure.s ipf_EndDoc()
   Shared s_tPDF
   Protected Buffer$   
   
   Buffer$ = "startxref" + #LF$
   Buffer$ + Str(s_tPDF\tXref\iDataOffset) + #LF$
   Buffer$ + "%%EOF"
   
   ProcedureReturn Buffer$
EndProcedure

Procedure.i ipf_SavePdfToFile (nFileNum)
   Shared s_tPDF
   Protected iRetVal, Buffer$
   Protected *page.T_PDF_OBJ_PAGE, *contents.T_PDF_OBJ_CONTENTS_STREAM, *image.T_PDF_OBJ_IMAGE_STREAM, *palette.T_PDF_OBJ_PALETTE
   Protected *xref.T_PDF_XREF
   
   ; pdf version header
   
   Buffer$ = ipf_StartDoc() 
   
   iRetVal = ipf_WriteBufferASCII (nFileNum, Buffer$, Len(Buffer$))
   
   IPF_CHECK_RET (iRetVal, ipf_SavePdfToFile_ERR)
   
   If Len(s_tPDF\UserComments$)
      
      Buffer$ = s_tPDF\UserComments$
      
      iRetVal = ipf_WriteBufferASCII (nFileNum, Buffer$, Len(Buffer$))
      
      IPF_CHECK_RET (iRetVal, ipf_SavePdfToFile_ERR)
      
   EndIf
   
   ; all pages obj
   
   *page = LinkedList_First(s_tPDF\tPages\PageList)
   
   While *page                           
      iRetVal = ipf_WriteBufferASCII (nFileNum, *page\DataBuffer$, *page\iDataLen)
      
      IPF_CHECK_RET (iRetVal, ipf_SavePdfToFile_ERR)
      
      *page = LinkedList_Next(s_tPDF\tPages\PageList) 
   Wend
   
   
   ; contents obj for each page
   
   *contents = LinkedList_First(s_tPDF\ContentList)
   
   While *contents                
      
      iRetVal = ipf_WriteBufferASCII (nFileNum, *contents\DataBuffer$, *contents\iDataLen)
      
      IPF_CHECK_RET (iRetVal, ipf_SavePdfToFile_ERR)
      
      *contents = LinkedList_Next(s_tPDF\ContentList)
   Wend
   
   
   ; pages obj
   
   iRetVal = ipf_WriteBufferASCII (nFileNum, s_tPDF\tPages\DataBuffer$, s_tPDF\tPages\iDataLen)
   
   IPF_CHECK_RET (iRetVal, ipf_SavePdfToFile_ERR)
   
   
   ; resources obj
   
   iRetVal = ipf_WriteBufferASCII (nFileNum, s_tPDF\tResources\DataBuffer$, s_tPDF\tResources\iDataLen)
   
   IPF_CHECK_RET (iRetVal, ipf_SavePdfToFile_ERR)
   
   
   ; font obj
   
   If s_tPDF\tFont\iObjNum     
      iRetVal = ipf_WriteBufferASCII (nFileNum, s_tPDF\tFont\DataBuffer$, s_tPDF\tFont\iDataLen)
      
      IPF_CHECK_RET (iRetVal, ipf_SavePdfToFile_ERR)
   EndIf
   
   
   ; images obj
   
   *image = LinkedList_First(s_tPDF\ImageList)
   
   While *image     
      iRetVal = ipf_WriteBufferASCII (nFileNum, *image\DataBufferBefore$, Len(*image\DataBufferBefore$))
      iRetVal = ipf_WriteBuffer (nFileNum, *image\Binary, *image\iStreamLen)
      iRetVal = ipf_WriteBufferASCII (nFileNum, *image\DataBufferAfter$, Len(*image\DataBufferAfter$))
      
      ; if one of the above fails, the last one fails too
      IPF_CHECK_RET (iRetVal, ipf_SavePdfToFile_ERR)
      
      ; free memory as soon as possible 
      FreeMemory(*image\Binary) : *image\Binary = 0
      
      If *image\iImageType = #PDF_IMAGE_PNG And *image\Palette > 0
         *palette = *image\Palette 
         
         iRetVal = ipf_WriteBufferASCII (nFileNum, *palette\DataBufferBefore$, Len(*palette\DataBufferBefore$))
         iRetVal = ipf_WriteBuffer (nFileNum, *palette\Binary, *palette\iStreamLen)
         iRetVal = ipf_WriteBufferASCII (nFileNum, *palette\DataBufferAfter$, Len(*palette\DataBufferAfter$))
         
         ; if one of the above fails, the last one fails too
         IPF_CHECK_RET (iRetVal, ipf_SavePdfToFile_ERR)
         
         ; free memory as soon as possible 
         FreeMemory(*palette\Binary) : *palette\Binary = 0
      EndIf
      
      *image = LinkedList_Next(s_tPDF\ImageList)
   Wend
   
   
   ; info obj
   
   iRetVal = ipf_WriteBufferASCII (nFileNum, s_tPDF\tInfo\DataBuffer$, s_tPDF\tInfo\iDataLen)
   
   IPF_CHECK_RET (iRetVal, ipf_SavePdfToFile_ERR)
   
   
   ; catalog obj
   
   iRetVal = ipf_WriteBufferASCII (nFileNum, s_tPDF\tCatalog\DataBuffer$, s_tPDF\tCatalog\iDataLen)
   
   IPF_CHECK_RET (iRetVal, ipf_SavePdfToFile_ERR)
   
   
   ; xref 
   
   iRetVal = ipf_WriteBufferASCII (nFileNum, s_tPDF\tXref\DataBuffer$, s_tPDF\tXref\iDataLen)
   
   IPF_CHECK_RET (iRetVal, ipf_SavePdfToFile_ERR)
   
   
   ; trailer
   
   iRetVal = ipf_WriteBufferASCII (nFileNum, s_tPDF\tTrailer\DataBuffer$, s_tPDF\tTrailer\iDataLen)
   
   IPF_CHECK_RET (iRetVal, ipf_SavePdfToFile_ERR)
   
   
   ; end doc 
   
   Buffer$ = ipf_EndDoc() 
   
   iRetVal = ipf_WriteBufferASCII (nFileNum, Buffer$, Len(Buffer$))
   
   IPF_CHECK_RET (iRetVal, ipf_SavePdfToFile_ERR)
   
   ProcedureReturn #PDF_OK
   
   
   ipf_SavePdfToFile_ERR:
   
   ProcedureReturn ipf_SetError (iRetVal, #PB_Compiler_Procedure)
   
EndProcedure

Procedure.i ipf_LinkDataStructure()
   Shared s_tPDF
   Protected iOffset, iFirstPageObj
   Protected Buffer$, Temp$
   Protected flgDST, iMinutesOffset
   Protected *page.T_PDF_OBJ_PAGE, *contents.T_PDF_OBJ_CONTENTS_STREAM, *image.T_PDF_OBJ_IMAGE_STREAM, *palette.T_PDF_OBJ_PALETTE 
   Protected *xref.T_PDF_XREF
   
   
   ; pdf version header
   
   iOffset + Len(ipf_StartDoc())
   
   iOffset + Len(s_tPDF\UserComments$)
   
   
   ; all pages
   
   If LinkedList_Count(s_tPDF\tPages\PageList)
      
      *page = LinkedList_First(s_tPDF\tPages\PageList)
      
      iFirstPageObj = *page\iObjNum
      
      While *page        
         
         *page\iDataOffset = iOffset
         
         *xref = LinkedList_Add(s_tPDF\tXref\XrefList)
         
         IPF_CHECK_MEM (*xref, ipf_LinkDataStructure_MEM)
         
         *xref\iObjNum = *page\iObjNum
         *xref\iXrefOffs = *page\iDataOffset
         
         
         Buffer$ = Str(*page\iObjNum) + " 0 obj" + #LF$
         Buffer$ + "<<" + #LF$
         Buffer$ + "/Type /Page" + #LF$
         Buffer$ + "/Parent " + Str(s_tPDF\tPages\iObjNum) + " 0 R" + #LF$
         Buffer$ + "/MediaBox [0 0 " + StrF(*page\fWidth, 2) + " " + StrF(*page\fHeight, 2) + "]" + #LF$
         Buffer$ + "/Resources " + Str(s_tPDF\tResources\iObjNum) + " 0 R" + #LF$
         Buffer$ + "/Contents " + Str(*page\ToContents\iObjNum) + " 0 R" 
         Buffer$ + ">>" + #LF$
         Buffer$ + "endobj" + #LF$
         
         *page\DataBuffer$ = Buffer$        
         *page\iDataLen = Len(Buffer$)        
         iOffset + *page\iDataLen 
         
         *page = LinkedList_Next(s_tPDF\tPages\PageList)
      Wend
      
      
      ; contents for each page
      
      If LinkedList_Count(s_tPDF\ContentList)
         
         *contents = LinkedList_First(s_tPDF\ContentList)
         
         While *contents                
            
            *contents\iDataOffset = iOffset
            
            
            *xref = LinkedList_Add(s_tPDF\tXref\XrefList)
            
            IPF_CHECK_MEM (*xref, ipf_LinkDataStructure_MEM)
            
            *xref\iObjNum = *contents\iObjNum
            *xref\iXrefOffs = *contents\iDataOffset
            
            
            Temp$ = ""                
            
            If Len(*contents\ImageBuffer$)
               Temp$ + *contents\ImageBuffer$ + #LF$
            EndIf
            
            If Len(*contents\FontBuffer$)
               Temp$ + *contents\FontBuffer$ + #LF$
            EndIf
            
            If Len(*contents\TextBuffer$)
               Temp$ + *contents\TextBuffer$ + #LF$
            EndIf
            
            *contents\iStreamLen = Len(Temp$)
            
            
            Buffer$ = Str(*contents\iObjNum) + " 0 obj" + #LF$
            Buffer$ + "<</Length " + Str(*contents\iStreamLen) + ">>" + #LF$
            Buffer$ + "stream" + #LF$
            
            Buffer$ + Temp$ + #LF$
            
            Buffer$ + "endstream" + #LF$
            Buffer$ + "endobj" + #LF$
            
            *contents\DataBuffer$ = Buffer$        
            *contents\iDataLen = Len(Buffer$)
            iOffset + *contents\iDataLen 
            
            *contents = LinkedList_Next(s_tPDF\ContentList)
         Wend
      EndIf
      
   EndIf
   
   
   ; pages obj
   
   s_tPDF\tPages\iDataOffset = iOffset
   
   
   *xref = LinkedList_Add(s_tPDF\tXref\XrefList)
   
   IPF_CHECK_MEM (*xref, ipf_LinkDataStructure_MEM) 
   
   *xref\iObjNum = s_tPDF\tPages\iObjNum
   *xref\iXrefOffs = s_tPDF\tPages\iDataOffset
   
   
   Buffer$ = Str(s_tPDF\tPages\iObjNum) + " 0 obj" + #LF$
   Buffer$ + "<<" + #LF$
   Buffer$ + "/Type /Pages" + #LF$
   Buffer$ + "/Kids [ "
   
   *page = LinkedList_First(s_tPDF\tPages\PageList)
   
   While *page                       
      Buffer$ + Str(*page\iObjNum) + " 0 R "
      *page = LinkedList_Next(s_tPDF\tPages\PageList)
   Wend
   
   Buffer$ + "]" + #LF$
   Buffer$ + "/Count " + Str(LinkedList_Count(s_tPDF\tPages\PageList)) + #LF$
   Buffer$ + ">>" + #LF$
   Buffer$ + "endobj" + #LF$
   
   s_tPDF\tPages\DataBuffer$ = Buffer$
   s_tPDF\tPages\iDataLen = Len(Buffer$)
   iOffset + s_tPDF\tPages\iDataLen 
   
   
   ;resources 
   
   s_tPDF\tResources\iDataOffset = iOffset
   
   *xref = LinkedList_Add(s_tPDF\tXref\XrefList)
   
   IPF_CHECK_MEM (*xref, ipf_LinkDataStructure_MEM)
   
   *xref\iObjNum = s_tPDF\tResources\iObjNum
   *xref\iXrefOffs = s_tPDF\tResources\iDataOffset
   
   Buffer$ = Str(s_tPDF\tResources\iObjNum) + " 0 obj" + #LF$
   Buffer$ + "<<" + #LF$
   Buffer$ + "/ProcSet [/PDF /Text /ImageB /ImageC /ImageI]" + #LF$
   
   If s_tPDF\tFont\iObjNum
      Buffer$ + "/Font <<" + #LF$
      Buffer$ + "/F" + Str(s_tPDF\tFont\iFontAlias) + " " + Str(s_tPDF\tFont\iObjNum) + " 0 R" + #LF$
      Buffer$ + ">>" + #LF$
   EndIf
   
   Buffer$ + "/XObject <<" + #LF$
   
   *image = LinkedList_First(s_tPDF\ImageList)
   
   While *image
      Buffer$ + "/I" + Str(*image\iImageAlias) + " " + Str(*image\iObjNum) + " 0 R" + #LF$
      *image = LinkedList_Next(s_tPDF\ImageList)
   Wend
   
   Buffer$ + ">>" + #LF$
   Buffer$ + ">>" + #LF$
   Buffer$ + "endobj" + #LF$
   
   s_tPDF\tResources\DataBuffer$ = Buffer$
   s_tPDF\tResources\iDataLen = Len(Buffer$)
   iOffset + s_tPDF\tResources\iDataLen 
   
   
   ; font
   
   If s_tPDF\tFont\iObjNum
      s_tPDF\tFont\iDataOffset = iOffset
      
      *xref = LinkedList_Add(s_tPDF\tXref\XrefList)
      
      IPF_CHECK_MEM (*xref, ipf_LinkDataStructure_MEM)
      
      *xref\iObjNum = s_tPDF\tFont\iObjNum
      *xref\iXrefOffs = s_tPDF\tFont\iDataOffset
      
      
      Buffer$ = Str(s_tPDF\tFont\iObjNum) + " 0 obj" + #LF$
      Buffer$ + "<<" + #LF$
      Buffer$ + "/Type /Font" + #LF$
      Buffer$ + "/BaseFont /" + s_tPDF\tFont\FamilyName$ + #LF$
      Buffer$ + "/Subtype /Type1"+ #LF$
      Buffer$ + "/Encoding /WinAnsiEncoding" + #LF$
      Buffer$ + ">>" + #LF$
      Buffer$ + "endobj" + #LF$
      
      s_tPDF\tFont\DataBuffer$ = Buffer$
      s_tPDF\tFont\iDataLen = Len(Buffer$)
      iOffset + s_tPDF\tFont\iDataLen
   EndIf
   
   
   ; images
   
   *image = LinkedList_First(s_tPDF\ImageList)
   
   While *image 
      *palette = 0
      
      *image\iDataOffset = iOffset
      *image\iStreamLen = MemorySize(*image\Binary)
      
      *xref = LinkedList_Add(s_tPDF\tXref\XrefList)
      
      IPF_CHECK_MEM (*xref, ipf_LinkDataStructure_MEM)
      
      *xref\iObjNum = *image\iObjNum
      *xref\iXrefOffs = *image\iDataOffset
      
      
      Buffer$ = Str(*image\iObjNum) + " 0 obj" + #LF$
      Buffer$ + "<<" + #LF$
      Buffer$ + "/Type /XObject" + #LF$
      Buffer$ + "/Subtype /Image" + #LF$
      Buffer$ + "/Width " + Str(*image\fWidth) + #LF$
      Buffer$ + "/Height " + Str(*image\fHeight) + #LF$
      
      
      Select *image\iImageType 
            
         Case #PDF_IMAGE_JPG
            Buffer$ + "/ColorSpace /DeviceRGB" + #LF$      
            Buffer$ + "/BitsPerComponent 8" + #LF$
            Buffer$ + "/Filter /DCTDecode" + #LF$
            
         Case #PDF_IMAGE_PNG
            Select *image\iImageBPP
                  
               Case #PDF_BPP_24
                  Buffer$ + "/ColorSpace /DeviceRGB" + #LF$  
                  Buffer$ + "/BitsPerComponent 8" + #LF$
                  Buffer$ + "/Filter /FlateDecode" + #LF$ 
                  Buffer$ + "/DecodeParms <</Predictor 15 /Colors 3 /BitsPerComponent 8 /Columns " + Str(*image\fWidth) + ">>" + #LF$
                  
               Case #PDF_BPP_8
                  *palette = *image\Palette
                  
                  Buffer$ + "/ColorSpace [/Indexed /DeviceRGB " + Str(*palette\iEntries) + " " + Str(*palette\iObjNum) + " 0 R]" + #LF$
                  Buffer$ + "/BitsPerComponent 8" + #LF$
                  Buffer$ + "/Filter /FlateDecode" + #LF$ 
                  Buffer$ + "/DecodeParms <</Predictor 15 /Colors 1 /BitsPerComponent 8 /Columns " + Str(*image\fWidth) + ">>" + #LF$
                  
               Case #PDF_BPP_8_GREY ; not tested but should work
                  Buffer$ + "/ColorSpace /DeviceGray" + #LF$
                  Buffer$ + "/BitsPerComponent 8" + #LF$
                  Buffer$ + "/Filter /FlateDecode" + #LF$ 
                  Buffer$ + "/DecodeParms <</Predictor 15 /Colors 1 /BitsPerComponent 8 /Columns " + Str(*image\fWidth) + ">>" + #LF$                    
            EndSelect
            
      EndSelect
      
      Buffer$ + "/Length " + Str(*image\iStreamLen) + ">>" + #LF$
      Buffer$ + "stream" + #LF$
      
      *image\DataBufferBefore$ = Buffer$
      
      Buffer$ = #LF$
      Buffer$ + "endstream" + #LF$
      Buffer$ + "endobj" + #LF$     
      
      *image\DataBufferAfter$ = Buffer$
      
      *image\iDataLen = Len(*image\DataBufferBefore$) + *image\iStreamLen + Len(*image\DataBufferAfter$)
      iOffset + *image\iDataLen 
      
      If *image\iImageType = #PDF_IMAGE_PNG And *palette > 0
         
         *palette\iDataOffset = iOffset
         *palette\iStreamLen = MemorySize(*palette\Binary)
         
         *xref = LinkedList_Add(s_tPDF\tXref\XrefList)
         
         IPF_CHECK_MEM (*xref, ipf_LinkDataStructure_MEM)
         
         *xref\iObjNum = *palette\iObjNum
         *xref\iXrefOffs = *palette\iDataOffset
         
         Buffer$ = Str(*palette\iObjNum) + " 0 obj" + #LF$
         Buffer$ + "<</Length " + Str(*palette\iStreamLen) + ">>" + #LF$
         Buffer$ + "stream" + #LF$
         
         *palette\DataBufferBefore$ = Buffer$
         
         Buffer$ = #LF$
         Buffer$ + "endstream" + #LF$
         Buffer$ + "endobj" + #LF$     
         
         *palette\DataBufferAfter$ = Buffer$
         
         *palette\iDataLen = Len(*palette\DataBufferBefore$) + *palette\iStreamLen + Len(*palette\DataBufferAfter$)
         iOffset + *palette\iDataLen         
      EndIf
      
      *image = LinkedList_Next(s_tPDF\ImageList)
   Wend
   
   
   ; info
   
   s_tPDF\tInfo\iDataOffset = iOffset
   
   
   *xref = LinkedList_Add(s_tPDF\tXref\XrefList)
   
   IPF_CHECK_MEM (*xref, ipf_LinkDataStructure_MEM)
   
   *xref\iObjNum = s_tPDF\tInfo\iObjNum
   *xref\iXrefOffs = s_tPDF\tInfo\iDataOffset
   
   
   Buffer$ = Str(s_tPDF\tInfo\iObjNum) + " 0 obj" + #LF$
   Buffer$ + "<<" + #LF$
   Buffer$ + "/Producer (" + ipf_Escape(s_tPDF\tInfo\Producer$) + ")" + #LF$
   
   If Len(s_tPDF\tInfo\Title$)
      Buffer$ + "/Title (" + ipf_Escape(s_tPDF\tInfo\Title$) + ")" + #LF$
   EndIf
   
   If Len(s_tPDF\tInfo\Author$)
      Buffer$ + "/Author (" + ipf_Escape(s_tPDF\tInfo\Author$) + ")" + #LF$
   EndIf    
   
   If Len(s_tPDF\tInfo\Creator$)
      Buffer$ + "/Creator (" + ipf_Escape(s_tPDF\tInfo\Creator$) + ")" + #LF$
   EndIf
   
   Buffer$ + "/CreationDate (" + "D:" + FormatDate("%yyyy%mm%dd%hh%ii%ss", Date()) 
   
   If GetSystemTimeZone(@flgDST, @iMinutesOffset)
      iMinutesOffset = - iMinutesOffset  
      
      Buffer$ + IIFs(Bool(iMinutesOffset > 0), "+", "-")
      
      Buffer$ + RSet(Str(iMinutesOffset / 60), 2, "0")
      Buffer$ + "'"
      Buffer$ + RSet(Str(iMinutesOffset % 60), 2, "0")
   EndIf
   
   Buffer$ + ")" + #LF$
   
   
   Buffer$ + ">>" + #LF$
   Buffer$ + "endobj" + #LF$
   
   s_tPDF\tInfo\DataBuffer$ = Buffer$
   s_tPDF\tInfo\iDataLen = Len(Buffer$)
   iOffset + s_tPDF\tInfo\iDataLen
   
   
   ; catalog
   
   s_tPDF\tCatalog\iDataOffset = iOffset
   
   
   *xref = LinkedList_Add(s_tPDF\tXref\XrefList)
   
   IPF_CHECK_MEM (*xref, ipf_LinkDataStructure_MEM)
   
   *xref\iObjNum = s_tPDF\tCatalog\iObjNum
   *xref\iXrefOffs = s_tPDF\tCatalog\iDataOffset
   
   
   Buffer$ = Str(s_tPDF\tCatalog\iObjNum) + " 0 obj" + #LF$
   Buffer$ + "<<" + #LF$
   Buffer$ + "/Type /Catalog" + #LF$
   Buffer$ + "/Pages " + Str(s_tPDF\tPages\iObjNum) + " 0 R" + #LF$
   
   Select s_tPDF\iPageZoom
      Case #PDF_ZOOM_FULLPAGE
         Buffer$ + "/OpenAction [" + Str(iFirstPageObj) + " 0 R /Fit]" + #LF$
      Case #PDF_ZOOM_FULLSCREEN
         Buffer$ + "/PageMode /FullScreen" + #LF$
      Case #PDF_ZOOM_FULLWIDTH
         Buffer$ + "/OpenAction [" + Str(iFirstPageObj) + " 0 R /FitH null]" + #LF$
      Case #PDF_ZOOM_REAL
         Buffer$ + "/OpenAction [" + Str(iFirstPageObj) + " 0 R /XYZ null null 1]" + #LF$
   EndSelect
   
   Select s_tPDF\iPageLayout
      Case #PDF_LAYOUT_CONTINUOUS
         Buffer$ + "/PageLayout /OneColumn" + #LF$
      Case #PDF_LAYOUT_SINGLE
         Buffer$ + "/PageLayout /SinglePage" + #LF$ 
   EndSelect
   
   Buffer$ + ">>" + #LF$
   Buffer$ + "endobj" + #LF$
   
   s_tPDF\tCatalog\DataBuffer$ = Buffer$ 
   s_tPDF\tCatalog\iDataLen = Len(Buffer$)
   iOffset + s_tPDF\tCatalog\iDataLen
   
   ; xref
   
   s_tPDF\tXref\iDataOffset = iOffset
   
   LinkedList_Sort(s_tPDF\tXref\XrefList, @ipf_SortXref())
   
   Buffer$ = "xref" + #LF$
   Buffer$ + "0 " + Str(LinkedList_Count(s_tPDF\tXref\XrefList) + 1) + #LF$
   Buffer$ + RSet("", 10, "0") + " " + "65535 f" + #SPLF$
   
   *xref = LinkedList_First(s_tPDF\tXref\XrefList)
   
   While *xref
      Buffer$ + RSet(Str(*xref\iXrefOffs), 10, "0") + " " + "00000 n" + #SPLF$
      
      *xref = LinkedList_Next(s_tPDF\tXref\XrefList)
   Wend
   
   s_tPDF\tXref\DataBuffer$ = Buffer$ 
   s_tPDF\tXref\iDataLen = Len(Buffer$)
   
   ; trailer
   
   Buffer$ = "trailer" + #LF$
   Buffer$ + "<<" + #LF$
   Buffer$ + "/Size " + Str(LinkedList_Count(s_tPDF\tXref\XrefList) + 1) + #LF$
   Buffer$ + "/Root " + Str(s_tPDF\tCatalog\iObjNum) + " 0 R" + #LF$
   Buffer$ + "/Info " + Str(s_tPDF\tInfo\iObjNum) + " 0 R" + #LF$
   Buffer$ + ">>" + #LF$  
   
   s_tPDF\tTrailer\DataBuffer$ = Buffer$ 
   s_tPDF\tTrailer\iDataLen = Len(Buffer$)
   
   ProcedureReturn #PDF_OK
   
   
   ipf_LinkDataStructure_MEM:
   
   ProcedureReturn ipf_SetError (#PDF_ERR_OUT_OF_MEMORY, #PB_Compiler_Procedure)
   
EndProcedure

Procedure.f ipf_GetStringWidth (Text$)
   Shared s_tPDF
   Protected iLen, k
   Protected fRetVal.f
   
   iLen = Len(Text$)
   
   For k = 1 To iLen
      fRetVal + s_tPDF\tFont\CharWidth [Asc(Mid(Text$,k,1))]   
   Next
   
   ProcedureReturn (fRetVal / 1000) * (s_tPDF\tFont\iFontSize  / s_tPDF\fUnitsK)
EndProcedure

Procedure.i ipf_IsImageLoaded (ImageName$)
   Shared s_tPDF
   Protected *image.T_PDF_OBJ_IMAGE_STREAM
   
   *image = LinkedList_First(s_tPDF\ImageList)
   
   While *image
      If *image\ImageName$ = ImageName$
         ProcedureReturn *image
      EndIf
      *image = LinkedList_Next(s_tPDF\ImageList)
   Wend
   
   ProcedureReturn 0
EndProcedure

Procedure.i ipf_Read32BIT (*mem)
   Protected iResult
   
   iResult = (PeekB(*mem    ) & $FF) << 24
   iResult + (PeekB(*mem + 1) & $FF) << 16
   iResult + (PeekB(*mem + 2) & $FF) << 8
   iResult + (PeekB(*mem + 3) & $FF) 
   
   ProcedureReturn iResult
EndProcedure

Procedure.i ipf_CalcRawDataSizePNG (*Binary, *iTotDataSize.Integer)
   
   Protected iDataSize, iCount, iTotDataSize
   Protected ChunkType$, flgIEND
   
   flgIEND = #False
   
   iTotDataSize = 0 : iDataSize = 0: iCount = 33
   
   Repeat      
      iDataSize = ipf_Read32BIT (*Binary + iCount)
      
      iCount + 4
      
      ChunkType$ = PeekS(*Binary + iCount, 4, #PB_Ascii)
      
      Select ChunkType$
         Case "IDAT"                                   
            iTotDataSize + iDataSize            
         Case "IEND"
            flgIEND = #True
            *iTotDataSize\i = iTotDataSize
            Break
      EndSelect    
      
      iCount + iDataSize + 8
      
   Until (iDataSize <= 0)  
   
   If flgIEND
      ProcedureReturn #PDF_OK
   EndIf
   
   ProcedureReturn ipf_SetError (#PDF_ERR_UNSUPPORTED_PNG, #PB_Compiler_Procedure, #PB_Compiler_Line)
   
EndProcedure



Procedure.i ipf_ParsePNG_Palette (*Binary)
   
   Protected iDataSize, iCount
   Protected ChunkType$
   Protected *ptr
   
   iDataSize = 0: iCount = 33
   
   Repeat      
      iDataSize = ipf_Read32BIT (*Binary + iCount)
      
      iCount + 4
      
      ChunkType$ = PeekS(*Binary + iCount, 4, #PB_Ascii)
      
      Select ChunkType$
            
         Case "PLTE"                           
            *ptr = AllocateMemory (iDataSize)
            CopyMemory(*Binary + iCount + 4, *ptr, iDataSize)                                
            
            Debug "PLTE"
         Case "IEND"                 
            Break
      EndSelect    
      
      iCount + iDataSize + 8
      
   Until (iDataSize <= 0)      
   
   ProcedureReturn *ptr 
EndProcedure

Procedure.i ipf_ParsePNG (*Binary, iTotDataSize)
   
   Protected iDataSize, iCount
   Protected ChunkType$
   Protected *ptr, iPngDataOffs
   
   *ptr = AllocateMemory (iTotDataSize)
   
   If *ptr 
      
      iPngDataOffs = 0 : iDataSize = 0: iCount = 33
      
      Repeat      
         iDataSize = ipf_Read32BIT (*Binary + iCount)
         
         iCount + 4
         
         ChunkType$ = PeekS(*Binary + iCount, 4, #PB_Ascii)
         
         Select ChunkType$
            Case "IDAT"           
               CopyMemory(*Binary + iCount + 4, *ptr + iPngDataOffs, iDataSize)
               iPngDataOffs + iDataSize
            Case "IEND"                 
               Break
         EndSelect    
         
         iCount + iDataSize + 8
         
      Until (iDataSize <= 0)      
      
   EndIf    
   
   ProcedureReturn *ptr
   
EndProcedure


Procedure.i pdf_GetErrorCode()
   Shared s_tPDF
   
   ProcedureReturn s_tPDF\iLastError
EndProcedure

Procedure.s pdf_GetErrorMessage()
   Shared s_tPDF
   
   ProcedureReturn s_tPDF\LastError$
EndProcedure

Procedure.i pdf_GetErrorLine()
   Shared s_tPDF
   
   ProcedureReturn s_tPDF\iLineError
EndProcedure

Procedure pdf_ClearError()
   Shared s_tPDF
   
   s_tPDF\iLastError = #PDF_OK
   s_tPDF\iLineError = 0
   s_tPDF\LastError$ = ""
EndProcedure


Procedure.s pdf_MakeCustomFormatString (fWidth.f, fHeight.f, iOrientation = #PDF_PORTRAIT)
   If iOrientation = #PDF_PORTRAIT
      ProcedureReturn StrF(fWidth, 2) + "," + StrF(fHeight, 2)
   Else
      ProcedureReturn StrF(fHeight, 2) + "," + StrF(fWidth, 2)
   EndIf
EndProcedure

Procedure.i pdf_SetFont (FamilyName$, iSize)
   Shared s_tPDF
   Protected flgMonospaced, iCharWidth, k
   
   s_tPDF\tFont\iObjNum = ipf_NextObjID()
   s_tPDF\tFont\FamilyName$ = FamilyName$
   s_tPDF\tFont\iFontSize = iSize
   s_tPDF\tFont\iFontAlias = 1 ; one font only at the moment
   
   Select UCase (FamilyName$)
      Case "COURIER"
         flgMonospaced = #True
      Case "HELVETICA"
         flgMonospaced = #False
         Restore Helvetica          
      Default
         ProcedureReturn ipf_SetError (#PDF_ERR_INVALID_PARAMETERS , #PB_Compiler_Procedure, #PB_Compiler_Line)        
   EndSelect
   
   Select flgMonospaced 
      Case #True
         For k = 0 To 255  
            s_tPDF\tFont\CharWidth[k] = 600
         Next      
      Case #False
         For k = 0 To 255
            Read.i iCharWidth        
            s_tPDF\tFont\CharWidth[k] = iCharWidth
         Next
   EndSelect
   
   ProcedureReturn #PDF_OK
   
EndProcedure

Procedure pdf_SetXY (fX.f, fY.f)
   Shared s_tPDF
   
   s_tPDF\fCurrX = IPF_2PIX(fX) ; convert to pixels
   s_tPDF\fCurrY = IPF_2PIX(fY) ; convert to pixels
EndProcedure

Procedure pdf_Cell (fCellWidth.f, fCellHeight.f, Text$, flgBorder = 0, iAlignment = #PDF_ALIGN_LEFT)
   Shared s_tPDF
   Protected fTextLen.f
   Protected Buffer$  
   Protected fPageWidth.f, fPageHeight.f, fCurrX.f, fCurrY.f
   Protected fPageMargin = 28.35 / s_tPDF\fUnitsK / 2; (5 MM)
   Protected fCellMargin = fPageMargin / 10
   Protected fDX.f
   
   fPageWidth = IPF_2UNI (s_tPDF\CurrPage\fWidth)
   fPageHeight = IPF_2UNI (s_tPDF\CurrPage\fHeight)
   fCurrX = IPF_2UNI (s_tPDF\fCurrX)
   fCurrY = IPF_2UNI (s_tPDF\fCurrY)
   
   If flgBorder
      Buffer$ = StrF(IPF_2PIX (fCurrX), 2) + " " + StrF(IPF_2PIX (fPageHeight - fCurrY), 2) + " " + StrF(IPF_2PIX (fCellWidth), 2) + " " + StrF(IPF_2PIX (-fCellHeight), 2) + " re S "
   EndIf
   
   If Len(Text$) 
      fTextLen = ipf_GetStringWidth (Text$)
      
      If (iAlignment = #PDF_ALIGN_RIGHT)
         fDX = fCellWidth - fPageMargin - fTextLen
      ElseIf (iAlignment = #PDF_ALIGN_CENTER)  
         fDX = (fCellWidth - fTextLen) / 2
      Else
         fDX = fPageMargin 
      EndIf  
      
      Text$ = ipf_Escape(Text$) 
      
      Buffer$ + "BT " + StrF(IPF_2PIX (fCurrX + fDX), 2) + " " + StrF(IPF_2PIX (fPageHeight - (fCurrY + 0.5 * fCellHeight + 0.3 * IPF_2UNI (s_tPDF\tFont\iFontSize))), 2) + " Td (" + Text$ + ") Tj ET"
   EndIf 
   
   ; update the appropriate contents buffer for the current page
   
   s_tPDF\CurrPage\ToContents\TextBuffer$ = Buffer$
   
   ; update the appropriate contents buffer for the current page
   
   Buffer$ = "BT /F" + Str(s_tPDF\tFont\iFontAlias) + " " + StrF(s_tPDF\tFont\iFontSize, 2) + " Tf ET"
   
   s_tPDF\CurrPage\ToContents\FontBuffer$ = Buffer$
   
EndProcedure

Procedure.i pdf_Save (FileName$)
   Shared s_tPDF
   Protected nFileNum, iRetVal
   
   nFileNum = ipf_OpenFile (FileName$)
   
   IPF_CHECK_HND (nFileNum, pdf_Save_HND)
   
   
   iRetVal = ipf_LinkDataStructure()
   
   IPF_CHECK_RET (iRetVal, pdf_Save_ERR)
   
   
   iRetVal = ipf_SavePdfToFile(nFileNum)
   
   ipf_CloseFile(nFileNum)
   
   IPF_CHECK_RET (iRetVal, pdf_Save_ERR)
   
   
   ProcedureReturn #PDF_OK
   
   
   pdf_Save_HND: 
   
   ProcedureReturn ipf_SetError (#PDF_ERR_FILE_CREATION, #PB_Compiler_Procedure)
   
   
   pdf_Save_ERR: 
   
   ProcedureReturn ipf_SetError (iRetVal, #PB_Compiler_Procedure, #PB_Compiler_Line)
   
EndProcedure

Procedure pdf_End()
   ipf_Initialize()
EndProcedure

Procedure.f pdf_MMtoPT (fMillimeters.f)
   ProcedureReturn fMillimeters * (72.0 / 25.4)
EndProcedure

Procedure.f pdf_PTtoMM (fPoints)
   ProcedureReturn fPoints / (72.0 / 25.4)
EndProcedure

Procedure pdf_PaperSizeToPT (PaperSize$, *fWidthPT.float, *fHeightPT.float)
   *fWidthPT\f = ValF(StringField(PaperSize$, 1, ","))
   *fHeightPT\f = ValF(StringField(PaperSize$, 2, ","))
EndProcedure

Procedure pdf_PaperSizeToMM (PaperSize$, *fWidthMM.float, *fHeightMM.float)
   *fWidthMM\f = pdf_PTtoMM (ValF(StringField(PaperSize$, 1, ",")))
   *fHeightMM\f = pdf_PTtoMM (ValF(StringField(PaperSize$, 2, ",")))
EndProcedure

Procedure pdf_Init()
   ipf_Initialize() 
EndProcedure

Procedure pdf_SetAuthor (Author$) 
   Shared s_tPDF
   
   s_tPDF\tInfo\Author$ = Author$
EndProcedure

Procedure pdf_SetTitle (Title$)
   Shared s_tPDF
   
   s_tPDF\tInfo\Title$ = Title$ 
EndProcedure

Procedure pdf_SetCreator (Creator$)  
   Shared s_tPDF
   
   s_tPDF\tInfo\Creator$ = Creator$ 
EndProcedure 

Procedure pdf_SetUserComments (UserComments$)
   Shared s_tPDF
   
   s_tPDF\UserComments$ = UserComments$
EndProcedure

Procedure pdf_SetDisplayMode (iZoom, iLayout)
   Shared s_tPDF
   
   iZoom = ipf_ValidateZoom (iZoom)
   
   iLayout = ipf_ValidateLayout (iLayout)
   s_tPDF\iPageZoom = iZoom 
   
   s_tPDF\iPageLayout = iLayout 
   
EndProcedure

Procedure.i pdf_Create (iOrientation = #PDF_DEFAULT, PaperSize$ = #PDF_DEFAULT$, iUnits = #PDF_DEFAULT) 
   Shared s_tPDF
   
   s_tPDF\iPageOrientation = ipf_ValidateOrientation(iOrientation)
   
   ipf_SetUnits (ipf_ValidateUnits(iUnits))
   
   
   If PaperSize$ <> #PDF_DEFAULT$    
      ; in the PDF structure the page size is stored always using the portrait orientation
      s_tPDF\fDefPageWidth = ValF(StringField(PaperSize$, 1, ","))
      
      s_tPDF\fDefPageHeight = ValF(StringField(PaperSize$, 2, ","))
   EndIf
   
   
   ; init xref list 
   
   s_tPDF\tXref\XrefList = LinkedList_New(SizeOf(T_PDF_XREF), 0)
   
   IPF_CHECK_MEM (s_tPDF\tXref\XrefList, pdf_Create_MEM)
   
   ; init the pages single object
   
   s_tPDF\tPages\iObjNum = ipf_NextObjID()
   
   ; and its page list
   
   s_tPDF\tPages\PageList = LinkedList_New(SizeOf(T_PDF_OBJ_PAGE), @ipf_FreePage())
   
   IPF_CHECK_MEM (s_tPDF\tPages\PageList, pdf_Create_MEM) 
   
   ; init single catalog object
   
   s_tPDF\tCatalog\iObjNum = ipf_NextObjID()
   
   ; init single info object
   
   s_tPDF\tInfo\iObjNum = ipf_NextObjID()
   
   ; init single resource object
   
   s_tPDF\tResources\iObjNum = ipf_NextObjID()
   
   
   ; init contents list
   
   s_tPDF\ContentList = LinkedList_New(SizeOf(T_PDF_OBJ_CONTENTS_STREAM), @ipf_FreeContent())
   
   IPF_CHECK_MEM (s_tPDF\ContentList, pdf_Create_MEM)  
   
   ; init images list
   
   s_tPDF\ImageList = LinkedList_New(SizeOf(T_PDF_OBJ_IMAGE_STREAM), @ipf_FreeImage())
   
   IPF_CHECK_MEM (s_tPDF\ImageList, pdf_Create_MEM)
   
   ProcedureReturn #PDF_OK
   
   
   pdf_Create_MEM:
   
   ProcedureReturn ipf_SetError (#PDF_ERR_OUT_OF_MEMORY, #PB_Compiler_Procedure)
   
EndProcedure 

Procedure.i pdf_ImageFromMem (ImageName$, iImageType, iImageBPP, *Binary, iSize,  fWidth.f, fHeight.f, fX.f, fY.f, fDestWidth.f, fDestHeight.f)
   Shared s_tPDF
   Protected *image.T_PDF_OBJ_IMAGE_STREAM
   Protected Buffer$, iTotDataSize, i
   Protected fPageHeight.f
   
   fPageHeight = IPF_2UNI(s_tPDF\CurrPage\fHeight) ; from pixels to current units
   
   If (iImageType <> #PDF_IMAGE_JPG) And (iImageType <> #PDF_IMAGE_PNG)
      ProcedureReturn ipf_SetError (#PDF_ERR_INVALID_PARAMETERS , #PB_Compiler_Procedure, #PB_Compiler_Line)
   EndIf
   
   Select iImageBPP 
      Case #PDF_BPP_24, #PDF_BPP_8, #PDF_BPP_8_GREY
         ; OK
      Default
         ProcedureReturn ipf_SetError (#PDF_ERR_INVALID_PARAMETERS , #PB_Compiler_Procedure, #PB_Compiler_Line)
   EndSelect
   
   If (iImageType = #PDF_IMAGE_JPG) And (iImageBPP <> #PDF_BPP_24)
      ProcedureReturn ipf_SetError (#PDF_ERR_INVALID_PARAMETERS , #PB_Compiler_Procedure, #PB_Compiler_Line)
   EndIf
   
   
   ; check if the image is already loaded
   
   *image = ipf_IsImageLoaded (ImageName$)
   
   If *image = 0
      ; go to last item, if any 
      LinkedList_Last(s_tPDF\ImageList)
      
      ; allocate the image object
      *image = LinkedList_Add(s_tPDF\ImageList)
      
      IPF_CHECK_MEM (*image, pdf_ImageFromMem_MEM)
      
      *image\iObjNum = ipf_NextObjID() 
      *image\ImageName$ = ImageName$
      
      *image\fWidth = IPF_2PIX(fWidth)     ; from units to pixels
      *image\fHeight = IPF_2PIX(fHeight)   ; from units to pixels
      
      *image\iImageType = iImageType
      *image\iImageBPP = iImageBPP
      
      *image\iImageAlias = ipf_NextImageAlias()
      
      Select iImageType
            
         Case #PDF_IMAGE_JPG 
            *image\Binary = AllocateMemory(iSize)
            
            IPF_CHECK_MEM (*image\Binary, pdf_ImageFromMem_MEM)
            
            CopyMemory(*Binary, *image\Binary, iSize)
            
         Case #PDF_IMAGE_PNG
            
            i = ipf_CalcRawDataSizePNG (*Binary, @iTotDataSize)
            
            If i <> 0
               ProcedureReturn ipf_SetError (i, #PB_Compiler_Procedure, #PB_Compiler_Line) 
            EndIf
            
            *image\Binary = ipf_ParsePNG (*Binary, iTotDataSize)
            
            IPF_CHECK_MEM (*image\Binary, pdf_ImageFromMem_MEM)
            
            If *image\iImageBPP = #PDF_BPP_8 ; 8 BPP, no greyscale
               *image\Palette = AllocateMemory(SizeOf(T_PDF_OBJ_PALETTE))
               IPF_CHECK_MEM (*image\Palette, pdf_ImageFromMem_MEM)
               
               *image\Palette\Binary = ipf_ParsePNG_Palette (*Binary)                
               IPF_CHECK_MEM (*image\Palette\Binary, pdf_ImageFromMem_MEM)
               
               *image\Palette\iObjNum = ipf_NextObjID()
               *image\Palette\iEntries = MemorySize(*image\Palette\Binary) / 3 - 1
            EndIf                       
      EndSelect
      
   EndIf
   
   Buffer$ = "q " + StrF(IPF_2PIX(fDestWidth), 2) + " 0 0 " + StrF(IPF_2PIX(fDestHeight), 2) + " " + StrF(IPF_2PIX(fX), 2) + " "
   Buffer$ + StrF(IPF_2PIX(fPageHeight - (fY + fDestHeight)), 2) + " cm /I" + Str(*image\iImageAlias) + " Do Q"
   
   s_tPDF\CurrPage\ToContents\ImageBuffer$ = Buffer$ 
   
   ProcedureReturn #PDF_OK
   
   
   pdf_ImageFromMem_MEM:
   
   ProcedureReturn ipf_SetError (#PDF_ERR_OUT_OF_MEMORY, #PB_Compiler_Procedure)
   
EndProcedure

Procedure.f pdf_GetPageWidth()
   Shared s_tPDF
   Protected *page.T_PDF_OBJ_PAGE
   
   *page = s_tPDF\CurrPage
   
   If *page
      ProcedureReturn *page\fWidth        ; in pixel
   Else
      ProcedureReturn s_tPDF\fDefPageWidth  ; in pixel
   EndIf
   
EndProcedure

Procedure.f pdf_GetPageHeight()
   Shared s_tPDF
   Protected *page.T_PDF_OBJ_PAGE
   
   *page = s_tPDF\CurrPage
   
   If *page
      ProcedureReturn *page\fHeight       ; in pixel
   Else
      ProcedureReturn s_tPDF\fDefPageHeight ; in pixel
   EndIf
   
EndProcedure

Procedure.i pdf_AddPage (iOrientation = #PDF_DEFAULT, PaperSize$ = #PDF_DEFAULT$)
   Shared s_tPDF
   Protected fPageWidth.f, fPageHeight.f
   Protected *page.T_PDF_OBJ_PAGE
   Protected *contents.T_PDF_OBJ_CONTENTS_STREAM
   
   iOrientation = ipf_ValidateOrientation (iOrientation)
   
   ; page size is always expressed in points (pixels)
   
   If PaperSize$ = #PDF_DEFAULT$    
      fPageWidth = s_tPDF\fDefPageWidth  
      fPageHeight = s_tPDF\fDefPageHeight
   Else
      fPageWidth = ValF(StringField(PaperSize$, 1, ","))
      fPageHeight = ValF(StringField(PaperSize$, 2, ","))
   EndIf
   
   ; in the page the sizes are oriented as needed and are the real, actual page sizes
   If iOrientation = #PDF_LANDSCAPE
      Swap fPageWidth, fPageHeight
   EndIf
   
   ; go to last item, if any
   LinkedList_Last(s_tPDF\tPages\PageList)
   
   ; allocate the page
   *page = LinkedList_Add(s_tPDF\tPages\PageList)
   
   IPF_CHECK_MEM (*page, pdf_AddPage_MEM)
   
   s_tPDF\CurrPage = *page
   
   *page\iObjNum = ipf_NextObjID() 
   *page\fWidth = fPageWidth
   *page\fHeight = fPageHeight
   
   ; go to last item, if any
   LinkedList_Last(s_tPDF\ContentList)
   
   ; allocate the contents object 
   *contents = LinkedList_Add(s_tPDF\ContentList)
   
   IPF_CHECK_MEM (*contents, pdf_AddPage_MEM)
   
   *contents\iObjNum = ipf_NextObjID()
   
   ; point to the contents object
   *page\ToContents = *contents
   
   ProcedureReturn #PDF_OK
   
   
   pdf_AddPage_MEM:
   
   ProcedureReturn ipf_SetError (#PDF_ERR_OUT_OF_MEMORY, #PB_Compiler_Procedure)
   
EndProcedure

; IDE Options = PureBasic 6.03 LTS (Windows - x86)
; CursorPosition = 60
; FirstLine = 31
; Folding = -------------
; EnableXP
; EnableUser
; CPU = 1
; CompileSourceDirectory
; EnableBuildCount = 0
; EnableExeConstant