EnableExplicit

; iBackColor
#SBTEXT_EX_TRANSPARENT    = -1

; iTextPos
#SBTEXT_EX_LEFT       = #DT_LEFT | #DT_VCENTER | #DT_SINGLELINE
#SBTEXT_EX_CENTER     = #DT_CENTER | #DT_VCENTER | #DT_SINGLELINE
#SBTEXT_EX_RIGHT      = #DT_RIGHT | #DT_VCENTER | #DT_SINGLELINE

; flgBlinkStatus
#SBTEXT_EX_BLINK_OFF      = 0
#SBTEXT_EX_BLINK_NORMAL   = 1
#SBTEXT_EX_BLINK_REVERSE  = 2

; internally used 

#_SBTEXT_EX_MAX_FIELDS           =  16
#_SBTEXT_EX_BLINK_SPEED_DEFAULT  =  500

#_SBTEXT_EX_BLINKING_OFF         = -1
#_SBTEXT_EX_BLINKING_STATE_A     =  0
#_SBTEXT_EX_BLINKING_STATE_B     =  1

Structure T_SB_TEXT_ITEM
 iField.i
 
 Text$ 
 iTextPos.i
 
 iForeColor.i
 iBackColor.i
 
 hFont.i
 
 flgBlinking.i
 flgBlinkStyle.i
EndStructure 

Structure T_SB_TEXT
 hStatus.i
 hTimer.i
 iBlinkSpeed.i
 *aiFields.T_SB_TEXT_ITEM[#_SBTEXT_EX_MAX_FIELDS]
EndStructure : Define tSB_TEXT.T_SB_TEXT


Procedure SB_FreeString (*StringAddress)
 Protected String.String 
 PokeI (@String, *StringAddress)  
EndProcedure

Procedure SB_WinMain_Subclass (hWnd, Msg, wParam, lParam)
 Shared tSB_TEXT.T_SB_TEXT
 
 Protected *tSB_TEXT_ITEM.T_SB_TEXT_ITEM
 Protected *DrawItem.DRAWITEMSTRUCT, tRECT.RECT 
 Protected *oldWinProc = GetProp_(hWnd, "oldWinProc")
 Protected hStatus, hBrush
 Protected iTextPos, iForeColor, iBackColor, k
 Protected flgTransparent, flgSkipDraw
   
 Select Msg
 
    Case #WM_SIZING
        ; update status bar
        hStatus = tSB_TEXT\hStatus
        InvalidateRect_(hStatus, 0, 1) 
        UpdateWindow_(hStatus)
         
    Case #WM_NCDESTROY
        ; cleanup props        
        RemoveProp_(hWnd, "oldWinProc")
        
        ; destroy timer (if any)
        If tSB_TEXT\hTimer > 0
            KillTimer_(0, tSB_TEXT\hTimer)
        EndIf
        
        ; cleanup allocated structures
        For k = 0 To #_SBTEXT_EX_MAX_FIELDS - 1
            If tSB_TEXT\aiFields[k] > 0                
                SB_FreeString(@tSB_TEXT\aiFields[k]\Text$)                
                FreeMemory(tSB_TEXT\aiFields[k])
            EndIf
        Next                
    
    Case #WM_DRAWITEM        
        hStatus = tSB_TEXT\hStatus
        
        If wParam = GetDlgCtrlID_(hStatus)
            
            flgSkipDraw = #False
            
            *DrawItem = lParam
            *tSB_TEXT_ITEM = tSB_TEXT\aiFields[*DrawItem\itemData]
            
            iTextPos = *tSB_TEXT_ITEM\iTextPos

            iForeColor = *tSB_TEXT_ITEM\iForeColor
            
            iBackColor = *tSB_TEXT_ITEM\iBackColor
                        
            If iBackColor = #SBTEXT_EX_TRANSPARENT
                flgTransparent = #True
            Else
                flgTransparent = #False
            EndIf

            ; some margins
                    
            If *tSB_TEXT_ITEM\iField = 0
                *DrawItem\rcItem\left + 1
            EndIf

            *DrawItem\rcItem\right - 3

            
            If *tSB_TEXT_ITEM\flgBlinking <> #_SBTEXT_EX_BLINKING_OFF                 
                ; text blinking 
                
                Select *tSB_TEXT_ITEM\flgBlinkStyle
                
                    Case #SBTEXT_EX_BLINK_REVERSE                  
                        If *tSB_TEXT_ITEM\flgBlinking = #_SBTEXT_EX_BLINKING_STATE_B
                            Swap iForeColor, iBackColor
                        EndIf
                                            
                    Case #SBTEXT_EX_BLINK_NORMAL                    
                        If *tSB_TEXT_ITEM\flgBlinking = #_SBTEXT_EX_BLINKING_STATE_B                               
                            flgSkipDraw = #True
                        EndIf

                EndSelect             
            EndIf
            

            SetTextColor_(*DrawItem\hDC, iForeColor)
            
            If flgTransparent
                SetBkMode_(*DrawItem\hDC, #TRANSPARENT)
            Else
                SetBkMode_(*DrawItem\hDC, #OPAQUE)
                SetBkColor_(*DrawItem\hDC, iBackColor)
                
                hBrush = CreateSolidBrush_(iBackColor)                                
                FillRect_(*DrawItem\hDC, *DrawItem\rcItem, hBrush)
                DeleteObject_(hBrush)                                                                                                                         
            EndIf                    
                        
            If flgSkipDraw = #False            
                InflateRect_(*DrawItem\rcItem, -2, 0)
                
                If *tSB_TEXT_ITEM\hFont <> #Null
                    SelectObject_(*DrawItem\hDC, *tSB_TEXT_ITEM\hFont)
                EndIf
                
                DrawText_(*DrawItem\hDC, *tSB_TEXT_ITEM\Text$, -1, *DrawItem\rcItem, iTextPos)
            EndIf


            ProcedureReturn 1
        EndIf
  EndSelect
  
  ; This will call the original winproc of the parent window or it will chain up 
  ; to the new winproc you have defined with SetWindowCallback() in PB
  
  ; In short: it's compatible with your own SetWindowCallback()
  
  ProcedureReturn CallWindowProc_(*oldWinProc, hWnd, Msg, wParam, lParam)   
EndProcedure 

Procedure SB_StatusBar_Subclass (hWnd, Msg, wParam, lParam)
 Protected tRECT.RECT
 Protected k, X, Y
 Protected *fpFunc
 Protected *oldWinProc = GetProp_(hWnd, "oldWinProc")
 
 Select Msg

    Case #WM_NCDESTROY
        ; cleanup
        RemoveProp_(hWnd, "oldWinProc")
        RemoveProp_(hWnd, "callback")
 
    Case #WM_LBUTTONDBLCLK, #WM_RBUTTONDBLCLK, #WM_LBUTTONDOWN, #WM_RBUTTONDOWN
        X = LOWORD (lParam) ; X mouse coordinates
        Y = HIWORD (lParam) ; Y mouse coordinates
        *fpFunc = GetProp_(hWnd, "callback") ; callback address
        
        If *fpFunc > 0               
            While SendMessage_(hWnd, #SB_GETRECT, k, @tRECT)           
                If X < tRECT\right
                    ; call callback procedure
                    CallFunctionFast(*fpFunc, Msg, k, X, Y)
                    Break
                EndIf                                            
                k + 1
            Wend
        EndIf
  EndSelect
 
  ProcedureReturn CallWindowProc_(*oldWinProc, hWnd, Msg, wParam, lParam)   
EndProcedure

Procedure SB_TimerEvent()
 Shared tSB_TEXT.T_SB_TEXT
 
 Protected k, nBlinkingCount
 
 For k = 0 To #_SBTEXT_EX_MAX_FIELDS - 1
    If tSB_TEXT\aiFields[k] > 0
        If tSB_TEXT\aiFields[k]\flgBlinking <> #_SBTEXT_EX_BLINKING_OFF
            ; something found
            nBlinkingCount + 1
            ; invert status
            If tSB_TEXT\aiFields[k]\flgBlinking = #_SBTEXT_EX_BLINKING_STATE_A
                tSB_TEXT\aiFields[k]\flgBlinking = #_SBTEXT_EX_BLINKING_STATE_B
            Else
                tSB_TEXT\aiFields[k]\flgBlinking = #_SBTEXT_EX_BLINKING_STATE_A
            EndIf        
        EndIf
    EndIf
 Next                

 If nBlinkingCount > 0
    ; maybe we could invalidate only a smaller region here ...
    ; but probabably it's not worth the effort
    InvalidateRect_(tSB_TEXT\hStatus, 0, 1) 
 Else 
    ; if there is no need for timer anymore ...
    KillTimer_(0, tSB_TEXT\hTimer)
    tSB_TEXT\hTimer = 0 ; fix
 EndIf
             
EndProcedure


Procedure StatusBarEx_GrabClickEvents (hWnd, *fpFunc)
 SetProp_(hWnd, "oldWinProc", GetWindowLongPtr_(hWnd, #GWL_WNDPROC))   
 SetProp_(hWnd, "callback", *fpFunc)
 SetWindowLongPtr_(hWnd, #GWL_WNDPROC, @SB_StatusBar_Subclass()) 
EndProcedure

Procedure.i StatusBarEx_SetBlinkSpeed (iBlinkSpeed)
 Shared tSB_TEXT.T_SB_TEXT
 
 If iBlinkSpeed < 100
    iBlinkSpeed = 100
 ElseIf iBlinkSpeed > 1000
    iBlinkSpeed = 1000
 EndIf
 
 If iBlinkSpeed <> tSB_TEXT\iBlinkSpeed
    ; if speed changed
    
    If tSB_TEXT\hTimer > 0
        KillTimer_(0, tSB_TEXT\hTimer)
    EndIf
    
    tSB_TEXT\iBlinkSpeed = iBlinkSpeed
    
    tSB_TEXT\hTimer = SetTimer_(0, 0, tSB_TEXT\iBlinkSpeed, @SB_TimerEvent())            
 EndIf
 
 If tSB_TEXT\hTimer = 0
    ProcedureReturn #False
 EndIf
 
 ProcedureReturn #True
EndProcedure

Procedure.i StatusBarEx_SetBlink (hStatus, iField, flgBlinkStatus)
 Shared tSB_TEXT.T_SB_TEXT
 
 If iField < 0 Or iField > #_SBTEXT_EX_MAX_FIELDS - 1 
    ; out of range
    ProcedureReturn #False
 EndIf
 
 If tSB_TEXT\aiFields[iField] = 0 
    ; field not initialized
    ProcedureReturn #False
 EndIf

 If flgBlinkStatus < #SBTEXT_EX_BLINK_OFF Or flgBlinkStatus > #SBTEXT_EX_BLINK_REVERSE  
    ProcedureReturn #False ; invalid blink style
 EndIf
 
 If (tSB_TEXT\aiFields[iField]\iBackColor = #SBTEXT_EX_TRANSPARENT) And (flgBlinkStatus = #SBTEXT_EX_BLINK_REVERSE)
    ProcedureReturn #False ; blink style not compatible with transparent background
 EndIf
 
 If flgBlinkStatus = #SBTEXT_EX_BLINK_OFF
    tSB_TEXT\aiFields[iField]\flgBlinking = #_SBTEXT_EX_BLINKING_OFF
    InvalidateRect_(hStatus, 0, 0) ; fix
    ProcedureReturn #True
 EndIf
 
 tSB_TEXT\aiFields[iField]\flgBlinking = #_SBTEXT_EX_BLINKING_STATE_A 
 tSB_TEXT\aiFields[iField]\flgBlinkStyle = flgBlinkStatus
 
 If tSB_TEXT\hTimer = 0 ; first time
    If tSB_TEXT\iBlinkSpeed = 0
        tSB_TEXT\iBlinkSpeed = #_SBTEXT_EX_BLINK_SPEED_DEFAULT
    EndIf
    
    tSB_TEXT\hTimer = SetTimer_(0, 0, tSB_TEXT\iBlinkSpeed, @SB_TimerEvent())
 
    If tSB_TEXT\hTimer = 0
        tSB_TEXT\aiFields[iField]\flgBlinking = #_SBTEXT_EX_BLINKING_OFF
        ProcedureReturn #False
    EndIf
 EndIf
 
 ProcedureReturn #True
 
EndProcedure

Procedure.i StatusBarEx_SetText (hStatus, iField, Text$, iForeColor, iBackColor = #SBTEXT_EX_TRANSPARENT, iTextPos = #SBTEXT_EX_LEFT, hFont = #Null) 
 Shared tSB_TEXT.T_SB_TEXT
 
 Protected tSB_TEXT_ITEM.T_SB_TEXT_ITEM
 Protected hWnd = GetParent_(hStatus)
 
 
 If iField < 0 Or iField > #_SBTEXT_EX_MAX_FIELDS - 1 ; out of range
    ProcedureReturn #False
 EndIf
 
 If GetProp_(hWnd, "oldWinProc") = 0 ; first time only
    tSB_TEXT\hStatus = hStatus 
    SetProp_(hWnd, "oldWinProc", GetWindowLongPtr_(hWnd, #GWL_WNDPROC))   
    SetWindowLongPtr_(hWnd, #GWL_WNDPROC, @SB_WinMain_Subclass())
 EndIf
 
 If tSB_TEXT\aiFields[iField] = 0 ; if it's the first time this field is referenced
    tSB_TEXT\aiFields[iField] = AllocateMemory(SizeOf(T_SB_TEXT_ITEM))
    
    If tSB_TEXT\aiFields[iField] = 0 ; out of memory
        ProcedureReturn #False
    EndIf

    ; first time init    
    With tSB_TEXT\aiFields[iField]
        \iField = iField
        \flgBlinking = #_SBTEXT_EX_BLINKING_OFF
    EndWith    
 EndIf
 
 
 ; these can be changed calling StatusBar_SetText() again
 With tSB_TEXT\aiFields[iField]
    \Text$ = Text$ 
    \iTextPos = iTextPos
    \iForeColor = iForeColor 
    \iBackColor = iBackColor        
    \hFont = hFont
 EndWith
  
 SendMessage_(hStatus, #SB_SETTEXT, iField | #SBT_OWNERDRAW, iField)
 
 ProcedureReturn #True
EndProcedure

; IDE Options = PureBasic 5.73 LTS (Windows - x86)
; CursorPosition = 1
; Folding = --
; EnableXP
; EnableUser
; CPU = 1
; CompileSourceDirectory