; i2pdf - Image to PDF
; Initial version built with PureBasic 4.3 on Nov 2008.
; This software is FREEWARE

; Dec 2023
; Updated for PB 6.03.
; Added the A2 paper format as requested.
; NOTE: due to some changes you should delete the old .ini and let the program recreate it from scratch.
 
; May 2022
; The structure of the program is pretty bad, uses a lot of globals and there are a lot of cross references between the includes.
; I wrote it in a couple of weeks in the year 2008 and used it as an excuse to learn PB at the same time, so it is what it is but I've never found the will to rewrite it better.
; I've converted the code to make it compilable with PB 5.73, there were a lot of problems due the removal of ASCII strings, altered messaging routing, syntax for pointers and more ...
; I hope I didn't break anything, but I've used it many times in different occasions and it seems to work.

EnableExplicit

IncludeFile "Support.pb"

;****************************
;*** STRUCTURES / GLOBALS ***
;****************************

Structure T_OPTIONS
   
   LastDirectory$
   
   tWinPos.T_WINDOW_RECT
   
   ; menu options
   
   flgStayOnTop.i
   flgRememberWinPos.i
   flgShowThumbs.i
   
   ; pdf creation
   
   flgJpgCompression.i
   iJpgQuality.i              ; 1-10
   iImageScaling.i            ; 1-100 %
   iResizeOnlyIfWider.i       ; >= 0
   iPaperOrientation.i        ; #PAPER_PORTRAIT, #PAPER_LANDSCAPE
   iPaperFormat.i             ; #PAPER_FORMAT_?
   iPdfStartup.i              ; #PDF_ZOOM_?
   flgEnlargeToFit.i
   flgKeepAspect.i
   
   ; program settings
   
   flgRememberLastDirectory.i
   flgSaveSameDirAsFiles.i
   flgClearBeforeDrop.i
   flgSortAfterDrop.i
   flgShowOptions.i
   flgShowFullPath.i
   flgOpenPdf.i
   flgOpenDir.i
   flgConfirmations.i
   flgMemoryLoad.i
   flgShowSizeCondensed.i
   flgSortIncludingPath.i
   flgPreprocessing.i
   flgDocTitleFromFileName.i
   flgSaveAuthor.i
   flgSaveTitle.i
   flgDeleteLogIfSuccessful.i
   flgEnableSubFolders.i
   flgJpgNoRecompression.i
   
   ; document info
   
   DocAuthor$
   DocTitle$
   
   ; thumbnail
   
   flgThumbLocked.i
   iThumbSide.i           ; #THUMB_LEFT, #THUMB_RIGHT
   iThumbSize.i           ; #THUMB_SIZE1, #THUMB_SIZE2, #THUMB_SIZE3
   iThumbColor.i          ; #THUMB_BLACK, #THUMB_WHITE
   
   ; page title
   
   flgDisplayHeader.i
   iHeaderFontSize.i      ; 8,9,10,11,12
   iHeaderDisplayWhat.i   ; #HEADER_SHOW_NAME, #HEADER_SHOW_FULL_NAME
   iHeaderDisplayWhere.i  ; #HEADER_TOP, #HEADER_BOTTOM
   iHeaderDisplayAlign.i  ; #HEADER_ALIGN_LEFT, #HEADER_ALIGN_CENTER, #HEADER_ALIGN_RIGHT
EndStructure

Structure T_ITEM_DATA
   iItemID.i
   FullFileName$
   FileName$
   iResX.i
   iResY.i
   iFileSize.i
   nImage.i
   flgStatus.i
EndStructure

Structure T_DRAG_CURSOR
   iX.i
   iY.i
   iWidth.i
   iHeight.i
   iBackColor.i
   flgVisible.i
EndStructure

; All following variables are shared only in procedures in this file,
; but not in procedures in other files.
Define s_ItemDragged, s_GdiToken, s_flgResizing
Define s_FontStatusBar, s_UniqueID, s_TotFilesSize
Define s_Options.T_OPTIONS
Define s_tDragCursor.T_DRAG_CURSOR
Define s_LastThumbPos.POINT
Define s_LastWinSize.T_WINDOW_RECT
Define *s_LastThumbHandle
Define NewList s_lsItem.T_ITEM_DATA()

;****************************
;*** CONSTANTS **************
;****************************

#PROGRAM_NAME$      = "i2pdf"
#PROGRAM_VERSION$   = "1.0"
#BUILD_VERSION      =   52

#WM_SHOW_THUMB    = #WM_USER
#WM_REFRESH_THUMB = #WM_USER + 1

#WIN_MIN_WIDTH  = 600
#WIN_MIN_HEIGHT = 400

#WIN_WIDTH  = #WIN_MIN_WIDTH
#WIN_HEIGHT = #WIN_MIN_HEIGHT

#RESOLUTION_COLUMN_WIDTH    = 100
#FILESIZE_COLUMN_WIDTH      = 100

#AREA_RIGHT_WIDTH   = 15
#AREA_BOTTOM_HEIGHT = 60

#PAPER_PORTRAIT     = 1
#PAPER_LANDSCAPE    = 2

#PAPER_FORMAT_A4          = 1
#PAPER_FORMAT_A3          = 2
#PAPER_FORMAT_A2          = 3
#PAPER_FORMAT_LETTER      = 4
#PAPER_FORMAT_LEGAL       = 5
#PAPER_FORMAT_PAPERLESS   = 6

#ITEM_BOTTOM    = 0
#ITEM_TOP       = 1

#THUMB_LEFT     = 1
#THUMB_RIGHT    = 2

#THUMB_SIZE1    = 1
#THUMB_SIZE2    = 2
#THUMB_SIZE3    = 3

#THUMB_BLACK    = 1
#THUMB_WHITE    = 2

#HEADER_SHOW_NAME        = 1
#HEADER_SHOW_FULL_NAME   = 2

#HEADER_TOP         = 1
#HEADER_BOTTOM      = 2

#HEADER_ALIGN_LEFT      = 1
#HEADER_ALIGN_CENTER    = 2
#HEADER_ALIGN_RIGHT     = 3

#HEADER_SIZE_MIN    = 8
#HEADER_SIZE_MAX    = 12

#I2PDF_LOG_UNSUPPORTED = 1
#I2PDF_LOG_JPG_STORING = 2

IncludeFile "GdiPlus.pb"
IncludeFile "StatusBarEx.pb"
IncludeFile "PDF.pb"
IncludeFile "LogFile.pb"

;****************************
;*** WINDOWS ****************
;****************************

Enumeration
   #WIN_MAIN
   #WIN_SETTINGS
   #WIN_ABOUT
   #WIN_PROCESS
   #WIN_BUILD_PDF
   #WIN_DRAG_TOOLTIP
   #WIN_THUMB
   #WIN_SKIPPED
EndEnumeration


;****************************
;- GADGETS
;****************************

Enumeration
   
   ;#WIN_MAIN
   #LIG_FILES
   #BTN_BUILD_PDF
   #STB_STATUS
   #TXT_JPG_STORING_MODE
   
   ;#WIN_PROCESS
   #PRB_PROCESS
   #BTN_PROCESS_CANCEL
   
   ;#WIN_BUILD_PDF
   #PRB_BUILDING
   #TXT_BUILD_PDF_1
   #TXT_BUILD_PDF_2
   #BTN_BUILD_CANCEL
   
   ;#WIN_SKIPPED
   #IMG_SKIPPED_ICON
   #BTN_SKIPPED_OK
   #BTN_SKIPPED_SHOW
   #TXT_SKIPPED_1
   
   ;#WIN_SETTINGS
   #PNL_SETTINGS
   
   ; TAB 1
   #FRM_PDF_IMAGE_PROC
   
   #CHK_COMPRESS
   
   #TXT_QUALITY_1
   #TRK_QUALITY
   #TXT_QUALITY_2
   
   #TXT_SCALING_1
   #TRK_SCALING
   #TXT_SCALING_2
   
   #TXT_RESIZE_IF
   #STG_RESIZE_IF
   
   #FRM_PDF_OUTPUT
   
   #TXT_PAPER_ORIENATION
   #OPT_PORTRAIT
   #OPT_LANDSCAPE
   
   #TXT_PAPER_FORMAT
   #CMB_PAPER_FORMAT
   
   #TXT_PDF_STARTUP
   #CMB_PDF_STARTUP
   
   #CHK_ENLARGE_TO_FIT
   
   #CHK_KEEP_ASPECT
   
   ; TAB 2
   #FRM_HEADER
   
   #CHK_SHOW_HEADER
   
   #TXT_HEADER_SIZE
   #CMB_HEADER_SIZE
   
   #TXT_HEADER_WHAT
   #CMB_HEADER_WHAT
   #TXT_HEADER_WHERE
   #CMB_HEADER_WHERE
   
   #TXT_HEADER_ALIGN
   #CMB_HEADER_ALIGN
   
   #FRM_DOC_INFO
   
   #TXT_DOC_AUTHOR_1
   #TXT_DOC_AUTHOR_2
   #STG_DOC_AUTHOR
   #CHK_SAVE_AUTHOR
   
   #TXT_DOC_TITLE_1
   #TXT_DOC_TITLE_2
   #STG_DOC_TITLE
   #CHK_SAVE_TITLE
   
   ; TAB 3
   #FRM_THUMB_POSITION
   #CHK_LOCKED_TO_MAIN
   #OPT_LOCKED_LEFT
   #OPT_LOCKED_RIGHT
   
   #FRM_THUMB_SIZE
   #OPT_THUMB_SIZE_1
   #OPT_THUMB_SIZE_2
   #OPT_THUMB_SIZE_3
   
   #FRM_THUMB_BACK_COLOR
   #OPT_THUMB_BLACK
   #OPT_THUMB_WHITE
   
   ; TAB 4
   #SCL_OPTIONS
   #CHK_SAVE_LAST_DIR
   #CHK_SAVE_SAME_AS_FILES
   #CHK_SHOW_OPTIONS
   #CHK_CLEAR_BEFORE_DROP
   #CHK_SHOW_FULL_PATH
   #CHK_SORT_ALWAYS
   #CHK_OPEN_PDF
   #CHK_OPEN_DIR
   #CHK_CONFIRMATIONS
   #CHK_MEMORY_LOAD
   #CHK_SHOW_SIZE_CONDENSED
   #CHK_SORT_INCLUDING_PATH
   #CHK_PREPROCESSING
   #CHK_DOCTITLE_FROM_FILENAME
   #CHK_DELETE_LOG_IF_SUCCESSFUL
   #CHK_ENABLE_SUBFOLDERS
   #CHK_JPG_NO_RECOMPRESSION
   
   ; confirm / cancel
   #BTN_OPTIONS_OK
   #BTN_OPTIONS_CANCEL
   
   ;#WIN_ABOUT
   #IMG_ABOUT_TITLE
   #TXT_ABOUT_1
   #TXT_ABOUT_2
   #TXT_ABOUT_3
   #TXT_ABOUT_4
   #BTN_PB_LOGO
   #BTN_ABOUT_OK
   
   ;#WIN_THUMB
   #IMG_THUMB
   #IMA_THUMB
EndEnumeration


;****************************
;- MENUS
;****************************

Enumeration ; menus
   #MNU_WIN_MAIN
   
   ;items 1
   #MNI_SORT_NOW
   #MNI_CLEAR
   #MNI_SELECT_ALL
   #MNI_DESELECT_ALL
   #MNI_QUIT
   
   ; items 2
   #MNI_STAYONTOP
   #MNI_REMEMBER_WIN_POS
   #MNI_SHOW_THUMBS
   #MNI_SETTINGS
   #MNI_ABOUT
   
   ; items 3
   #MNI_HELP
   
   ; menu popup
   #MNU_POPUP
   
   ;items
   #MNI_POP_MOVE_TO_TOP
   #MNI_POP_MOVE_TO_BOTTOM
   #MNI_POP_DELETE
   #MNI_POP_CLOSE
EndEnumeration

;****************************
;*** MACROS *****************
;****************************

Macro PROGRAM_BUILD()
   Str(#BUILD_VERSION)
EndMacro

Macro PROGRAM_TITLE()
   #PROGRAM_NAME$ + " " + #PROGRAM_VERSION$ + "." + PROGRAM_BUILD()
EndMacro

CompilerSelect  (#PB_Compiler_Processor)
   CompilerCase #PB_Processor_x86
      Macro PROGRAM_TITLE_FULL()
         PROGRAM_TITLE() + " (x86)"
      EndMacro
   CompilerCase #PB_Processor_x64
      Macro PROGRAM_TITLE_FULL()
         PROGRAM_TITLE() + " (x64)"
      EndMacro
CompilerEndSelect

Macro DisableWin (nWindow)
   If IsWindow(nWindow)
      DisableWindow(nWindow, 1)
   EndIf
EndMacro

Macro EnableWin (nWindow)
   If IsWindow(nWindow)
      DisableWindow(nWindow, 0)
   EndIf
EndMacro

Macro HideWin (nWindow)
   If IsWindow(nWindow)
      HideWindow (nWindow, 1)
   EndIf
EndMacro

Macro ShowWin (nWindow)
   If IsWindow(nWindow)
      HideWindow (nWindow, 0)
   EndIf
EndMacro

Macro PumpMessages()
   While WindowEvent(): Delay(1): Wend
EndMacro

Macro I2P_CHECK_PDF (label)
   If pdf_GetErrorCode() <> #PDF_OK: Goto label : EndIf
EndMacro

Macro I2P_CHECK_GDI (ret, label)
   If ret <> #GDIPLUS_OK: Goto label : EndIf
EndMacro

Macro I2P_CHECK_JPG_STORE (ret, label)
   If ret = 0: Goto label : EndIf
EndMacro

;- DECLARES

Declare 	SetVistaProgressBarValue (nGadget, iValue)
Declare 	StatusBar_ClickEvents_CallBack (iMessage, iField, iXPos, iYPos)
Declare 	Init()
Declare 	Close()
Declare 	Abort()
Declare.i 	HandleError (iResult, CallingProc$, Message$, iLineNum)
Declare.i 	FindElement (iItemID)
Declare 	ShowDragCursor()
Declare 	HideDragCursor()
Declare 	DrawDragCursor()
Declare 	CreateDragCursor()
Declare 	DestroyDragCursor()
Declare 	SetThumbLocking(flgThumbLocked)
Declare 	RefreshThumbnail()
Declare.i 	CallBack_Thumb (hWnd, iMessage, wParam, lParam)
Declare.i 	CallBack_WinMain (hWnd, iMessage, wParam, lParam)
Declare.i 	ThumbSize()
Declare 	CloseThumbnail()
Declare 	ShowThumbnail (iItem)
Declare 	CheckAllItems (nGadget)
Declare 	UncheckAllItems (nGadget)
Declare.i 	CountCheckedItems (nGadget)
Declare 	MoveCheckedItems (nGadget, iDirection)
Declare 	MoveCheckedItem (iItemCurrent, iDirection)
Declare 	DeleteItem (nPos)
Declare 	DeleteCheckedItems (nGadget)
Declare.i 	CallBack_DragMain (iAction)
Declare.i 	CallBack_ListViewHdr (hWnd, iMessage, wParam, lParam)
Declare.i 	CallBack_ListView (hWnd, iMessage, wParam, lParam)
Declare 	WinResize()
Declare 	CopyItem (*dest.T_ITEM_DATA, *source.T_ITEM_DATA)
Declare 	MoveItemToList (iItemID, iItemSource, iItemDest)
Declare 	MoveItemToGadget (iItemID, iItemSource, iItemDest)
Declare 	ItemDroppedOuter (iItemDragged, iOuterPos)
Declare 	ItemDropped (iItemDragged, iItemCurrent)
Declare 	AddItemToList (*tItem.T_ITEM_DATA)
Declare 	SetItemStatus (iItemCurrent)
Declare 	AddItemToListView (*tItem.T_ITEM_DATA)
Declare 	ClearItemsData()
Declare 	SortListView()
Declare 	EnableHeaderGadgets (flgStatus)
Declare 	SetBuildButtonState()
Declare.i 	ProcessFile (*tItem.T_ITEM_DATA)
Declare 	RefreshListView()
Declare 	AddFilesDropped (List lsFilesDropped$())
Declare.i 	ProcesFilesDropped (List lsFilesDropped$(), List lsFilesSkipped$(), *flgCanceled.Integer)
Declare 	SetJpgQuality (iQuality)
Declare 	SetScaling (iScaling)
Declare.i 	SaveSkippedList (FileName$, List lsFilesSkipped$())
Declare.i 	ShowSkippedFiles (List lsFilesSkipped$())
Declare 	ShowAbout()
Declare 	EnableImageQualityGadgets (flgStatus)
Declare 	EnableImageResizeGadgets (iResize)
Declare.s 	PaperSizeToMM_STR (Format$)
Declare.s 	PaperSizeToInches_STR (Format$)
Declare 	LoadPaperFormats (nGadget)
Declare 	LoadPdfZoom (nGadget)
Declare 	LoadHeaderSize (nGadget)
Declare 	LoadHeaderAlignment (nGadget)
Declare 	LoadHeaderPosition (nGadget)
Declare 	LoadHeaderWhat (nGadget)
Declare.i 	CheckSettings()
Declare.s 	PrefFileName()
Declare.i 	SaveSettings()
Declare.i 	LoadSettings()
Declare 	UpdateSettings()
Declare 	EnablePaperFormatGadgets (flgStatus)
Declare.i 	ShowSettingsPanel()
Declare.s 	AskSkippedListName ()
Declare.i 	CheckIfAllFromSamePath()
Declare.s 	AskPdfName(FileName$)
Declare.i 	LoadJpgFileIntoBuffer (ImageFileName$, *StreamBuffer.Integer)
Declare.i 	BuildPdf (OutFile$, *tLogfile.T_LOGFILE)
Declare 	PurgeMemory()
Declare.i 	ShowOverLoadWarning()
Declare 	ClearListView()
Declare.i 	AskForExit()
Declare.i 	OverWriteIfAlreadyThere (FileName$)
Declare.i 	GetOuterPosition (nGadget, iYdrop)
Declare 	GetLocalCursorPosXY (hWin, *iX, *iY)
Declare.s 	GetLowestDirPart(FullPath$)
Declare.s 	GetTitleFromFilename (FileName$)
Declare.i 	ExitMain()
Declare 	UpdateNumFiles()
Declare 	UpdateStatusBar()
Declare 	ExpandDirectory (RootDirectory$, List lsFiles$(), flgRecursive)
Declare.i 	ExpandDirectories (List lsFiles$(), flgRecursive)
Declare.i 	StringGetWidth (nWindow, nFont, Text$)
Declare 	Main(*tLogfile.T_LOGFILE)

CompilerIf (#PB_Compiler_Debugger = 1)
   SetCurrentDirectory("C:\Dev\PB\projects\i2pdf\out32")
CompilerEndIf

DataSection
   lbl_purebasic_logo:
   IncludeBinary "..\res\pb_logo.bmp"
   lbl_i2pdf_logo:
   IncludeBinary "..\res\i2pdf_logo.bmp"
EndDataSection

#VISTA_PROGBAR_RESOLUTION = 100

Macro VISTA_PROGBAR_FIX (topvalue)
   topvalue * #VISTA_PROGBAR_RESOLUTION
EndMacro

Procedure SetVistaProgressBarValue  (nGadget, iValue)
   ; hack to partially fix a stupid Vista/Win 7 behaviour
   SetGadgetState(nGadget, iValue * #VISTA_PROGBAR_RESOLUTION)
   SetGadgetState(nGadget, iValue * #VISTA_PROGBAR_RESOLUTION - 1)
   SetGadgetState(nGadget, iValue * #VISTA_PROGBAR_RESOLUTION )
EndProcedure

Macro HE (result, MSG)
   HandleError (result, #PB_Compiler_Procedure, MSG, #PB_Compiler_Line)
EndMacro

Procedure StatusBar_ClickEvents_CallBack (iMessage, iField, iXPos, iYPos)
   If iMessage = #WM_LBUTTONDBLCLK
      ShowSettingsPanel()
      InvalidateRect_(GadgetID(#LIG_FILES), 0, 1) ; needed for 64 bit version ...
      UpdateStatusBar()
   EndIf
EndProcedure

Procedure Init()
   Shared s_GdiToken
   
   If OSVersion() < #PB_OS_Windows_XP
      MessageBox_(#Null, "This program requires Windows XP or later to run.", PROGRAM_TITLE(), #MB_OK | #MB_ICONSTOP)
      Abort()
   EndIf
   
   If GdiPlusInit(@s_GdiToken) <> #GDIPLUS_OK
      MessageBox_(#Null, "GDI+ initialization error.", PROGRAM_TITLE(), #MB_OK | #MB_ICONSTOP)
      Abort()
   EndIf
   
   If CheckSingleInstanceEx (PROGRAM_TITLE()) = #False
      End
   EndIf
   
EndProcedure

Procedure Close()
   Shared s_GdiToken
   
   If s_GdiToken
      GdiplusShutdown(s_GdiToken)
   EndIf
EndProcedure

Procedure Abort()
   Close()
   End
EndProcedure

Procedure.i HandleError (iResult, CallingProc$, Message$, iLineNum)
   
   ; Handles the procedures return values and quit the program in case of fatal error
   
   Protected Text$
   
   If Len(CallingProc$)
      CallingProc$ + "()"
   Else
      CallingProc$ = "main source"
   EndIf
   
   If iResult = 0
      CenterRequesterPositionToWindow(WindowID(#WIN_MAIN))
      
      Text$ = "Fatal error in " + CallingProc$ + " on line " + Str(iLineNum) + "." + #CRLF$ + #CRLF$
      Text$ + Message$
      
      MessageBox_(WindowID(#WIN_MAIN), Text$, PROGRAM_TITLE(), #MB_OK | #MB_ICONSTOP | #MB_APPLMODAL)
      
      Abort()
   EndIf
   
   ProcedureReturn iResult
   
EndProcedure

Procedure.i FindElement (iItemID)
   Shared s_lsItem()
   
   ResetList(s_lsItem())
   
   While NextElement(s_lsItem())
      If s_lsItem()\iItemID = iItemID
         ProcedureReturn s_lsItem()
      EndIf
   Wend
   
   ProcedureReturn #Null
EndProcedure

Procedure ShowDragCursor()
   Shared s_tDragCursor
   
   If Not s_tDragCursor\flgVisible
      ShowWin (#WIN_DRAG_TOOLTIP)
      s_tDragCursor\flgVisible = #True
   EndIf
EndProcedure

Procedure HideDragCursor()
   Shared s_tDragCursor
   
   If s_tDragCursor\flgVisible
      HideWin (#WIN_DRAG_TOOLTIP)
      s_tDragCursor\flgVisible = #False
   EndIf
EndProcedure

Procedure DrawDragCursor()
   Shared s_ItemDragged, s_tDragCursor, s_lsItem()
   Protected hWin, hDC, hPen, hBrush, iRectRgn
   Protected iEnlarge = 20, iAngle = 5
   Protected iItemID
   
   ResizeWindow(#WIN_DRAG_TOOLTIP, s_tDragCursor\iX, s_tDragCursor\iY, s_tDragCursor\iWidth + iEnlarge, s_tDragCursor\iHeight + iEnlarge / 2)
   
   hWin = WindowID(#WIN_DRAG_TOOLTIP)
   
   hDC = GetDC_(hWin)
   
   hPen = CreatePen_(#PS_INSIDEFRAME, 1, #Black)
   hBrush = CreateSolidBrush_(s_tDragCursor\iBackColor)
   
   SelectObject_(hDC, hPen)
   SelectObject_(hDC, hBrush)
   
   iRectRgn = CreateRoundRectRgn_(0, 0, s_tDragCursor\iWidth + iEnlarge , s_tDragCursor\iHeight + iEnlarge / 2, iAngle, iAngle)
   SetWindowRgn_(hWin, iRectRgn, #True)
   DeleteObject_(iRectRgn)
   
   RoundRect_(hDC, 0, 0, s_tDragCursor\iWidth + iEnlarge - 1, s_tDragCursor\iHeight + iEnlarge / 2 - 1, iAngle, iAngle)
   
   DeleteObject_(hPen)
   DeleteObject_(hBrush)
   
   ReleaseDC_(hWin, hDC)
   
   iItemID = GetGadgetItemData(#LIG_FILES, s_ItemDragged)
   
   HE(FindElement(iItemID), "FindElement() failed.")
   
   StartDrawing(WindowOutput(#WIN_DRAG_TOOLTIP))
   DrawingFont(GetGadgetFont(#LIG_FILES))
   DrawText(iEnlarge / 2, iEnlarge / 4, s_lsItem()\FileName$, #Black, s_tDragCursor\iBackColor)
   StopDrawing()
   
   SetActiveWindow(#WIN_MAIN)
EndProcedure

Procedure CreateDragCursor()
   Shared s_ItemDragged, s_tDragCursor, s_lsItem()
   Protected tSize.SIZE
   Protected iItemID
   
   s_tDragCursor\iBackColor = GetSysColor_(#COLOR_INFOBK) ; tooltip color
   
   HE(OpenWindow(#WIN_DRAG_TOOLTIP, 0,0,0,0, "", #PB_Window_BorderLess | #PB_Window_Invisible, WindowID(#WIN_MAIN)), "Tooltip window creation failed.")
   
   iItemid = GetGadgetItemData(#LIG_FILES, s_ItemDragged)
   
   HE(FindElement(iItemID), "FindElement() failed.")
   
   Gadget_GetStringSize (#WIN_DRAG_TOOLTIP, #LIG_FILES, s_lsItem()\FileName$, @tSize)
   
   s_tDragCursor\iWidth = tSize\cx
   s_tDragCursor\iHeight = tSize\cy
   s_tDragCursor\flgVisible = #False
   
EndProcedure

Procedure DestroyDragCursor()
   Shared s_tDragCursor
   
   CloseWindow(#WIN_DRAG_TOOLTIP)
   s_tDragCursor\flgVisible = #False
EndProcedure

Procedure SetThumbLocking(flgThumbLocked)
   Protected hWnd = WindowID(#WIN_THUMB)
   
   GetSystemMenu_(hWnd, #True)
   
   DeleteMenu_(GetSystemMenu_(hWnd, #False), #SC_SIZE, #MF_BYCOMMAND)
   DeleteMenu_(GetSystemMenu_(hWnd, #False), #SC_RESTORE, #MF_BYCOMMAND)
   DeleteMenu_(GetSystemMenu_(hWnd, #False), #SC_MINIMIZE, #MF_BYCOMMAND)
   DeleteMenu_(GetSystemMenu_(hWnd, #False), #SC_MAXIMIZE, #MF_BYCOMMAND)
   DeleteMenu_(GetSystemMenu_(hWnd, #False), 0, #MF_SEPARATOR)
   
   If flgThumbLocked
      DeleteMenu_(GetSystemMenu_(hWnd, #False), #SC_MOVE, #MF_BYCOMMAND)
   EndIf
   
   DrawMenuBar_(hWnd)
EndProcedure

Procedure RefreshThumbnail()
   Shared s_Options, s_flgResizing, s_LastThumbPos
   Static flgThumbLocked = -1
   
   If IsWindow(#WIN_THUMB)     
      If s_Options\flgThumbLocked
         If s_Options\iThumbSide = #THUMB_RIGHT
            ResizeWindow(#WIN_THUMB, WindowX(#WIN_MAIN) + WindowWidth(#WIN_MAIN) + 2, WindowY(#WIN_MAIN) + 1, #PB_Ignore, #PB_Ignore)  
         Else            
            ResizeWindow(#WIN_THUMB, WindowX(#WIN_MAIN) - WindowWidth(#WIN_THUMB) - 2, WindowY(#WIN_MAIN) + 1, #PB_Ignore, #PB_Ignore) 
         EndIf
         
         s_LastThumbPos\x = WindowX(#WIN_THUMB)
         s_LastThumbPos\y = WindowY(#WIN_THUMB)
      Else
         ResizeWindow(#WIN_THUMB, s_LastThumbPos\x, s_LastThumbPos\y, #PB_Ignore, #PB_Ignore)
      EndIf
      
      If s_Options\flgThumbLocked <> flgThumbLocked
         flgThumbLocked = s_Options\flgThumbLocked
         SetThumbLocking (flgThumbLocked)
      EndIf
      
      If s_flgResizing
         UpdateWindow_(WindowID(#WIN_THUMB))
      EndIf
   EndIf
   
EndProcedure

Procedure.i CallBack_Thumb (hWnd, iMessage, wParam, lParam)
   Shared s_LastThumbPos
   Protected iRetVal = #PB_ProcessPureBasicEvents
   Protected *ptRECT.RECT
   
   Select iMessage
         
      Case #WM_MOVING
         *ptRECT = lParam
         
         s_LastThumbPos\x = *ptRECT\left
         s_LastThumbPos\y = *ptRECT\top
   EndSelect
   
   ProcedureReturn iRetVal
   
EndProcedure

Procedure.i CallBack_WinMain (hWnd, iMessage, wParam, lParam)
   Shared s_Options, s_flgResizing, s_LastWinSize
   Protected iRetVal = #PB_ProcessPureBasicEvents
   Protected *tNMLV.NM_LISTVIEW
   Protected *tNMH.NMHEADER
   Protected tPOINT.POINT
   
   Select iMessage
         
      Case #WM_SYSCOMMAND
         If wParam = #SC_MINIMIZE
            s_LastWinSize\X = WindowX(#WIN_MAIN)
            s_LastWinSize\Y = WindowY(#WIN_MAIN)
            s_LastWinSize\W = WindowWidth(#WIN_MAIN)
            s_LastWinSize\H = WindowHeight(#WIN_MAIN)
         EndIf
         
      Case #WM_SHOW_THUMB
         ShowThumbnail (wParam)
         
      Case #WM_REFRESH_THUMB
         RefreshThumbnail()
         
      Case #WM_MOUSEWHEEL
         If wParam > 0
            ListIconGadget_Scroll(#LIG_FILES, #K_SCROLL_UP)
         Else
            ListIconGadget_Scroll(#LIG_FILES, #K_SCROLL_DOWN)
         EndIf
         
      Case #WM_NCHITTEST
         HideDragCursor()
         
      Case #WM_NOTIFY
         If (s_flgResizing = #False) ; if not resizing through code
            
            *tNMH = lParam
            
            If *tNMH\hdr\code = #HDN_ITEMCHANGING
               ; prevents header colums resize
               iRetVal = 1
            EndIf
            
            *tNMLV = lParam
            
            If *tNMLV\hdr\code = #LVN_COLUMNCLICK
               ; intercept click on the header colums
               ; Debug "Header item " + Str(*tNMLV\iSubItem) + " clicked!"
            EndIf
            
         EndIf
         
      Case #WM_MOVING
         If (s_Options\flgShowThumbs)
            PostMessage_(WindowID(#WIN_MAIN), #WM_REFRESH_THUMB, 0, 0)
         EndIf
         
      Case #WM_SIZING
         WinResize()
         
   EndSelect
   
   ProcedureReturn iRetVal
   
EndProcedure

Procedure.i ThumbSize()
   Shared s_Options
   
   Select s_Options\iThumbSize
      Case #THUMB_SIZE1: ProcedureReturn 320
      Case #THUMB_SIZE2: ProcedureReturn 480
      Case #THUMB_SIZE3: ProcedureReturn 640
   EndSelect
   
   ProcedureReturn 250
   
EndProcedure

Procedure CloseThumbnail()
   Shared *s_LastThumbHandle
   
   If IsWindow(#WIN_THUMB)
      CloseWindow(#WIN_THUMB)
      FreeImage(#IMA_THUMB)
      *s_LastThumbHandle = 0
   EndIf
   
EndProcedure

Procedure ShowThumbnail (iItem)
   Shared s_Options, s_LastThumbPos, s_lsItem(), *s_LastThumbHandle
   Protected iXpos, iYpos, iDeskX, iDeskY, iBackColor
   Static flgFirstShow = #True
   
   If iItem = -1
      CloseThumbnail()
      ProcedureReturn
   EndIf
   
   If s_Options\flgShowThumbs = #False
      ProcedureReturn
   EndIf
   
   If CountGadgetItems(#LIG_FILES) = 0
      ProcedureReturn
   EndIf
   
   HE(FindElement(GetGadgetItemData(#LIG_FILES, iItem)), "FindElement() failed.")
   
   If *s_LastThumbHandle = ImageID(s_lsItem()\nImage)
      ProcedureReturn
   Else
      *s_LastThumbHandle = ImageID(s_lsItem()\nImage)
   EndIf
   
   
   If IsWindow(#WIN_THUMB) = 0
      HE(CreateImage(#IMA_THUMB, ThumbSize(), ThumbSize(), 24), "CreateImage failed.")
      
      HE(OpenWindow(#WIN_THUMB, 1, 1, ThumbSize(), ThumbSize(), #PROGRAM_NAME$ + " - Thumbnail", #PB_Window_SystemMenu | #PB_Window_Invisible, WindowID(#WIN_MAIN)), "Open window failed For #WIN_THUMB.")
      
      SetWindowLongPtr_(WindowID(#WIN_THUMB), #GWL_EXSTYLE, GetWindowLongPtr_(WindowID(#WIN_THUMB),#GWL_EXSTYLE) | #WS_EX_TOOLWINDOW)
      
      ImageGadget(#IMG_THUMB, 0, 0, ThumbSize(), ThumbSize(), ImageID(#IMA_THUMB))
      
      SetActiveWindow(#WIN_MAIN)
      
      SetActiveGadget(#LIG_FILES)
      
      If flgFirstShow
         iDeskX = GetDesktopWidth()
         iDeskY = GetDesktopHeight()
         
         s_LastThumbPos\x = iDeskX - ThumbSize() - 20
         s_LastThumbPos\y = 10
      EndIf
      
      RefreshThumbnail()
      
      ShowWin(#WIN_THUMB)
      
      If flgFirstShow : flgFirstShow = #False : EndIf
      
      SetWindowCallback(@CallBack_Thumb(), #WIN_THUMB)
      
      SmartWindowRefresh(#WIN_THUMB, 1)
   EndIf
   
   Select s_Options\iThumbColor
      Case #THUMB_BLACK
         iBackColor = #Black
      Case #THUMB_WHITE
         iBackColor = #White
   EndSelect
   
   StartDrawing(ImageOutput(#IMA_THUMB))
   
   Box(0,0, ThumbSize(), ThumbSize(), iBackColor)
   iXpos = (ThumbSize() - ImageWidth(s_lsItem()\nImage)) / 2
   iYpos = (ThumbSize() - ImageHeight(s_lsItem()\nImage)) / 2
   DrawImage(ImageID(s_lsItem()\nImage), iXpos, iYpos)
   
   StopDrawing()
   
   SetGadgetState(#IMG_THUMB, ImageID(#IMA_THUMB))
   
EndProcedure

Procedure CheckAllItems (nGadget)
   Protected k, iItems, iCurrent
   
   iItems = CountGadgetItems(nGadget)
   
   iCurrent = GetGadgetState(nGadget)
   
   For k = 0 To iItems - 1
      SetGadgetItemState(nGadget, k, #PB_ListIcon_Checked)
   Next
   
   SetGadgetState(nGadget, iCurrent)
   
EndProcedure

Procedure UncheckAllItems (nGadget)
   Protected k, iItems, iCurrent
   
   iItems = CountGadgetItems(nGadget)
   
   iCurrent = GetGadgetState(nGadget)
   
   For k = 0 To iItems - 1
      SetGadgetItemState(nGadget, k, 0)
   Next
   
   SetGadgetState(nGadget, iCurrent)
EndProcedure

Procedure.i CountCheckedItems (nGadget)
   
   Protected k, iItems, iCount = 0
   
   iItems = CountGadgetItems(nGadget)
   
   For k = 0 To iItems - 1
      If (GetGadgetItemState(nGadget, k) & #PB_ListIcon_Checked)
         iCount + 1
      EndIf
   Next
   
   ProcedureReturn iCount
   
EndProcedure

Procedure MoveCheckedItems (nGadget, iDirection)
   
   Protected k
   Protected iItems = CountGadgetItems(nGadget)
   Protected iCount = CountCheckedItems(nGadget)
   
   CloseThumbnail()
   
   If iDirection = #ITEM_TOP
      While CountCheckedItems(nGadget)
         For k = iItems - 1 To 0 Step -1
            If (GetGadgetItemState(nGadget, k) & #PB_ListIcon_Checked)
               SetGadgetItemState(nGadget, k, 0)
               ItemDroppedOuter (k, iDirection)
               Break
            EndIf
         Next
      Wend
      
      For k = 0 To iCount - 1
         SetGadgetItemState(nGadget, k, #PB_ListIcon_Checked)
      Next
      
      SendMessage_(GadgetID(nGadget), #LVM_ENSUREVISIBLE, 0, #False)
   Else
      While CountCheckedItems(nGadget)
         For k = 0 To iItems - 1
            If (GetGadgetItemState(nGadget, k) & #PB_ListIcon_Checked)
               SetGadgetItemState(nGadget, k, 0)
               ItemDroppedOuter (k, iDirection)
               Break
            EndIf
         Next
      Wend
      
      For k = iItems - iCount To iItems - 1
         SetGadgetItemState(nGadget, k, #PB_ListIcon_Checked)
      Next
      
      SendMessage_(GadgetID(nGadget), #LVM_ENSUREVISIBLE, iItems - 1, #False)
   EndIf
   
EndProcedure

Procedure MoveCheckedItem (iItemCurrent, iDirection)
   
   ItemDroppedOuter (iItemCurrent, iDirection)
   
   If iDirection = #ITEM_TOP
      If iItemCurrent > CountGadgetItems(#LIG_FILES) - 1
         iItemCurrent - 1
      EndIf
   EndIf
   
   SetGadgetState(#LIG_FILES, iItemCurrent)
EndProcedure

Procedure DeleteItem (nPos)
   Shared s_TotFilesSize, s_lsItem()
   Protected iItemID = GetGadgetItemData(#LIG_FILES, nPos)
   
   HE(FindElement(iItemID), "FindElement() failed.")
   
   If IsImage(s_lsItem()\nImage)
      FreeImage(s_lsItem()\nImage)
   EndIf
   
   s_TotFilesSize - s_lsItem()\iFileSize
   
   DeleteElement(s_lsItem())
   
   RemoveGadgetItem(#LIG_FILES, nPos)
   
   UpdateStatusBar()
EndProcedure

Procedure DeleteCheckedItems (nGadget)
   
   Protected k, iItems
   
   CloseThumbnail()
   
   While CountCheckedItems(nGadget)
      iItems = CountGadgetItems(nGadget)
      
      For k = 0 To iItems - 1
         If (GetGadgetItemState(nGadget, k) & #PB_ListIcon_Checked)
            DeleteItem (k)
            Break
         EndIf
      Next
   Wend
   
   If CountGadgetItems(#LIG_FILES)
      SetActiveGadget(#BTN_BUILD_PDF)
   EndIf
   
EndProcedure

Procedure.i CallBack_DragMain (iAction)
   Shared s_tDragCursor
   Protected tPOINT.POINT, iTop, iBottom
   Protected iItemHeight, hListView
   Static iXold, iYold
   
   If iAction = #PB_Drag_Move ; the one expected when dragging the items in the ListIconGadget
      
      GetCursorPos_(@tPOINT) ; get the current mouse pointer position
      
      s_tDragCursor\iX = tPOINT\x + 5
      s_tDragCursor\iY = tPOINT\y + 20
      
      ; handle of the gadget
      hListView = GadgetID(#LIG_FILES)
      
      ; height of a single row
      
      iItemHeight = ListIconGadget_GetHeaderHeight(#LIG_FILES)
      
      If (s_tDragCursor\iX <> iXold) Or (s_tDragCursor\iY <> iYold)
         ShowDragCursor()
         
         DrawDragCursor()
         
         UpdateWindow_(hListView)
         
         iXold = s_tDragCursor\iX
         iYold = s_tDragCursor\iY
      EndIf
      
      MapWindowPoints_(#HWND_DESKTOP, hListView, @tPOINT, 1)
      
      If tPOINT\y <= iItemHeight + 8
         
         ListIconGadget_Scroll(#LIG_FILES, #K_SCROLL_UP)
         
      ElseIf tPOINT\y > GadgetHeight(#LIG_FILES) - 8
         
         ListIconGadget_Scroll(#LIG_FILES, #K_SCROLL_DOWN)
         
      EndIf
      
   EndIf
   
   ProcedureReturn #True
   
EndProcedure


Procedure.i CallBack_ListViewHdr (hWnd, iMessage, wParam, lParam)
   
   Protected *oldWinProc = GetProp_(hWnd, "oldWinProc")
   
   Select iMessage
      Case #WM_NCDESTROY
         RemoveProp_(hWnd, "oldWinProc")
      Case #WM_SETCURSOR
         ProcedureReturn 1
   EndSelect
   
   ProcedureReturn CallWindowProc_(*oldWinProc, hWnd, iMessage, wParam, lParam)
EndProcedure


Procedure.i CallBack_ListView (hWnd, iMessage, wParam, lParam)
   Shared s_lsItem()
   Protected tHitInfo.LVHITTESTINFO, tPOINT.POINT, tRECT.RECT, tSIZE.SIZE
   Protected hBrush, iItemCurrent, iItemid, iCount
   Protected nFontText, TextL1$, TextL2$
   Protected *oldWinProc = GetProp_(hWnd, "oldWinProc")
   
   Select iMessage
      Case #WM_NCDESTROY
         RemoveProp_(hWnd, "oldWinProc")
         
      Case #WM_ERASEBKGND
         If CountGadgetItems(#LIG_FILES) = 0 ; if the list is empty
            
            TextL1$ = "Drop your files here"
            TextL2$ = "( JPG, PNG, BMP, TIFF, GIF )"
            
            GetClientRect_(hWnd, @tRECT)
            
            hBrush = CreateSolidBrush_(GetSysColor_(#COLOR_WINDOW))
            FillRect_(wParam, @tRECT, hBrush)
            DeleteObject_(hBrush)
            
            nFontText = LoadFont(#PB_Any,"Comic Sans MS", 12)
            SelectObject_(wParam, FontID(nFontText))
            FreeFont(nFontText)
            
            GetTextExtentPoint32_(wParam, TextL1$, Len(TextL1$),  @tSIZE)
            
            tRECT\bottom - (tSIZE\cy * 1.5)
            DrawText_(wParam, TextL1$, Len(TextL1$), @tRECT, #DT_CENTER|#DT_VCENTER|#DT_SINGLELINE)
            
            tRECT\bottom + (tSIZE\cy * 3)
            DrawText_(wParam, TextL2$, Len(TextL2$), @tRECT, #DT_CENTER|#DT_VCENTER|#DT_SINGLELINE)
            
            ProcedureReturn 1
         EndIf
         
      Case #WM_LBUTTONDOWN
         
         If CountGadgetItems(#LIG_FILES) ; if the list is not empty
            
            tPOINT\x = WindowMouseX(#WIN_MAIN)
            tPOINT\y = WindowMouseY(#WIN_MAIN)
            
            MapWindowPoints_(WindowID(#WIN_MAIN), hwnd, @tPOINT, 1)
            
            tHitInfo\pt\x = tPOINT\x
            tHitInfo\pt\y = tPOINT\y
            
            iItemCurrent = SendMessage_(hWnd, #LVM_SUBITEMHITTEST,0,@tHitInfo)
            
            SetActiveGadget(#LIG_FILES)
            
            SetGadgetState(#LIG_FILES, iItemCurrent)
            
            SendMessage_(WindowID(#WIN_MAIN), #WM_SHOW_THUMB, iItemCurrent, 0)
            
         EndIf
         
      Case #WM_RBUTTONDOWN
         
         If CountGadgetItems(#LIG_FILES) ; if the list is not empty
            
            tPOINT\x = WindowMouseX(#WIN_MAIN)
            tPOINT\y = WindowMouseY(#WIN_MAIN)
            
            MapWindowPoints_(WindowID(#WIN_MAIN), hwnd, @tPOINT, 1)
            
            tHitInfo\pt\x = tPOINT\x
            tHitInfo\pt\y = tPOINT\y
            
            iItemCurrent = SendMessage_(hWnd, #LVM_SUBITEMHITTEST,0,@tHitInfo)
            
            SetActiveGadget(#LIG_FILES)
            
            
            If iItemCurrent <> - 1 ; if I haven't clicked in a empty area of the gadget
               iCount = CountCheckedItems(#LIG_FILES)
               
               If iCount
                  CreatePopupMenu(#MNU_POPUP)
                  MenuItem(#MNI_POP_MOVE_TO_TOP, "Move selected items (" + Str(iCount) + ") to TOP")
                  MenuItem(#MNI_POP_MOVE_TO_BOTTOM, "Move selected items (" + Str(iCount) + ") to BOTTOM")
                  MenuBar()
                  MenuItem(#MNI_POP_DELETE, "Remove selected items (" + Str(iCount) + ")"  + Chr(9) + "[Del]")
               Else
                  iItemid = GetGadgetItemData(#LIG_FILES, iItemCurrent)
                  
                  HE(FindElement(iItemID), "FindElement() failed.")
                  
                  CreatePopupMenu(#MNU_POPUP)
                  
                  MenuItem(#MNI_POP_MOVE_TO_TOP, "Move " + s_lsItem()\FileName$ + " to TOP")
                  MenuItem(#MNI_POP_MOVE_TO_BOTTOM, "Move " + s_lsItem()\FileName$ + " to BOTTOM")
                  MenuBar()
                  MenuItem(#MNI_POP_DELETE, "Remove " + s_lsItem()\FileName$ + Chr(9) + "[Del]")
                  
                  SetGadgetState(#LIG_FILES, iItemCurrent)
               EndIf
               
               MenuItem(#MNI_POP_CLOSE, "Close")
               
               SetGadgetState(#LIG_FILES, iItemCurrent)
               
               SendMessage_(WindowID(#WIN_MAIN), #WM_SHOW_THUMB, iItemCurrent, 0)
               
               DisplayPopupMenu(#MNU_POPUP, WindowID(#WIN_MAIN))
               
               FreeMenu(#MNU_POPUP) ; fix 05/05/09
               
            EndIf
            
            ProcedureReturn 1
         EndIf
         
   EndSelect
   
   ProcedureReturn CallWindowProc_(*oldWinProc, hWnd, iMessage, wParam, lParam)
   
EndProcedure

Procedure WinResize()
   Shared s_Options, s_flgResizing
   Protected iListGadgetWidth, iListGadgetHeight
   Protected iStartX, iStartY, iNewWidth, flgBefore
   Protected hWin = WindowID(#WIN_MAIN)
   
   s_flgResizing = #True
   
   iListGadgetWidth = WindowWidth(#WIN_MAIN) - #AREA_RIGHT_WIDTH - 15
   iListGadgetHeight = WindowHeight(#WIN_MAIN) - MenuHeight() - #AREA_BOTTOM_HEIGHT - 15 - StatusBarHeight(#STB_STATUS)
   
   flgBefore = IIF(Bool(iListGadgetWidth > GadgetWidth(#LIG_FILES)), #True, #False)
   
   If flgBefore
      ResizeGadget(#LIG_FILES, #PB_Ignore, #PB_Ignore, iListGadgetWidth, iListGadgetHeight)
   EndIf
   
   iNewWidth = iListGadgetWidth - #RESOLUTION_COLUMN_WIDTH - #FILESIZE_COLUMN_WIDTH - (IsScrollBarVisible(GadgetID(#LIG_FILES), #WS_VSCROLL) * GetSystemMetrics_(#SM_CXVSCROLL)) - GetSystemMetrics_(#SM_CXBORDER) * 4
   
   SetGadgetItemAttribute(#LIG_FILES, #Null, #PB_ListIcon_ColumnWidth, iNewWidth, 0)
   
   SetGadgetItemAttribute(#LIG_FILES, #Null, #PB_ListIcon_ColumnWidth, #FILESIZE_COLUMN_WIDTH , 1)
   
   If Not flgBefore
      ResizeGadget(#LIG_FILES, #PB_Ignore, #PB_Ignore, iListGadgetWidth, iListGadgetHeight)
   EndIf
   
   iStartY = iListGadgetHeight
   
   ResizeGadget(#BTN_BUILD_PDF, #PB_Ignore, iListGadgetHeight + 27, #PB_Ignore, #PB_Ignore)
   
   If CountGadgetItems(#LIG_FILES) = 0
      InvalidateRect_(hWin, #Null, #False)
   EndIf
   
   ResizeGadget(#TXT_JPG_STORING_MODE, WindowWidth(#WIN_MAIN) - 160, WindowHeight(#WIN_MAIN) - 65, #PB_Ignore, #PB_Ignore)
   If s_Options\flgJpgNoRecompression
      SetGadgetText(#TXT_JPG_STORING_MODE, "JPG storing is enabled")
   Else
      SetGadgetText(#TXT_JPG_STORING_MODE, "")
   EndIf
   
   ;;; No longer needed with PB 5.73 using SmartWindowRefresh()
   ;;; UpdateWindow_(hWin)
   
   If (s_Options\flgShowThumbs)
      RefreshThumbnail()
   EndIf
   
   s_flgResizing = #False
   
EndProcedure

Procedure CopyItem (*dest.T_ITEM_DATA, *source.T_ITEM_DATA)
   
   *dest\iItemID = *source\iItemID
   *dest\FullFileName$ = *source\FullFileName$
   *dest\FileName$ = *source\FileName$
   *dest\iResX = *source\iResX
   *dest\iResY = *source\iResY
   *dest\iFileSize = *source\iFileSize
   *dest\nImage = *source\nImage
   *dest\flgStatus = *source\flgStatus
   
EndProcedure

Procedure MoveItemToList (iItemID, iItemSource, iItemDest)
   Shared s_lsItem()
   Protected iCount, *lstPtr, tItem.T_ITEM_DATA
   
   ; copy item data
   
   ResetList(s_lsItem())
   
   SelectElement(s_lsItem(), iItemSource)
   
   CopyItem(@tItem, s_lsItem())
   
   ; save pointer to its original position
   
   *lstPtr = s_lsItem()
   
   
   If iItemDest = ListSize(s_lsItem())
      ; select destination item
      
      SelectElement(s_lsItem(), iItemDest - 1)
      
      ; add the new one one place after
      
      AddElement(s_lsItem())
      
   Else
      ; select destination item
      
      SelectElement(s_lsItem(), iItemDest)
      
      ; insert the new one one place before
      
      InsertElement(s_lsItem())
      
   EndIf
   
   
   CopyItem(s_lsItem(), @tItem)
   
   ; delete the original element
   
   ChangeCurrentElement(s_lsItem(), *lstPtr)
   
   DeleteElement(s_lsItem())
   
   ; move is complete !
   
EndProcedure

Procedure MoveItemToGadget (iItemID, iItemSource, iItemDest)
   Shared s_Options, s_lsItem()
   Protected Item$, iOffset, flgStatus
   
   flgStatus = GetGadgetItemState(#LIG_FILES, iItemSource)
   
   RemoveGadgetItem(#LIG_FILES, iItemSource)
   
   iOffset = IIF(Bool(iItemSource < iItemDest), -1, 0)
   
   HE(FindElement(iItemID), "FindElement() failed.")
   
   If s_Options\flgShowFullPath
      Item$ = s_lsItem()\FullFileName$ + Chr(10)
   Else
      Item$ = s_lsItem()\FileName$ + Chr(10)
   EndIf
   
   If s_lsItem()\iResX = 0
      Item$ + "? x ?" + Chr(10)
   Else
      Item$ + Str(s_lsItem()\iResX) + " x " + Str(s_lsItem()\iResY) + Chr(10)
   EndIf
   
   If s_Options\flgShowSizeCondensed
      Item$ + FormatQuadWithByteSuffix(s_lsItem()\iFileSize, #False, 1)
   Else
      Item$ + FormatQuad (s_lsItem()\iFileSize)
   EndIf
   
   AddGadgetItem(#LIG_FILES, iItemDest + iOffset, Item$)
   
   SetGadgetItemState(#LIG_FILES, iItemDest + iOffset, flgStatus)
   
   SetGadgetItemData(#LIG_FILES, iItemDest + iOffset, s_lsItem()\iItemID)
   
EndProcedure

Procedure ItemDroppedOuter (iItemDragged, iOuterPos)
   Protected iItemID, iItemCurrent
   
   iItemID = GetGadgetItemData(#LIG_FILES, iItemDragged)
   
   If iOuterPos = #ITEM_TOP
      iItemCurrent = 0
   Else
      iItemCurrent = CountGadgetItems(#LIG_FILES)
   EndIf
   
   MoveItemToGadget (iItemID, iItemDragged, iItemCurrent)
   
   MoveItemToList (iItemID, iItemDragged, iItemCurrent)
   
EndProcedure

Procedure ItemDropped (iItemDragged, iItemCurrent)
   
   Protected iItemID
   
   If iItemDragged <> iItemCurrent
      iItemID = GetGadgetItemData(#LIG_FILES, iItemDragged)
      
      MoveItemToGadget (iItemID, iItemDragged, iItemCurrent)
      
      MoveItemToList (iItemID, iItemDragged, iItemCurrent)
      
   EndIf
   
EndProcedure

Procedure AddItemToList (*tItem.T_ITEM_DATA)
   Shared s_TotFilesSize, s_lsItem()
   
   LastElement(s_lsItem())
   
   HE(AddElement(s_lsItem()), "AddElement failed.")
   
   CopyItem (s_lsItem(), *tItem)
   
   s_TotFilesSize + s_lsItem()\iFileSize
   
EndProcedure

Procedure SetItemStatus (iItemCurrent)
   Shared s_lsItem()
   Protected iItemID = GetGadgetItemData(#LIG_FILES, iItemCurrent)
   
   HE(FindElement(iItemID), "FindElement() failed.")
   
   s_lsItem()\flgStatus = GetGadgetItemState(#LIG_FILES, iItemCurrent)
   
EndProcedure

Procedure AddItemToListView (*tItem.T_ITEM_DATA)
   Shared s_Options
   Protected Item$
   
   If s_Options\flgShowFullPath
      Item$ = *tItem\FullFileName$ + Chr(10)
   Else
      Item$ = *tItem\FileName$ + Chr(10)
   EndIf
   
   If *tItem\iResX = 0
      Item$ + "? x ?" + Chr(10)
   Else
      Item$ + Str(*tItem\iResX) + " x " + Str(*tItem\iResY) + Chr(10)
   EndIf
   
   If s_Options\flgShowSizeCondensed
      Item$ + FormatQuadWithByteSuffix(*tItem\iFileSize, #False, 1)
   Else
      Item$ + FormatQuad (*tItem\iFileSize)
   EndIf
   
   AddGadgetItem(#LIG_FILES, -1, Item$)
   
   SetGadgetItemState(#LIG_FILES, CountGadgetItems(#LIG_FILES) - 1, *tItem\flgStatus)
   
   SetGadgetItemData(#LIG_FILES, CountGadgetItems(#LIG_FILES) - 1, *tItem\iItemID)
   
   UpdateNumFiles()
   
EndProcedure

Procedure ClearItemsData()
   Shared s_Options, s_TotFilesSize, s_UniqueID, s_lsItem()
   
   ; free thumbnails data
   If s_Options\flgPreprocessing
      ForEach s_lsItem()
         FreeImage(s_lsItem()\nImage)
      Next
   EndIf
   
   ; free list
   ClearList(s_lsItem())
   
   s_TotFilesSize = 0
   
   s_UniqueID = 0
EndProcedure

Procedure SortListView()
   Shared s_Options, s_lsItem()
   Protected iOffset
   
   If s_Options\flgSortIncludingPath
      iOffset = OffsetOf(T_ITEM_DATA\FullFileName$)
   Else
      iOffset = OffsetOf(T_ITEM_DATA\FileName$)
   EndIf
   
   SortStructuredList(s_lsItem(), #PB_Sort_Ascending | #PB_Sort_NoCase, iOffset, #PB_String)
EndProcedure

Procedure EnableHeaderGadgets (flgStatus)
   DisableGadget(#CMB_HEADER_WHAT, Bool(Not flgStatus))
   DisableGadget(#CMB_HEADER_SIZE, Bool(Not flgStatus))
   DisableGadget(#CMB_HEADER_ALIGN, Bool(Not flgStatus))
   DisableGadget(#CMB_HEADER_WHERE, Bool(Not flgStatus))
EndProcedure

Procedure SetBuildButtonState()
   Protected flgStatus = Bool(CountGadgetItems(#LIG_FILES) = 0)
   
   DisableGadget(#BTN_BUILD_PDF, flgStatus)
   DisableMenuItem(#MNU_WIN_MAIN, #MNI_CLEAR, flgStatus)
   DisableMenuItem(#MNU_WIN_MAIN, #MNI_SORT_NOW, flgStatus)
   DisableMenuItem(#MNU_WIN_MAIN, #MNI_SELECT_ALL, flgStatus)
   DisableMenuItem(#MNU_WIN_MAIN, #MNI_DESELECT_ALL, flgStatus)
   
   If flgStatus
      CloseThumbnail()
   Else
      If GetGadgetState(#LIG_FILES) <> -1
         ShowThumbnail(GetGadgetState(#LIG_FILES))
      Else
         CloseThumbnail()
      EndIf
   EndIf
   
EndProcedure

Procedure.i ProcessFile (*tItem.T_ITEM_DATA)
   
   Protected hImage, nImage, iFileSize, i
   Protected fDestWidth.f, fDestHeight.f, fThumbWidth.f, fThumbHeight.f
   Protected ImageFile$
   
   fThumbWidth = ThumbSize()
   fThumbHeight = ThumbSize()
   
   ImageFile$ = *tItem\FullFileName$
   
   iFileSize = FileSize(ImageFile$)
   
   ; check if file exist
   If  iFileSize = -1
      ProcedureReturn #False
   EndIf
   
   *tItem\iFileSize = iFileSize
   
   
   ; try to load the image
   i = GdiPlusLoadImage (@hImage, ImageFile$)
   
   ; check if image is loaded successfully
   If (i = #GDIPLUS_UNKNOWN_IMAGE_FORMAT) Or (i = #GDIPLUS_INVALID_PARAMETER)
      ProcedureReturn #False ; invalid file type, skipping
   EndIf
   
   ; check for other errors
   HE(Bool(i = 0), "GDI+ unexpected error -> " + Str(i))
   
   i = GdipGetImageWidth(hImage, @*tItem\iResX)
   HE(Bool(i = 0), "GDI+ unexpected error -> " + Str(i))
   
   i = GdipGetImageHeight(hImage, @*tItem\iResY)
   HE(Bool(i = 0), "GDI+ unexpected error -> " + Str(i))
   
   
   
   ; try to convert the image drawing it on a PB image
   i = GdiPlusImageToPbImage(hImage, @nImage)
   
   ; check if image is successfully converted
   If (i = #GDIPLUS_WIN32_ERROR)
      ProcedureReturn #False ; probably the image is corrupted
   EndIf
   
   ; check for other errors
   HE(Bool(i = 0), "GDI+ unexpected error -> " + Str(i))
   
   
   i = GdipDisposeImage(hImage)
   HE(Bool(i = 0), "GDI+ unexpected error -> " + Str(i))
   
   ; if need a shrink to fit in the preview ...
   If (*tItem\iResX > ThumbSize()) Or (*tItem\iResY > ThumbSize())
      
      Repeat
         
         fDestWidth = fThumbWidth
         fDestHeight = *tItem\iResY / (*tItem\iResX / fThumbWidth)
         If fDestHeight <= fThumbHeight
            Break
         EndIf
         
         fDestHeight = fThumbHeight
         fDestWidth = *tItem\iResX / (*tItem\iResY / fThumbHeight)
         If fDestWidth <= fThumbWidth
            Break
         EndIf
         
         HE(#False, "Internal error!")
         Break
         
      ForEver
      
      ResizeImage(nImage, fDestWidth, fDestHeight, #PB_Image_Smooth)
   EndIf
   
   *tItem\nImage = nImage
   
   ProcedureReturn #True
   
EndProcedure


Procedure RefreshListView()
   Shared s_lsItem()
   
   ClearGadgetItems(#LIG_FILES)
   
   ForEach s_lsItem()
      AddItemToListView(s_lsItem())
   Next
EndProcedure


Procedure AddFilesDropped (List lsFilesDropped$())
   Shared s_Options, s_UniqueID, s_lsItem()
   Protected iAdded, iEvent, iFilesDropped, flgAbort
   Protected tItem.T_ITEM_DATA, tWinProcess.T_WINDOW_RECT
   
   CloseThumbnail()
   
   tWinProcess\X = 0
   tWinProcess\Y = 0
   tWinProcess\W = 450
   tWinProcess\H = 70
   
   
   ; we loop through the list of the dropped files
   
   iFilesDropped = ListSize(lsFilesDropped$())
   
   ResetList(lsFilesDropped$())
   
   HE(OpenWindow(#WIN_PROCESS, tWinProcess\X, tWinProcess\Y, tWinProcess\W, tWinProcess\H, "Processing " + FormatQuad(ListSize(lsFilesDropped$())) + " files ...", #PB_Window_SystemMenu | #PB_Window_Invisible, WindowID(#WIN_MAIN)), "Open window failed For #WIN_PROCESS.")
   
   CenterWindowToWindow(WindowID(#WIN_PROCESS), WindowID(#WIN_MAIN))
   
   ProgressBarGadget(#PRB_PROCESS, 5, 10, WindowWidth(#WIN_PROCESS) - 10, 20, 0, VISTA_PROGBAR_FIX(iFilesDropped))
   
   ButtonGadget(#BTN_PROCESS_CANCEL, (tWinProcess\W - 100) / 2, tWinProcess\H - 35, 100, 30, "Cancel")
   
   DisableWin(#WIN_MAIN)
   
   PumpMessages()
   
   LastElement(s_lsItem())
   
   ForEach lsFilesDropped$()
      
      tItem\iItemID = s_UniqueID + iAdded
      tItem\FullFileName$ = lsFilesDropped$()
      tItem\FileName$ = GetFilePart(lsFilesDropped$())
      tItem\iFileSize = FileSize(tItem\FullFileName$)
      tItem\iResX = 0
      tItem\iResY = 0
      
      ; we add it to the global list of items
      
      AddItemToList (@tItem)
      
      AddItemToListView (@tItem)
      
      iAdded + 1
      
      WinResize()
      
      ; update the progress bar after the file is processed
      
      SetVistaProgressBarValue(#PRB_PROCESS, iAdded)
      
      Repeat
         iEvent = WindowEvent()
         
         If EventWindow() = #WIN_PROCESS
            
            If iEvent = #PB_Event_CloseWindow
               flgAbort = #True
            EndIf
            
            If EventGadget() = #BTN_PROCESS_CANCEL
               flgAbort = #True
            EndIf
            
         EndIf
         
         Delay(0)
         
      Until iEvent = 0
      
      If flgAbort : Break: EndIf
      
   Next
   
   SetVistaProgressBarValue(#PRB_PROCESS, iFilesDropped)
   
   PumpMessages()
   
   Delay(250) ; if only a file or so are dropped
   
   FreeGadget(#PRB_PROCESS)
   
   EnableWin(#WIN_MAIN)
   
   CloseWindow(#WIN_PROCESS)
   
   ResetList(s_lsItem())
   
   ; if sort always is enabled
   If s_Options\flgSortAfterDrop And iAdded
      SortListView()
      
      RefreshListView()
      
      SetGadgetState(#LIG_FILES, 0)
   EndIf
   
   
   ; we update the listview contents
   
   ProcedureReturn iAdded ; return the number of items added
   
EndProcedure


Procedure.i ProcesFilesDropped (List lsFilesDropped$(), List lsFilesSkipped$(), *flgCanceled.Integer)
   Shared s_Options, s_UniqueID, s_lsItem()
   Protected iAdded, iProcessed, iEvent, iFilesDropped, flgAbort
   Protected tItem.T_ITEM_DATA, tWinProcess.T_WINDOW_RECT
   
   CloseThumbnail()
   
   ; we loop through the list of the dropped files
   
   iFilesDropped = ListSize(lsFilesDropped$())
   
   ClearList(lsFilesSkipped$())
   
   tWinProcess\X = 0
   tWinProcess\Y = 0
   tWinProcess\W = 450
   tWinProcess\H = 70
   
   HE(OpenWindow(#WIN_PROCESS, tWinProcess\X, tWinProcess\Y, tWinProcess\W, tWinProcess\H, "Processing " + FormatQuad(ListSize(lsFilesDropped$())) + " files ...", #PB_Window_SystemMenu | #PB_Window_Invisible, WindowID(#WIN_MAIN)), "Open window failed For #WIN_PROCESS.")
   
   CenterWindowToWindow(WindowID(#WIN_PROCESS), WindowID(#WIN_MAIN))
   
   ProgressBarGadget(#PRB_PROCESS, 5, 10, WindowWidth(#WIN_PROCESS) - 10, 20, 0, VISTA_PROGBAR_FIX(iFilesDropped))
   
   ButtonGadget(#BTN_PROCESS_CANCEL, (tWinProcess\W - 100) / 2, tWinProcess\H - 35, 100, 30, "Cancel")
   
   DisableWin(#WIN_MAIN)
   
   PumpMessages()
   
   flgAbort = #False
   
   *flgCanceled\i = #False
   
   LastElement(s_lsItem())
   
   ForEach lsFilesDropped$()
      
      SetActiveWindow(#WIN_PROCESS)
      
      iProcessed + 1
      
      tItem\iItemID = s_UniqueID + iAdded
      tItem\FullFileName$ = lsFilesDropped$()
      tItem\FileName$ = GetFilePart(lsFilesDropped$())
      
      If ProcessFile (@tItem) ; if the file is valid
         
         ; we add it to the global list of items
         
         AddItemToList (@tItem)
         
         AddItemToListView (@tItem)
         
         iAdded + 1
         
         WinResize()
      Else
         HE(AddElement(lsFilesSkipped$()), "AddElement failed.")
         lsFilesSkipped$() = tItem\FullFileName$
      EndIf
      
      
      ; update the progress bar after the file is processed
      
      SetVistaProgressBarValue(#PRB_PROCESS, iProcessed)
      
      Repeat
         iEvent = WindowEvent()
         
         If EventWindow() = #WIN_PROCESS
            
            If iEvent = #PB_Event_CloseWindow
               flgAbort = #True
            EndIf
            
            If EventGadget() = #BTN_PROCESS_CANCEL
               *flgCanceled\i = #True
               flgAbort = #True
            EndIf
            
         EndIf
         
      Until iEvent = 0
      
      If flgAbort : Break: EndIf
   Next
   
   SetVistaProgressBarValue(#PRB_PROCESS, iFilesDropped)
   
   PumpMessages()
   
   Delay(250) ; if only a file or so are dropped
   
   FreeGadget(#PRB_PROCESS)
   
   EnableWin(#WIN_MAIN)
   
   CloseWindow(#WIN_PROCESS)
   
   ResetList(s_lsItem())
   
   ; if sort always is enabled
   If s_Options\flgSortAfterDrop And iAdded
      SortListView()
      
      RefreshListView()
      
      SetGadgetState(#LIG_FILES, 0)
      
   EndIf
   
   PurgeMemory()
   
   ; we update the listview contents
   
   ProcedureReturn iAdded ; return the number of items added
   
EndProcedure

Procedure SetJpgQuality (iQuality)
   Protected Text$
   
   Select iQuality
      Case 1, 2, 3, 4
         Text$ = "Low"
      Case 5, 6, 7
         Text$ = "Medium"
      Case 8, 9, 10
         Text$ = "High"
   EndSelect
   
   Text$ + " ( " + Str(iQuality) + " )"
   
   SetGadgetState(#TRK_QUALITY, iQuality)
   SetGadgetText(#TXT_QUALITY_2, Text$)
EndProcedure

Procedure SetScaling (iScaling)
   Protected Text$
   
   If iScaling = 100
      Text$ = "Original size"
   Else
      Text$ = Str(iScaling) + " %"
   EndIf
   
   EnableImageResizeGadgets(iScaling)
   
   SetGadgetState(#TRK_SCALING, iScaling)
   SetGadgetText(#TXT_SCALING_2, Text$)
EndProcedure

Procedure.i SaveSkippedList (FileName$, List lsFilesSkipped$())
   Protected nFileNum, iBytes
   Protected flgResult
   
   flgResult = #False
   
   nFileNum = CreateFile(#PB_Any, FileName$)
   
   If nFileNum
      If ListWriteToTextFile(nFileNum, lsFilesSkipped$(), @iBytes) = #RET_OK
         flgResult = #True
      EndIf
      CloseFile(nFileNum)
   EndIf
   
   ProcedureReturn flgResult
   
EndProcedure


Procedure.i ShowSkippedFiles (List lsFilesSkipped$())
   
   Protected iEvent, flgExitLoop, iBaseY, iSkipped, nImage
   Protected tWinSkipped.T_WINDOW_RECT
   Protected OutFile$
   
   iSkipped = ListSize(lsFilesSkipped$())
   
   tWinSkipped\X = 0
   tWinSkipped\Y = 0
   tWinSkipped\W = 330
   tWinSkipped\H = 100
   
   DisableWin(#WIN_MAIN)
   DisableWin(#WIN_THUMB)
   
   HE(OpenWindow(#WIN_SKIPPED, tWinSkipped\X, tWinSkipped\Y, tWinSkipped\W, tWinSkipped\H, PROGRAM_TITLE() , #PB_Window_SystemMenu | #PB_Window_Invisible, WindowID(#WIN_MAIN)), "Open window failed For #WIN_SKIPPED.")
   
   CenterWindowToWindow(WindowID(#WIN_SKIPPED), WindowID(#WIN_MAIN))
   
   nImage = HE(CreateImage(#PB_Any, 32, 32), "CreateImage failed.")
   
   StartDrawing(ImageOutput(nImage))
   Box(0, 0, ImageWidth(nImage), ImageHeight(nImage), GetSysColor_(#COLOR_3DFACE))
   DrawImage(LoadIcon_(0, #IDI_EXCLAMATION), 0, 0)
   StopDrawing()
   
   ImageGadget(#IMG_SKIPPED_ICON, 10, 10, 32, 32, ImageID(nImage))
   
   iBaseY = 25
   TextGadget(#TXT_SKIPPED_1, 55, iBaseY, tWinSkipped\W - 55, 20, "Some files (" + Str(iSkipped) +") were skipped, format not supported.")
   
   ButtonGadget(#BTN_SKIPPED_OK, 10, tWinSkipped\H - 30, 100, 25, " OK ")
   ButtonGadget(#BTN_SKIPPED_SHOW, (tWinSkipped\W - 100 - 10), tWinSkipped\H - 30, 100, 25, " Export the list ")
   
   flgExitLoop = #False
   
   Repeat
      
      iEvent = WaitWindowEvent()
      
      Select iEvent
         Case #PB_Event_Gadget
            
            Select EventGadget()
               Case #BTN_SKIPPED_SHOW
                  
                  HideWin(#WIN_SKIPPED)
                  
                  CenterRequesterPositionToWindow(WindowID(#WIN_MAIN))
                  
                  OutFile$ = AskSkippedListName()
                  
                  If Len(OutFile$)
                     If SaveSkippedList(OutFile$, lsFilesSkipped$()) = #False
                        MessageBox_(WindowID(#WIN_SKIPPED), "Error saving the skipped files list.", PROGRAM_TITLE(), #MB_OK | #MB_ICONEXCLAMATION | #MB_APPLMODAL)
                     Else
                        flgExitLoop = #True
                     EndIf
                  EndIf
                  
                  ShowWin(#WIN_SKIPPED)
                  
               Case #BTN_SKIPPED_OK
                  flgExitLoop = #True
            EndSelect
      EndSelect
      
   Until (iEvent = #PB_Event_CloseWindow) Or flgExitLoop
   
   CloseWindow(#WIN_SKIPPED)
   
   FreeImage(nImage)
   
   EnableWin(#WIN_THUMB)
   EnableWin(#WIN_MAIN)
   
EndProcedure

Procedure ShowAbout()
   
   Protected iEvent, flgExitLoop, iBaseY, nFontText
   Protected nPbLogoImage, nImageTitle, X, Y
   Protected tWinAbout.T_WINDOW_RECT
   Protected qMemAvail.q
   
   
   tWinAbout\X = 0
   tWinAbout\Y = 0
   tWinAbout\W = 300
   tWinAbout\H = 300
   
   nFontText = LoadFont(#PB_Any,"Comic Sans MS", 10)
   
   SetGadgetFont(#PB_Default, FontID(nFontText))
   
   DisableWin(#WIN_MAIN)
   
   HE(OpenWindow(#WIN_ABOUT, tWinAbout\X, tWinAbout\Y, tWinAbout\W, tWinAbout\H, PROGRAM_TITLE_FULL() , #PB_Window_SystemMenu | #PB_Window_Invisible, WindowID(#WIN_MAIN)), "Open window failed For #WIN_ABOUT.")
   
   CenterWindowToWindow(WindowID(#WIN_ABOUT), WindowID(#WIN_MAIN))
   
   nImageTitle = CatchImage(#PB_Any, ?lbl_i2pdf_logo)
   
   ImageGadget(#IMG_ABOUT_TITLE, 0, 0, tWinAbout\W, 60, ImageID(nImageTitle))
   
   
   iBaseY = 70
   TextGadget(#TXT_ABOUT_1, 0, iBaseY, tWinAbout\W, 24, "(c) 2009-2023 by LMG Software", #PB_Text_Center)
   
   iBaseY + 35
   TextGadget(#TXT_ABOUT_2, 0, iBaseY, tWinAbout\W, 24, "This software is FREEWARE", #PB_Text_Center)
   
   iBaseY + 24
   TextGadget(#TXT_ABOUT_3, 0, iBaseY, tWinAbout\W, 24, "and developed using the amazing", #PB_Text_Center)
   
   iBaseY + 34
   nPbLogoImage = CatchImage(#PB_Any, ?lbl_purebasic_logo)
   ButtonImageGadget(#BTN_PB_LOGO, (tWinAbout\W - ImageWidth(nPbLogoImage) - 5) / 2, iBaseY, ImageWidth(nPbLogoImage) + 5, ImageHeight(nPbLogoImage) + 5, ImageID(nPbLogoImage))
   
   
   iBaseY + 62
   GetFreePhysicalMemory(@qMemAvail)
   TextGadget(#TXT_ABOUT_4, 0, iBaseY, tWinAbout\W, 30, "Free memory: " + FormatQuad(qMemAvail / 1024) + " KB", #PB_Text_Center)
   
   
   SetGadgetFont(#PB_Default, #PB_Default)
   
   ButtonGadget(#BTN_ABOUT_OK, (tWinAbout\W - 100) / 2, tWinAbout\H - 40, 100, 30, "Great !")
   
   
   flgExitLoop = #False
   
   Repeat
      
      iEvent = WaitWindowEvent()
      
      Select iEvent
         Case #PB_Event_Gadget
            
            Select EventGadget()
               Case #BTN_PB_LOGO
                  RunProgram("http://www.purebasic.com/")
                  
               Case #BTN_ABOUT_OK
                  flgExitLoop = #True
                  
            EndSelect
      EndSelect
      
   Until (iEvent = #PB_Event_CloseWindow) Or flgExitLoop
   
   CloseWindow(#WIN_ABOUT)
   
   FreeImage(nImageTitle)
   FreeImage(nPbLogoImage)
   FreeFont(nFontText)
   
   EnableWin(#WIN_MAIN)
   
EndProcedure

Procedure EnableImageQualityGadgets (flgStatus)
   DisableGadget(#TXT_QUALITY_1, Bool(Not flgStatus))
   DisableGadget(#TRK_QUALITY, Bool(Not flgStatus))
   DisableGadget(#TXT_QUALITY_2, Bool(Not flgStatus))
EndProcedure

Procedure EnableImageResizeGadgets (iResize)
   If iResize = 100
      DisableGadget(#STG_RESIZE_IF, 1)
   Else
      DisableGadget(#STG_RESIZE_IF, 0)
   EndIf
EndProcedure

Procedure.s PaperSizeToMM_STR (Format$)
   Protected iWidthMM, iHeightMM
   Protected kPtIn.f = 72.0 / 25.4
   
   iWidthMM = ValF(StringField(Format$, 1, ",")) / kPtIn
   iHeightMM = ValF(StringField(Format$, 2, ",")) / kPtIn
   
   ProcedureReturn " (" + Str(iWidthMM) + " x " + Str(iHeightMM) + " mm)"
EndProcedure

Procedure.s PaperSizeToInches_STR (Format$)
   Protected fWidth.f, fHeight.f
   Protected kPtIn.f = 72.0
   
   fWidth = ValF(StringField(Format$, 1, ",")) / kPtIn
   fHeight = ValF(StringField(Format$, 2, ",")) / kPtIn
   
   ProcedureReturn " (" + StrF(fWidth,1) + Chr(34) + " x " + StrF(fHeight,1) + Chr(34) + ")"
EndProcedure


Procedure LoadPaperFormats (nGadget)
   AddGadgetItem(nGadget, -1, "A4" + PaperSizeToMM_str(#PDF_PAGE_FORMAT_A4))
   AddGadgetItem(nGadget, -1, "A3" + PaperSizeToMM_str(#PDF_PAGE_FORMAT_A3))
   AddGadgetItem(nGadget, -1, "A2" + PaperSizeToMM_str(#PDF_PAGE_FORMAT_A2))
   AddGadgetItem(nGadget, -1, "US Letter" + PaperSizeToInches_STR(#PDF_PAGE_FORMAT_LETTER))
   AddGadgetItem(nGadget, -1, "US Legal" + PaperSizeToInches_STR(#PDF_PAGE_FORMAT_LEGAL))
   AddGadgetItem(nGadget, -1, "Paperless")
EndProcedure

Procedure LoadPdfZoom (nGadget)
   AddGadgetItem(nGadget, -1, "Fit page")
   AddGadgetItem(nGadget, -1, "Fit width")
   AddGadgetItem(nGadget, -1, "True size")
   AddGadgetItem(nGadget, -1, "Full screen")
EndProcedure

Procedure LoadHeaderSize (nGadget)
   Protected k
   
   For k = #HEADER_SIZE_MIN To #HEADER_SIZE_MAX
      AddGadgetItem(nGadget, -1, Str(k))
   Next
EndProcedure

Procedure LoadHeaderAlignment (nGadget)
   AddGadgetItem(nGadget, -1, "Left")
   AddGadgetItem(nGadget, -1, "Center")
   AddGadgetItem(nGadget, -1, "Right")
EndProcedure

Procedure LoadHeaderPosition (nGadget)
   AddGadgetItem(nGadget, -1, "Top")
   AddGadgetItem(nGadget, -1, "Bottom")
EndProcedure

Procedure LoadHeaderWhat (nGadget)
   AddGadgetItem(nGadget, -1, "Filename")
   AddGadgetItem(nGadget, -1, "Path + Filename")
EndProcedure

Procedure.i CheckSettings()
   ProcedureReturn #True
EndProcedure

Procedure.s PrefFileName()
   Protected ExeDir$
   
   ExeDir$ = GetPathPart(ProgramFilename())
   
   ProcedureReturn ExeDir$ + #PROGRAM_NAME$ + ".ini"   
EndProcedure

Procedure.i SaveSettings()
   Shared s_Options, s_LastWinSize
   Protected PrefsFile$
   
   PrefsFile$ = PrefFileName()
   
   If Not CreatePreferences(PrefsFile$)
      ProcedureReturn #False
   EndIf
   
   With s_Options
      
      PreferenceComment(LSet("", 80, "*"))
      PreferenceComment("Configuration file for " + PROGRAM_TITLE())
      PreferenceComment("You should not manually edit this file, unless you know what you are doing, but")
      PreferenceComment("you can safely delete it if you want to restore all settings to factory defaults.")
      PreferenceComment(LSet("", 80, "*"))
      
      PreferenceGroup("PDF creation")
      
      WritePreferenceInteger("JpgCompression", \flgJpgCompression)
      
      If \flgJpgCompression
         WritePreferenceInteger("JpgQuality", \iJpgQuality)
      EndIf
      
      WritePreferenceInteger("ImageScaling", \iImageScaling)
      WritePreferenceInteger("ResizeOnlyIfWider", \iResizeOnlyIfWider)
      WritePreferenceInteger("EnlargeToFit", \flgEnlargeToFit)
      WritePreferenceInteger("PaperOrientation", \iPaperOrientation)
      WritePreferenceInteger("PaperFormat", \iPaperFormat)
      WritePreferenceInteger("PdfStartup", \iPdfStartup)
      WritePreferenceInteger("KeepAspect", \flgKeepAspect)
      
      WritePreferenceInteger ("DisplayHeader", \flgDisplayHeader)
      WritePreferenceInteger ("HeaderFontSize", \iHeaderFontSize)
      WritePreferenceInteger ("HeaderDisplayWhat", \iHeaderDisplayWhat)
      WritePreferenceInteger ("HeaderDisplayWhere", \iHeaderDisplayWhere)
      WritePreferenceInteger ("HeaderDisplayAlign", \iHeaderDisplayAlign)
      
      If s_Options\flgSaveAuthor
         WritePreferenceString ("DocAuthor", \DocAuthor$)
      EndIf
      
      If s_Options\flgSaveTitle
         WritePreferenceString ("DocTitle", \DocTitle$)
      EndIf
      
      WritePreferenceInteger("SaveAuthor", \flgSaveAuthor)
      WritePreferenceInteger("SaveTitle", \flgSaveTitle)
      
      
      PreferenceGroup("Thumbnail")
      
      WritePreferenceInteger ("ThumbLocked", \flgThumbLocked)
      WritePreferenceInteger ("ThumbSide", \iThumbSide)
      WritePreferenceInteger ("ThumbSize", \iThumbSize)
      WritePreferenceInteger ("ThumbColor", \iThumbColor)
      
      PreferenceGroup("General")
      
      WritePreferenceInteger("StayOnTop", \flgStayOnTop)
      WritePreferenceInteger("RememberWinPos", \flgRememberWinPos)
      WritePreferenceInteger("ShowThumbs", \flgShowThumbs)
      
      If \flgRememberWinPos
         If GetWindowState(#WIN_MAIN) = #PB_Window_Minimize
            WritePreferenceInteger("X", s_LastWinSize\X)
            WritePreferenceInteger("Y", s_LastWinSize\Y)
            WritePreferenceInteger("W", s_LastWinSize\W)
            WritePreferenceInteger("H", s_LastWinSize\H)
         Else
            WritePreferenceInteger("X", WindowX(#WIN_MAIN))
            WritePreferenceInteger("Y", WindowY(#WIN_MAIN))
            WritePreferenceInteger("W", WindowWidth(#WIN_MAIN))
            WritePreferenceInteger("H", WindowHeight(#WIN_MAIN))
         EndIf
      EndIf
      
      WritePreferenceInteger("RememberLastDirectory", \flgRememberLastDirectory)
      WritePreferenceInteger("SaveSameDirAsFiles", \flgSaveSameDirAsFiles)
      
      If \flgRememberLastDirectory And Len(\LastDirectory$)
         WritePreferenceString("LastDirectory", \LastDirectory$)
      EndIf
      
      WritePreferenceInteger("ClearBeforeDrop", \flgClearBeforeDrop)
      WritePreferenceInteger("ShowOptions", \flgShowOptions)
      WritePreferenceInteger("ShowFullPath", \flgShowFullPath)
      WritePreferenceInteger("SortAfterDrop", \flgSortAfterDrop)
      WritePreferenceInteger("OpenPdf", \flgOpenPdf)
      WritePreferenceInteger("OpenDir", \flgOpenDir)
      WritePreferenceInteger("Confirmations", \flgConfirmations)
      WritePreferenceInteger("MemoryLoad", \flgMemoryLoad)
      WritePreferenceInteger("ShowSizeCondensed", \flgShowSizeCondensed)
      WritePreferenceInteger("SortIncludingPath", \flgSortIncludingPath)
      WritePreferenceInteger("Preprocessing", \flgPreprocessing)
      WritePreferenceInteger("DocTitleFromFileName", \flgDocTitleFromFileName)
      WritePreferenceInteger("DeleteLogIfSuccessful", \flgDeleteLogIfSuccessful)
      WritePreferenceInteger("EnableSubFolders", \flgEnableSubFolders)
      WritePreferenceInteger("JpgNoRecompression", \flgJpgNoRecompression)
      
   EndWith
   
   ClosePreferences()
   
   ProcedureReturn #True
   
EndProcedure

Procedure.i LoadSettings()
   Shared s_Options
   Protected PrefsFile$
   Protected iRetVal
   
   PrefsFile$ = PrefFileName()
   
   iRetVal = OpenPreferences(PrefsFile$)
   
   With s_Options
      
      PreferenceGroup("PDF creation")
      
      \flgJpgCompression = ReadPreferenceIntegerEx ("JpgCompression", #True, 0, 1)
      \iJpgQuality = ReadPreferenceIntegerEx ("JpgQuality", 8, 1, 10)
      \iImageScaling = ReadPreferenceIntegerEx ("ImageScaling", 100, 1, 100)
      \iResizeOnlyIfWider = ReadPreferenceIntegerEx ("ResizeOnlyIfWider", 0, 0, 9999)
      \iPaperOrientation = ReadPreferenceIntegerEx ("PaperOrientation", #PAPER_PORTRAIT, 1, 2)
      \iPaperFormat = ReadPreferenceIntegerEx ("PaperFormat", #PAPER_FORMAT_PAPERLESS, #PAPER_FORMAT_A4, #PAPER_FORMAT_PAPERLESS)
      \iPdfStartup = ReadPreferenceIntegerEx ("PdfStartup", #PDF_ZOOM_FULLPAGE, 1, 4)
      \flgEnlargeToFit = ReadPreferenceIntegerEx ("EnlargeToFit", #False, 0, 1)
      \flgKeepAspect = ReadPreferenceIntegerEx ("KeepAspect", #True, 0, 1)
      
      \flgDisplayHeader = ReadPreferenceIntegerEx ("DisplayHeader", #False, 0, 1)
      \iHeaderFontSize = ReadPreferenceIntegerEx ("HeaderFontSize", 10, #HEADER_SIZE_MIN, #HEADER_SIZE_MAX)
      \iHeaderDisplayWhat = ReadPreferenceIntegerEx ("HeaderDisplayWhat", #HEADER_SHOW_NAME, #HEADER_SHOW_NAME, #HEADER_SHOW_FULL_NAME)
      \iHeaderDisplayWhere = ReadPreferenceIntegerEx ("HeaderDisplayWhere", #HEADER_TOP, #HEADER_TOP, #HEADER_BOTTOM)
      \iHeaderDisplayAlign = ReadPreferenceIntegerEx ("HeaderDisplayAlign", #HEADER_ALIGN_LEFT, #HEADER_ALIGN_LEFT, #HEADER_ALIGN_RIGHT)
      \DocAuthor$ = ReadPreferenceString ("DocAuthor", "")
      \DocTitle$ = ReadPreferenceString ("DocTitle", "")
      \flgSaveAuthor = ReadPreferenceIntegerEx ("SaveAuthor", #False, 0, 1)
      \flgSaveTitle = ReadPreferenceIntegerEx ("SaveTitle", #False, 0, 1)
      
      PreferenceGroup("Thumbnail")
      
      \flgThumbLocked = ReadPreferenceIntegerEx ("ThumbLocked", #True, 0, 1)
      \iThumbSize = ReadPreferenceIntegerEx ("ThumbSize", #THUMB_SIZE2, #THUMB_SIZE1, #THUMB_SIZE3)
      \iThumbSide = ReadPreferenceIntegerEx ("ThumbSide", #THUMB_RIGHT, #THUMB_LEFT, #THUMB_RIGHT)
      \iThumbColor = ReadPreferenceIntegerEx ("ThumbColor", #THUMB_BLACK, #THUMB_BLACK, #THUMB_WHITE)
      
      PreferenceGroup("General")
      
      \flgStayOnTop = ReadPreferenceIntegerEx ("StayOnTop", #False, 0, 1)
      \flgRememberWinPos = ReadPreferenceIntegerEx ("RememberWinPos", #False, 0, 1)
      \flgShowThumbs = ReadPreferenceIntegerEx ("ShowThumbs", #False, 0, 1)
      
      If \flgRememberWinPos
         \tWinPos\X = ReadPreferenceInteger("X", 0)
         \tWinPos\Y = ReadPreferenceInteger("Y", 0)
         \tWinPos\W = ReadPreferenceIntegerEx("W", #WIN_MIN_WIDTH, #WIN_MIN_WIDTH, #PB_Ignore)
         \tWinPos\H = ReadPreferenceIntegerEx("H", #WIN_MIN_HEIGHT, #WIN_MIN_HEIGHT, #PB_Ignore)
         
         If \tWinPos\X < 0 Or \tWinPos\X > GetDesktopWidth() Or \tWinPos\Y < 0 Or \tWinPos\Y > GetDesktopHeight()
            \tWinPos\X = 100
            \tWinPos\Y = 100
            \tWinPos\W = #WIN_MIN_WIDTH
            \tWinPos\H = #WIN_MIN_HEIGHT
         EndIf
      EndIf
      
      \flgRememberLastDirectory = ReadPreferenceIntegerEx ("RememberLastDirectory", #True, 0, 1)
      
      \flgSaveSameDirAsFiles = ReadPreferenceIntegerEx ("SaveSameDirAsFiles", #False, 0, 1)
      
      If \flgRememberLastDirectory
         \LastDirectory$ = ReadPreferenceString ("LastDirectory", "")
      Else
         \LastDirectory$ = ""
      EndIf
      
      If Len(\LastDirectory$)
         If IsDirThere(\LastDirectory$) = #False
            \LastDirectory$ = ""
         EndIf
      EndIf
      
      \flgClearBeforeDrop = ReadPreferenceIntegerEx ("ClearBeforeDrop", #False, 0, 1)
      \flgShowOptions = ReadPreferenceIntegerEx ("ShowOptions", #True, 0, 1)
      \flgShowFullPath = ReadPreferenceIntegerEx ("ShowFullPath",#False, 0, 1)
      \flgSortAfterDrop = ReadPreferenceIntegerEx ("SortAfterDrop", #True, 0, 1)
      \flgOpenPdf = ReadPreferenceIntegerEx ("OpenPdf",#False, 0, 1)
      \flgOpenDir = ReadPreferenceIntegerEx ("OpenDir",#True, 0, 1)
      \flgConfirmations = ReadPreferenceIntegerEx ("Confirmations", #True, 0, 1)
      \flgMemoryLoad = ReadPreferenceIntegerEx ("MemoryLoad", #True, 0, 1)
      \flgShowSizeCondensed = ReadPreferenceIntegerEx ("ShowSizeCondensed", #True, 0, 1)
      \flgSortIncludingPath = ReadPreferenceIntegerEx ("SortIncludingPath", #False, 0, 1)
      \flgPreprocessing = ReadPreferenceIntegerEx ("Preprocessing", #True, 0, 1)
      \flgDocTitleFromFileName = ReadPreferenceIntegerEx ("DocTitleFromFileName", #False, 0, 1)
      \flgDeleteLogIfSuccessful = ReadPreferenceIntegerEx ("DeleteLogIfSuccessful", #False, 0, 1)
      \flgEnableSubFolders = ReadPreferenceIntegerEx ("EnableSubFolders", #False, 0, 1)
      \flgJpgNoRecompression = ReadPreferenceIntegerEx ("JpgNoRecompression", #False, 0, 1)
   EndWith
   
   If iRetVal
      ClosePreferences()
   EndIf
   
   ProcedureReturn iRetVal
   
EndProcedure

Procedure UpdateSettings()
   Shared s_Options
   
   With s_Options
      ; pdf creation
      
      \flgJpgCompression = GetGadgetState(#CHK_COMPRESS)
      \iJpgQuality = GetGadgetState(#TRK_QUALITY)
      \iImageScaling = GetGadgetState(#TRK_SCALING)
      \iResizeOnlyIfWider = Val(GetGadgetText(#STG_RESIZE_IF))
      \flgEnlargeToFit = GetGadgetState(#CHK_ENLARGE_TO_FIT)
      
      If GetGadgetState(#OPT_PORTRAIT)
         \iPaperOrientation = #PAPER_PORTRAIT
      Else
         \iPaperOrientation = #PAPER_LANDSCAPE
      EndIf
      
      \iPaperFormat = GetGadgetState(#CMB_PAPER_FORMAT) + 1
      
      \iPdfStartup = GetGadgetState(#CMB_PDF_STARTUP) + 1
      
      \flgKeepAspect = GetGadgetState(#CHK_KEEP_ASPECT)
      
      
      ; text
      
      \flgDisplayHeader = GetGadgetState(#CHK_SHOW_HEADER)
      \iHeaderFontSize = GetGadgetState(#CMB_HEADER_SIZE) + #HEADER_SIZE_MIN
      \iHeaderDisplayWhat = GetGadgetState(#CMB_HEADER_WHAT) + 1
      \iHeaderDisplayWhere = GetGadgetState(#CMB_HEADER_WHERE) + 1
      \iHeaderDisplayAlign = GetGadgetState(#CMB_HEADER_ALIGN) + 1
      
      ; document info
      
      \DocAuthor$ = GetGadgetText(#STG_DOC_AUTHOR)
      \DocTitle$ = GetGadgetText(#STG_DOC_TITLE)
      \flgSaveAuthor = GetGadgetState(#CHK_SAVE_AUTHOR)
      \flgSaveTitle = GetGadgetState(#CHK_SAVE_TITLE)
      
      
      ; thumbnail
      
      \flgThumbLocked = GetGadgetState(#CHK_LOCKED_TO_MAIN)
      
      
      If GetGadgetState(#OPT_THUMB_SIZE_1)
         \iThumbSize = #THUMB_SIZE1
      EndIf
      
      If GetGadgetState(#OPT_THUMB_SIZE_2)
         \iThumbSize = #THUMB_SIZE2
      EndIf
      
      If GetGadgetState(#OPT_THUMB_SIZE_3)
         \iThumbSize = #THUMB_SIZE3
      EndIf
      
      
      If GetGadgetState(#OPT_LOCKED_LEFT)
         \iThumbSide = #THUMB_LEFT
      EndIf
      
      If GetGadgetState(#OPT_LOCKED_RIGHT)
         \iThumbSide = #THUMB_RIGHT
      EndIf
      
      
      If GetGadgetState(#OPT_THUMB_BLACK)
         \iThumbColor = #THUMB_BLACK
      EndIf
      
      If GetGadgetState(#OPT_THUMB_WHITE)
         \iThumbColor = #THUMB_WHITE
      EndIf
      
      
      ; general settings
      
      \flgRememberLastDirectory = GetGadgetState(#CHK_SAVE_LAST_DIR)
      \flgSaveSameDirAsFiles = GetGadgetState(#CHK_SAVE_SAME_AS_FILES)
      \flgShowOptions = GetGadgetState(#CHK_SHOW_OPTIONS)
      \flgClearBeforeDrop = GetGadgetState(#CHK_CLEAR_BEFORE_DROP)
      \flgShowFullPath = GetGadgetState(#CHK_SHOW_FULL_PATH)
      \flgSortAfterDrop = GetGadgetState(#CHK_SORT_ALWAYS)
      \flgOpenPdf = GetGadgetState(#CHK_OPEN_PDF)
      \flgOpenDir = GetGadgetState(#CHK_OPEN_DIR)
      \flgConfirmations = GetGadgetState(#CHK_CONFIRMATIONS)
      \flgMemoryLoad = GetGadgetState(#CHK_MEMORY_LOAD)
      \flgShowSizeCondensed = GetGadgetState(#CHK_SHOW_SIZE_CONDENSED)
      \flgSortIncludingPath = GetGadgetState(#CHK_SORT_INCLUDING_PATH)
      \flgPreprocessing = GetGadgetState(#CHK_PREPROCESSING)
      \flgDocTitleFromFileName = GetGadgetState(#CHK_DOCTITLE_FROM_FILENAME)
      \flgDeleteLogIfSuccessful = GetGadgetState(#CHK_DELETE_LOG_IF_SUCCESSFUL)
      \flgEnableSubFolders = GetGadgetState(#CHK_ENABLE_SUBFOLDERS)
      \flgJpgNoRecompression = GetGadgetState(#CHK_JPG_NO_RECOMPRESSION)
      
      If \flgPreprocessing = #False
         s_Options\flgShowThumbs = #False
         SetMenuItemState(#MNU_WIN_MAIN, #MNI_SHOW_THUMBS, #False)
      EndIf
      
      DisableMenuItem(#MNU_WIN_MAIN, #MNI_SHOW_THUMBS, Bool(s_Options\flgPreprocessing = #False))
      
   EndWith
   
EndProcedure

Procedure EnablePaperFormatGadgets (flgStatus)
   DisableGadget(#TXT_PAPER_ORIENATION, flgStatus)
   DisableGadget(#OPT_PORTRAIT, flgStatus)
   DisableGadget(#OPT_LANDSCAPE, flgStatus)
   
   DisableGadget(#CHK_ENLARGE_TO_FIT, flgStatus)
   
   If GetGadgetState(#CHK_ENLARGE_TO_FIT) = #True
      DisableGadget(#CHK_KEEP_ASPECT, flgStatus)
   EndIf
EndProcedure

Procedure.i ShowSettingsPanel()
   Shared s_Options, *s_LastThumbHandle
   Protected iEvent, iBaseY, iSaveBaseY, flgExitLoop, iRetVal, iStyle
   Protected tWinSettings.T_WINDOW_RECT
   Protected flgSave_ShowFullPath = s_Options\flgShowFullPath
   Protected flgSave_ShowSizeCondensed = s_Options\flgShowSizeCondensed
   Protected flgSave_JpgNoRecompression = s_Options\flgJpgNoRecompression
   
   tWinSettings\X = 0
   tWinSettings\Y = 0
   tWinSettings\W = 500
   tWinSettings\H = 340
   
   iStyle = #WS_CAPTION | #WS_BORDER | #WS_SYSMENU | #WS_VISIBLE
   
   HE(OpenWindow(#WIN_SETTINGS, tWinSettings\X, tWinSettings\Y, tWinSettings\W, tWinSettings\H, "Options", iStyle, WindowID(#WIN_MAIN)), "Open window failed for #WIN_SETTINGS.")
   
   CenterWindowToWindow(WindowID(#WIN_SETTINGS), WindowID(#WIN_MAIN))
   
   AddKeyboardShortcut(#WIN_SETTINGS, #PB_Shortcut_F1, #MNI_HELP)
   
   DisableWin(#WIN_MAIN)
   
   PanelGadget(#PNL_SETTINGS, 5, 5, tWinSettings\W - 10, tWinSettings\H - 45)
   
   ; disable themes for the panel gadget
   ;SetWindowTheme(GadgetID(#PNL_SETTINGS), "", "")
   
   
   ;- TAB 1, PDF CREATION
   
   ; IMAGE PROCESSING
   
   AddGadgetItem(#PNL_SETTINGS, -1, " PDF Creation ")
   
   FrameGadget(#FRM_PDF_IMAGE_PROC, 10, 8, 190, 250, " Image Processing ")
   
   iBaseY = 40
   CheckBoxGadget(#CHK_COMPRESS, 25, iBaseY, 140, 20, " Use JPG compression")
   
   iBaseY + 38
   TextGadget(#TXT_QUALITY_1, 20, iBaseY, 170, 20, "JPG image quality", #PB_Text_Center)
   iBaseY + 17
   TrackBarGadget(#TRK_QUALITY, 20, iBaseY, 170, 21, 1, 10, #PB_TrackBar_Ticks)
   iBaseY + 30
   TextGadget(#TXT_QUALITY_2, 20, iBaseY, 170, 20, "", #PB_Text_Center)
   
   iBaseY + 32
   TextGadget(#TXT_SCALING_1, 20, iBaseY, 170, 20, "Image resize", #PB_Text_Center)
   iBaseY + 17
   TrackBarGadget(#TRK_SCALING, 20, iBaseY, 170, 21, 1, 100)
   iBaseY + 20
   TextGadget(#TXT_SCALING_2, 20, iBaseY, 170, 20, "", #PB_Text_Center)
   
   iBaseY + 37
   TextGadget(#TXT_RESIZE_IF, 25, iBaseY, 140, 20, "Resize if larger than ")
   StringGadget(#STG_RESIZE_IF, 150, iBaseY - 4, 35, 20, "0", #ES_RIGHT | #ES_NUMBER)
   StringGadget_SetLimit(#STG_RESIZE_IF, 4)
   SetGadgetText(#STG_RESIZE_IF, Str(s_Options\iResizeOnlyIfWider))
   
   SetGadgetState(#CHK_COMPRESS, s_Options\flgJpgCompression)
   
   EnableImageQualityGadgets (GetGadgetState(#CHK_COMPRESS)):  SetJpgQuality (s_Options\iJpgQuality)
   
   SetScaling (s_Options\iImageScaling)
   
   ; PAGE OUTPUT
   
   FrameGadget(#FRM_PDF_OUTPUT, 210, 8, 265, 250, " Page Output ")
   
   iBaseY = 45
   TextGadget(#TXT_PAPER_ORIENATION, 225, iBaseY, 150, 20, "Orientation")
   OptionGadget(#OPT_PORTRAIT,  308, iBaseY - 14, 70, 20, "Portrait")
   OptionGadget(#OPT_LANDSCAPE, 308, iBaseY + 12, 100, 20, "Landscape")
   
   iBaseY + 60
   TextGadget(#TXT_PAPER_FORMAT, 225, iBaseY, 145, 20, "Paper format")
   ComboBoxGadget(#CMB_PAPER_FORMAT, 308, iBaseY - 5, 155, 22)
   LoadPaperFormats (#CMB_PAPER_FORMAT)
   
   iBaseY + 45
   TextGadget(#TXT_PDF_STARTUP, 225, iBaseY, 145, 20, "Startup mode")
   ComboBoxGadget(#CMB_PDF_STARTUP, 308, iBaseY - 5, 155, 22)
   LoadPdfZoom (#CMB_PDF_STARTUP)
   
   iBaseY + 45
   CheckBoxGadget(#CHK_ENLARGE_TO_FIT, 225, iBaseY, 200, 20, " Enlarge to fit page if smaller")
   
   iBaseY + 30
   CheckBoxGadget(#CHK_KEEP_ASPECT, 225, iBaseY, 230, 20, " Keep original image aspect ratio")
   
   
   If s_Options\iPaperOrientation = #PAPER_PORTRAIT
      SetGadgetState(#OPT_PORTRAIT, 1)
   Else
      SetGadgetState(#OPT_LANDSCAPE, 1)
   EndIf
   
   SetGadgetState(#CMB_PAPER_FORMAT, s_Options\iPaperFormat - 1)
   SetGadgetState(#CMB_PDF_STARTUP, s_Options\iPdfstartup - 1)
   SetGadgetState(#CHK_ENLARGE_TO_FIT, s_Options\flgEnlargeToFit)
   SetGadgetState(#CHK_KEEP_ASPECT, s_Options\flgKeepAspect)
   DisableGadget(#CHK_KEEP_ASPECT, Bool(Not GetGadgetState(#CHK_ENLARGE_TO_FIT)))
   EnablePaperFormatGadgets (Bool(GetGadgetState(#CMB_PAPER_FORMAT) + 1 = #PAPER_FORMAT_PAPERLESS))
   
   
   ;- TAB 2, TEXT
   
   AddGadgetItem(#PNL_SETTINGS, -1, Space(5) + "Text" + Space(5))
   
   iBaseY = 8
   FrameGadget(#FRM_HEADER, 10, iBaseY, 465, 125, " Page Header / Footer ")
   
   iBaseY + 25
   CheckBoxGadget(#CHK_SHOW_HEADER, 25, iBaseY, 400, 20, "")
   
   
   iBaseY + 35
   TextGadget(#TXT_HEADER_WHERE, 25, iBaseY, 80, 20, "Text Position")
   ComboBoxGadget(#CMB_HEADER_WHERE, 120, iBaseY - 4, 70, 20)
   LoadHeaderPosition(#CMB_HEADER_WHERE)
   
   TextGadget(#TXT_HEADER_WHAT, 280, iBaseY, 80, 20, "Contents")
   ComboBoxGadget(#CMB_HEADER_WHAT, 340, iBaseY - 4, 120, 20)
   LoadHeaderWhat (#CMB_HEADER_WHAT)
   
   iBaseY + 35
   TextGadget(#TXT_HEADER_ALIGN, 25, iBaseY, 150, 20, "Text Alignment")
   ComboBoxGadget(#CMB_HEADER_ALIGN, 120, iBaseY - 4, 70, 20)
   LoadHeaderAlignment(#CMB_HEADER_ALIGN)
   
   TextGadget(#TXT_HEADER_SIZE, 280, iBaseY, 120, 20, "Text Size")
   ComboBoxGadget(#CMB_HEADER_SIZE, 410, iBaseY - 4, 50, 20)
   LoadHeaderSize(#CMB_HEADER_SIZE)
   
   SetGadgetState(#CHK_SHOW_HEADER, s_Options\flgDisplayHeader)
   SetGadgetState(#CMB_HEADER_SIZE, s_Options\iHeaderFontSize - #HEADER_SIZE_MIN)
   SetGadgetState(#CMB_HEADER_WHAT, s_Options\iHeaderDisplayWhat - 1)
   SetGadgetState(#CMB_HEADER_WHERE, s_Options\iHeaderDisplayWhere - 1)
   SetGadgetState(#CMB_HEADER_ALIGN, s_Options\iHeaderDisplayAlign - 1)
   
   EnableHeaderGadgets(GetGadgetState(#CHK_SHOW_HEADER))
   
   
   ; DOCUMENT INFO
   
   iBaseY = 138
   FrameGadget(#FRM_DOC_INFO, 10, iBaseY, 465, 120, " Document Info ")
   
   iBaseY + 25
   TextGadget(#TXT_DOC_AUTHOR_1, 25, iBaseY, 395, 18, "Author of the document")
   TextGadget(#TXT_DOC_AUTHOR_2, 340, iBaseY, 100, 18, "Save this", #PB_Text_Right)
   CheckBoxGadget(#CHK_SAVE_AUTHOR, 415 + 32, iBaseY, 16, 16, "")
   
   iBaseY + 18
   StringGadget(#STG_DOC_AUTHOR, 25, iBaseY, 435, 18, s_Options\DocAuthor$)
   
   iBaseY + 32
   TextGadget(#TXT_DOC_TITLE_1, 25, iBaseY, 395, 18, "Title of the document")
   TextGadget(#TXT_DOC_TITLE_2, 340, iBaseY, 100, 18, "Save this", #PB_Text_Right)
   CheckBoxGadget(#CHK_SAVE_TITLE, 415 + 32, iBaseY, 16, 16, "")
   
   iBaseY + 18
   StringGadget(#STG_DOC_TITLE, 25, iBaseY, 435, 18, s_Options\DocTitle$)
   
   SetGadgetState(#CHK_SAVE_AUTHOR, s_Options\flgSaveAuthor)
   SetGadgetState(#CHK_SAVE_TITLE, s_Options\flgSaveTitle)
   
   
   ;- TAB 3, THUMBNAIL
   
   AddGadgetItem(#PNL_SETTINGS, -1, "  Thumbnail  ")
   
   iBaseY = 8
   
   FrameGadget(#FRM_THUMB_POSITION, 10, iBaseY, 465, 120, " Thumbnail Position ")
   
   iBaseY + 20
   CheckBoxGadget(#CHK_LOCKED_TO_MAIN, 10 + 15, iBaseY, 300, 20, " The thumbnail will be locked to the main window")
   
   iBaseY + 30
   OptionGadget(#OPT_LOCKED_LEFT, 10 + 35, iBaseY, 300, 20, "to its LEFT side")
   
   iBaseY + 25
   OptionGadget(#OPT_LOCKED_RIGHT, 10 + 35, iBaseY, 300, 20, "to its RIGHT side")
   
   
   iBaseY + 55: iSaveBaseY = iBaseY
   
   FrameGadget(#FRM_THUMB_SIZE, 10, iBaseY, 240, 120, " Thumbnail Size ")
   
   iBaseY + 25
   OptionGadget(#OPT_THUMB_SIZE_1, 10 + 15, iBaseY, 140, 20, "320 x 320  pixels")
   
   iBaseY + 25
   OptionGadget(#OPT_THUMB_SIZE_2, 10 + 15, iBaseY, 140, 20, "480 x 480  pixels")
   
   iBaseY + 25
   OptionGadget(#OPT_THUMB_SIZE_3, 10 + 15, iBaseY, 140, 20, "640 x 640  pixels")
   
   
   iBaseY = iSaveBaseY
   FrameGadget(#FRM_THUMB_BACK_COLOR, 260, iBaseY, 215, 120, " Thumbnail Background ")
   
   iBaseY + 25
   OptionGadget(#OPT_THUMB_BLACK, 260 + 15, iBaseY, 100, 20, "Black")
   
   iBaseY + 25
   OptionGadget(#OPT_THUMB_WHITE, 260 + 15, iBaseY, 100, 20, "White")
   
   
   SetGadgetState(#CHK_LOCKED_TO_MAIN, s_Options\flgThumbLocked)
   
   DisableGadget(#OPT_LOCKED_LEFT, Bool(GetGadgetState(#CHK_LOCKED_TO_MAIN) = 0))
   DisableGadget(#OPT_LOCKED_RIGHT, Bool(GetGadgetState(#CHK_LOCKED_TO_MAIN) = 0))
   
   If CountGadgetItems(#LIG_FILES)
      DisableGadget(#OPT_THUMB_SIZE_1, 1)
      DisableGadget(#OPT_THUMB_SIZE_2, 1)
      DisableGadget(#OPT_THUMB_SIZE_3, 1)
   EndIf
   
   If s_Options\iThumbSide = #THUMB_LEFT
      SetGadgetState(#OPT_LOCKED_LEFT, 1)
   Else
      SetGadgetState(#OPT_LOCKED_RIGHT, 1)
   EndIf
   
   Select s_Options\iThumbSize
      Case #THUMB_SIZE1
         SetGadgetState(#OPT_THUMB_SIZE_1, 1)
      Case #THUMB_SIZE2
         SetGadgetState(#OPT_THUMB_SIZE_2, 1)
      Case #THUMB_SIZE3
         SetGadgetState(#OPT_THUMB_SIZE_3, 1)
   EndSelect
   
   
   Select s_Options\iThumbColor
      Case #THUMB_BLACK
         SetGadgetState(#OPT_THUMB_BLACK, 1)
      Case #THUMB_WHITE
         SetGadgetState(#OPT_THUMB_WHITE, 1)
   EndSelect
   
   
   ;- TAB 4, GENERAL SETTINGS
   
   
   AddGadgetItem(#PNL_SETTINGS, -1, "   General    ")
   
   ScrollAreaGadget(#SCL_OPTIONS, 0, 1, GadgetWidth(#PNL_SETTINGS) - 9, GadgetHeight(#PNL_SETTINGS) - 31, GadgetWidth(#PNL_SETTINGS) - 9, 405, 5, #PB_ScrollArea_BorderLess)
   
   ShowScrollBar_(GadgetID(#SCL_OPTIONS), #SB_HORZ, 0)
   
   iBaseY = 10
   CheckBoxGadget(#CHK_CLEAR_BEFORE_DROP, 20, iBaseY, 400, 20, " Clear the file list before a drop")
   
   iBaseY + 25
   CheckBoxGadget(#CHK_SORT_ALWAYS, 20, iBaseY, 400, 20, " Sort the file list after each drop")
   
   iBaseY + 25
   CheckBoxGadget(#CHK_SHOW_FULL_PATH, 20, iBaseY, 400, 20, " Show the full path of the files in the list view")
   
   iBaseY + 25
   CheckBoxGadget(#CHK_SORT_INCLUDING_PATH, 20, iBaseY, 400, 20, " Include the path of the files in the sort")
   
   iBaseY + 25
   CheckBoxGadget(#CHK_SHOW_SIZE_CONDENSED, 20, iBaseY, 400, 20, " Display the size of the files in condensed format")
   
   iBaseY + 25
   CheckBoxGadget(#CHK_SHOW_OPTIONS, 20, iBaseY, 400, 20, " Always offer the PDF options window before saving")
   
   iBaseY + 25
   CheckBoxGadget(#CHK_OPEN_DIR, 20, iBaseY, 400, 20, " Open the destination directory in Explorer after creation")
   
   iBaseY + 25
   CheckBoxGadget(#CHK_OPEN_PDF, 20, iBaseY, 400, 20, " Open the created PDF with the default viewer")
   
   iBaseY + 25
   CheckBoxGadget(#CHK_SAVE_LAST_DIR, 20, iBaseY, 400, 20, " Remember the directory used to save the last PDF")
   
   iBaseY + 25
   CheckBoxGadget(#CHK_SAVE_SAME_AS_FILES, 20, iBaseY, 400, 20, " Save the PDF to the same directory of the image files")
   
   iBaseY + 25
   CheckBoxGadget(#CHK_CONFIRMATIONS, 20, iBaseY, 400, 20, " Always ask for confirmations")
   
   iBaseY + 25
   CheckBoxGadget(#CHK_PREPROCESSING, 20, iBaseY, 400, 20, " Enable image preprocessing")
   
   iBaseY + 25
   CheckBoxGadget(#CHK_DOCTITLE_FROM_FILENAME, 20, iBaseY, 400, 20, " Set document title from filename")
   
   iBaseY + 25
   CheckBoxGadget(#CHK_DELETE_LOG_IF_SUCCESSFUL, 20, iBaseY, 400, 20, " Delete the log if the PDF has been built without errors")
   
   iBaseY + 25
   CheckBoxGadget(#CHK_ENABLE_SUBFOLDERS, 20, iBaseY, 400, 20, " Enable subfolders recursion")
   
   iBaseY + 25
   CheckBoxGadget(#CHK_JPG_NO_RECOMPRESSION, 20, iBaseY, 400, 20, " Store JPG files without reprocessing them")
   
   
   iBaseY + 25
   CheckBoxGadget(#CHK_MEMORY_LOAD, 20, iBaseY, 400, 20, " Enable high memory load warning")
   
   
   SetGadgetState(#CHK_SAVE_LAST_DIR, s_Options\flgRememberLastDirectory)
   SetGadgetState(#CHK_SAVE_SAME_AS_FILES, s_Options\flgSaveSameDirAsFiles)
   SetGadgetState(#CHK_CLEAR_BEFORE_DROP, s_Options\flgClearBeforeDrop)
   SetGadgetState(#CHK_SHOW_OPTIONS, s_Options\flgShowOptions)
   SetGadgetState(#CHK_SHOW_FULL_PATH, s_Options\flgShowFullPath)
   SetGadgetState(#CHK_SORT_ALWAYS, s_Options\flgSortAfterDrop)
   SetGadgetState(#CHK_OPEN_PDF, s_Options\flgOpenPdf)
   SetGadgetState(#CHK_OPEN_DIR, s_Options\flgOpenDir)
   SetGadgetState(#CHK_SHOW_SIZE_CONDENSED, s_Options\flgShowSizeCondensed)
   SetGadgetState(#CHK_CONFIRMATIONS, s_Options\flgConfirmations)
   SetGadgetState(#CHK_MEMORY_LOAD, s_Options\flgMemoryLoad)
   SetGadgetState(#CHK_SORT_INCLUDING_PATH, s_Options\flgSortIncludingPath)
   SetGadgetState(#CHK_PREPROCESSING, s_Options\flgPreprocessing)
   SetGadgetState(#CHK_DOCTITLE_FROM_FILENAME, s_Options\flgDocTitleFromFileName)
   SetGadgetState(#CHK_DELETE_LOG_IF_SUCCESSFUL, s_Options\flgDeleteLogIfSuccessful)
   SetGadgetState(#CHK_ENABLE_SUBFOLDERS, s_Options\flgEnableSubFolders)
   SetGadgetState(#CHK_JPG_NO_RECOMPRESSION, s_Options\flgJpgNoRecompression)
   
   
   If CountGadgetItems(#LIG_FILES)
      DisableGadget(#CHK_PREPROCESSING, 1)
   EndIf
   
   CloseGadgetList() ; scrollareagadget
   
   CloseGadgetList() ; panelgadget
   
   ButtonGadget(#BTN_OPTIONS_CANCEL, 5, tWinSettings\H - 35, 100, 30, "Cancel")
   ButtonGadget(#BTN_OPTIONS_OK, tWinSettings\W - 105, tWinSettings\H - 35, 100, 30, "OK", #PB_Button_Default)
   
   StringGadget_SetLimit (#STG_DOC_AUTHOR, 60)
   StringGadget_SetLimit (#STG_DOC_TITLE, 60)
   
   
   flgExitLoop = #False
   
   Repeat
      
      iEvent = WaitWindowEvent()
      
      Select iEvent
            
         Case #PB_Event_Menu
            Select EventMenu()
               Case #MNI_HELP
                  Select GetGadgetState(#PNL_SETTINGS)
                     Case 0 : OpenHelp(#PROGRAM_NAME$ + ".chm", "Tab1.html")
                     Case 1 : OpenHelp(#PROGRAM_NAME$ + ".chm", "Tab2.html")
                     Case 2 : OpenHelp(#PROGRAM_NAME$ + ".chm", "Tab3.html")
                     Case 3 : OpenHelp(#PROGRAM_NAME$ + ".chm", "Tab4.html")
                  EndSelect
            EndSelect
            
         Case #PB_Event_Gadget
            
            Select EventGadget()
               Case #CHK_SAVE_SAME_AS_FILES
                  If GetGadgetState(#CHK_SAVE_SAME_AS_FILES)
                     SetGadgetState(#CHK_SAVE_LAST_DIR, 0)
                  EndIf
                  
               Case #CHK_SAVE_LAST_DIR
                  If GetGadgetState(#CHK_SAVE_LAST_DIR)
                     SetGadgetState(#CHK_SAVE_SAME_AS_FILES, 0)
                  EndIf
                  
               Case #PNL_SETTINGS
                  If GetGadgetState(#PNL_SETTINGS) = 1 ; TAB 2
                     
                     SetGadgetText(#CHK_SHOW_HEADER, " Show a text header on every page")
                     
                     If (Bool(GetGadgetState(#CMB_PAPER_FORMAT) + 1 = #PAPER_FORMAT_PAPERLESS))
                        SetGadgetText(#CHK_SHOW_HEADER, GetGadgetText(#CHK_SHOW_HEADER) + "  ( ignored for " + Chr(34) +  "Paperless" + Chr(34) + " )")
                     EndIf
                  EndIf
                  
                  If GetGadgetState(#PNL_SETTINGS) = 3 ; TAB 4
                                                       ; to make the mouse wheel scroll without the need
                                                       ; to click beforehand
                     SetActiveGadget(#CHK_CLEAR_BEFORE_DROP)
                  EndIf
                  
               Case #CHK_ENLARGE_TO_FIT
                  DisableGadget(#CHK_KEEP_ASPECT, Bool(Not GetGadgetState(#CHK_ENLARGE_TO_FIT)))
                  
               Case #CMB_PAPER_FORMAT
                  EnablePaperFormatGadgets (Bool(GetGadgetState(#CMB_PAPER_FORMAT) + 1 = #PAPER_FORMAT_PAPERLESS))
                  
               Case #CHK_COMPRESS
                  EnableImageQualityGadgets (GetGadgetState(#CHK_COMPRESS))
                  
               Case #CHK_SHOW_HEADER
                  EnableHeaderGadgets(GetGadgetState(#CHK_SHOW_HEADER))
                  
               Case #TRK_QUALITY
                  SetJpgQuality(GetGadgetState(#TRK_QUALITY))
                  
               Case #TRK_SCALING
                  SetScaling(GetGadgetState(#TRK_SCALING))
                  
               Case #CHK_LOCKED_TO_MAIN
                  DisableGadget(#OPT_LOCKED_LEFT, Bool(GetGadgetState(#CHK_LOCKED_TO_MAIN) = 0))
                  DisableGadget(#OPT_LOCKED_RIGHT, Bool(GetGadgetState(#CHK_LOCKED_TO_MAIN) = 0))
                  
               Case #BTN_OPTIONS_OK
                  
                  If CheckSettings()
                     
                     UpdateSettings()
                     
                     If s_Options\flgShowThumbs
                        *s_LastThumbHandle = 0
                        If GetGadgetState(#LIG_FILES) <> -1
                           SendMessage_(WindowID(#WIN_MAIN), #WM_SHOW_THUMB, GetGadgetState(#LIG_FILES), 0)
                        EndIf
                        RefreshThumbnail()
                     EndIf
                     
                     If (s_Options\flgShowFullPath <> flgSave_ShowFullPath) Or (s_Options\flgShowSizeCondensed <> flgSave_ShowSizeCondensed) Or (s_Options\flgJpgNoRecompression <> flgSave_JpgNoRecompression)
                        RefreshListView()
                        WinResize()
                     EndIf
                     
                     flgExitLoop = #True
                     iRetVal = #True
                  EndIf
                  
               Case #BTN_OPTIONS_CANCEL
                  flgExitLoop = #True
                  iRetVal = #False
            EndSelect
            
         Case #PB_Event_CloseWindow
            While WindowEvent(): Delay(1): Wend
            flgExitLoop = #True
            iRetVal = #False
      EndSelect
      
   Until flgExitLoop
   
   EnableWin(#WIN_MAIN)
   
   CloseWindow(#WIN_SETTINGS)
   
   ProcedureReturn iRetVal
EndProcedure

Procedure.s AskSkippedListName ()
   
   Protected BaseDir$, Pattern$, RetVal$
   Protected iPattern
   
   BaseDir$ = GetDesktopPath() + "\"
   
   Pattern$ = "text (*.txt)|*.txt"
   
   iPattern = 0 ; use the first pattern as standard
   
   RetVal$ = SaveFileRequester("", BaseDir$ + "i2pdf skipped files.txt", Pattern$, iPattern)
   
   If RetVal$
      
      If Len(GetExtensionPart(RetVal$)) = 0 ; if without extension
         If SelectedFilePattern() = 0       ; if current pattern selected is "*.txt"
            RetVal$ + ".txt"
         EndIf
      EndIf
      
   EndIf
   
   ProcedureReturn RetVal$
   
EndProcedure

Procedure.i CheckIfAllFromSamePath()
   Shared s_lsItem()
   Protected flgRetVal.i
   Protected Path$, LastPath$
   
   flgRetVal = #True
   
   ForEach s_lsItem()
      
      Path$ = GetPathPart(s_lsItem()\FullFileName$)
      
      If LastPath$
         If LastPath$ <> Path$
            flgRetVal = #False
            Break
         EndIf
      Else
         LastPath$ = Path$
      EndIf
   Next
   
   ProcedureReturn flgRetVal
EndProcedure

Procedure.s AskPdfName(FileName$)
   Shared s_Options, s_lsItem()
   Protected BaseDir$, Pattern$, RetVal$
   Protected iPattern
   
   BaseDir$ = GetDesktopPath() + "\"
   
   If (s_Options\flgRememberLastDirectory) And Len(s_Options\LastDirectory$)
      If IsDirThere(s_Options\LastDirectory$)
         BaseDir$ = s_Options\LastDirectory$
      EndIf
   EndIf
   
   If (s_Options\flgSaveSameDirAsFiles)
      If CheckIfAllFromSamePath()
         FirstElement(s_lsItem())
         BaseDir$ = GetPathPart(s_lsItem()\FullFileName$)
      EndIf
   EndIf
   
   Pattern$ = "PDF documents (*.pdf)|*.pdf;|All files (*.*)|*.*"
   
   iPattern = 0    ; use the first of the two possible patterns as standard
   
   RetVal$ = SaveFileRequester("PDF file name", BaseDir$ + FileName$, Pattern$, iPattern)
   
   If RetVal$
      
      If Len(GetExtensionPart(RetVal$)) = 0 ; if without extension
         If SelectedFilePattern() = 0       ; if current pattern selected is "*.pdf"
            RetVal$ + ".pdf"
         EndIf
      EndIf
      
   EndIf
   
   ProcedureReturn RetVal$
   
EndProcedure

Procedure.i LoadJpgFileIntoBuffer (ImageFileName$, *StreamBuffer.Integer)
   Protected iStreamSize, nFileIn
   
   iStreamSize = FileSize(ImageFileName$)
   
   If iStreamSize = 0 : Goto LoadJpgFileIntoBuffer_ERR : EndIf
   
   *StreamBuffer\i = AllocateMemory(iStreamSize)
   
   If *StreamBuffer\i = 0 : Goto LoadJpgFileIntoBuffer_ERR : EndIf
   
   nFileIn = ReadFile(#PB_Any, ImageFileName$)
   
   If nFileIn  = 0 : Goto LoadJpgFileIntoBuffer_ERR : EndIf
   
   If ReadData(nFileIn, *StreamBuffer\i, iStreamSize) <> iStreamSize : Goto LoadJpgFileIntoBuffer_ERR : EndIf
   
   CloseFile(nFileIn)
   ProcedureReturn #True
   
   LoadJpgFileIntoBuffer_ERR:
   If IsFile(nFileIn) : CloseFile(nFileIn) : EndIf
   If *StreamBuffer\i : FreeMemory(*StreamBuffer\i) : EndIf
   ProcedureReturn #False
EndProcedure

Procedure.i BuildPdf (OutFile$, *tLogfile.T_LOGFILE)
   Shared s_Options, s_lsItem()
   Protected qFreeMem.q, qLowMem.q
   Protected iResizePerc, iJpgQuality, iProcessed, iWidth, iHeight, iOrientation, iImageType, iHeaderAlign
   Protected iEvent, flgAbort, flgWarned, iRetVal
   Protected fNewWidth.f, fNewHeight.f, fPageWidth.f, fPageHeight.f, fDestWidth.f, fDestHeight.f, fXpos.f, fYpos.f, fCellHeightMM.f = 10
   Protected tWinBuilding.T_WINDOW_RECT
   Protected hCurrImage, *ImageBuffer, i
   Protected tGUID.GUID, GUID$, GUIDBuffer${39}
   Protected PaperSize$, ImageFileName$, Header$, Message$
   Protected FontName$, iFontSize, nFontBold
   Protected flgCurrImageIsJPG, iPixelFormat
   
   flgAbort       = #False
   flgWarned      = #False
   *ImageBuffer   = 0
   
   If s_Options\iPaperOrientation = #PAPER_PORTRAIT
      iOrientation = #PDF_PORTRAIT
   Else
      iOrientation = #PDF_LANDSCAPE
   EndIf
   
   Select s_Options\iPaperFormat
      Case #PAPER_FORMAT_A2         : PaperSize$ = #PDF_PAGE_FORMAT_A2
      Case #PAPER_FORMAT_A3         : PaperSize$ = #PDF_PAGE_FORMAT_A3
      Case #PAPER_FORMAT_A4         : PaperSize$ = #PDF_PAGE_FORMAT_A4
      Case #PAPER_FORMAT_LEGAL      : PaperSize$ = #PDF_PAGE_FORMAT_LEGAL
      Case #PAPER_FORMAT_LETTER     : PaperSize$ = #PDF_PAGE_FORMAT_LETTER
      Case #PAPER_FORMAT_PAPERLESS  : PaperSize$ = ""
   EndSelect
   
   pdf_Init()
   
   pdf_SetAuthor (s_Options\DocAuthor$)
   
   pdf_SetTitle (s_Options\DocTitle$)
   
   pdf_SetCreator (PROGRAM_TITLE_FULL())
   
   
   LogStart(PROGRAM_TITLE_FULL(), OutFile$, LogBuildFileName(OutFile$), s_Options)
   
   
   pdf_Create (iOrientation, PaperSize$, #PDF_UNITS_MM) ; for example A4 = 210 x 297
   
   I2P_CHECK_PDF (BuildPdf_PDF)
   
   
   pdf_SetDisplayMode (s_Options\iPdfStartup, #PDF_LAYOUT_SINGLE)
   
   
   If s_Options\flgDisplayHeader
      pdf_SetFont (#PDF_FONT_HELVETICA, s_Options\iHeaderFontSize)
   EndIf
   
   
   iResizePerc = s_Options\iImageScaling
   
   tWinBuilding\X = 0
   tWinBuilding\Y = 0
   tWinBuilding\W = 450
   tWinBuilding\H = 120
   
   HE(OpenWindow(#WIN_BUILD_PDF, tWinBuilding\X, tWinBuilding\Y, tWinBuilding\W, tWinBuilding\H, "", #PB_Window_SystemMenu | #PB_Window_Invisible, WindowID(#WIN_MAIN)), "Open window failed for #WIN_BUILD_PDF.")
   
   CenterWindowToWindow(WindowID(#WIN_BUILD_PDF), WindowID(#WIN_MAIN))
   
   TextGadget(#TXT_BUILD_PDF_1, 0, tWinBuilding\H - 110, tWinBuilding\W, 20, "", #PB_Text_Center)
   
   TextGadget(#TXT_BUILD_PDF_2, 0, tWinBuilding\H - 90, tWinBuilding\W, 15, "", #PB_Text_Center)
   
   ProgressBarGadget(#PRB_BUILDING, 5, tWinBuilding\H - 65, WindowWidth(#WIN_BUILD_PDF) - 10, 20, 0, ListSize(s_lsItem()))
   
   ButtonGadget(#BTN_BUILD_CANCEL, (tWinBuilding\W - 100) / 2, tWinBuilding\H - 35, 100, 30, "Cancel")
   
   ; load bold font
   FontName$ = Control_GetFontName (GadgetID(#TXT_BUILD_PDF_1))
   iFontSize = Control_GetFontSize (GadgetID(#TXT_BUILD_PDF_1))
   nFontBold = LoadFont (#PB_Any, FontName$, iFontSize, #PB_Font_Bold)
   
   SetGadgetFont(#TXT_BUILD_PDF_1, FontID(nFontBold))
   SetGadgetFont(#TXT_BUILD_PDF_2, FontID(nFontBold))
   
   DisableWin(#WIN_MAIN)
   
   PumpMessages()
   
   GetFreePhysicalMemory(@qLowMem)
   
   qLowMem = qLowMem / 20 ; 5% of free physical memory
   
   ForEach s_lsItem()
      
      GetFreePhysicalMemory(@qFreeMem)
      
      SetWindowTitle(#WIN_BUILD_PDF, "Building PDF ...  ( Free memory = " + FormatQuadWithByteSuffix(qFreeMem, #False, 2) + " )")
      
      SetActiveWindow(#WIN_BUILD_PDF)
      
      iProcessed + 1
      
      SetGadgetText(#TXT_BUILD_PDF_1, Str(iProcessed) + " of " + Str(ListSize(s_lsItem())))
      SetGadgetText(#TXT_BUILD_PDF_2, s_lsItem()\FileName$)
      
      ImageFileName$ = s_lsItem()\FullFileName$
      
      If FileSize(ImageFileName$) = -1
         LogAddMissing (ImageFileName$)
         Continue
      EndIf
      
      
      ; try to load the image
      i = GdiPlusLoadImage(@hCurrImage, ImageFileName$)
      
      ; check if image was loaded successfully
      If (i = #GDIPLUS_UNKNOWN_IMAGE_FORMAT) Or (i = #GDIPLUS_INVALID_PARAMETER)
         LogAddFailed (#I2PDF_LOG_UNSUPPORTED, ImageFileName$)
         Continue ; invalid file type, skipping
      EndIf
      
      ; chek for other errors
      I2P_CHECK_GDI (i, BuildPdf_GDI)
      
      ; check image type (JPG ?)
      GdipGetImageRawFormat (hCurrImage, @tGUID)
      StringFromGUID2_(@tGUID, @GUIDBuffer$, 39)
      GUID$ = PeekS(@GUIDBuffer$, 39, #PB_Unicode)
      
      If GUID$ = #ImageFormatJPEG$
         flgCurrImageIsJPG = #True
      Else
         flgCurrImageIsJPG = #False
      EndIf
      
      ; image width
      i = GdipGetImageWidth(hCurrImage, @iWidth)
      I2P_CHECK_GDI (i, BuildPdf_GDI)
      
      ; image height
      i = GdipGetImageHeight(hCurrImage, @iHeight)
      I2P_CHECK_GDI (i, BuildPdf_GDI)
      
      
      ; if the image is a JPG file and
      ; if JPG storing is active
      ; the JPG file must be 24 bit / True Color (skip 8 bit files)
      If s_Options\flgJpgNoRecompression And flgCurrImageIsJPG
         
         i = GdipGetImagePixelFormat(hCurrImage, @iPixelFormat)
         I2P_CHECK_GDI (i, BuildPdf_GDI)
         
         If s_Options\flgJpgNoRecompression = #True And iPixelFormat <> #PixelFormat24bppRGB
            LogAddFailed (#I2PDF_LOG_UNSUPPORTED, ImageFileName$)
            Continue ; invalid file type, skipping
         EndIf
      EndIf
      
      If (s_Options\flgJpgNoRecompression = #False) Or (flgCurrImageIsJPG = #False)
         ; try to convert color depth for the image
         i = GdiPlusChangeImageColorDepth (hCurrImage, @hCurrImage, 24) ; 24 bit to be able to do all I need
         
         ; check if image is successfully converted
         If (i = #GDIPLUS_WIN32_ERROR)
            LogAddFailed (#I2PDF_LOG_UNSUPPORTED, ImageFileName$)
            Continue ; probably the image is corrupted
         EndIf
         
         ; check for other errors
         I2P_CHECK_GDI (i, BuildPdf_GDI)
         
         If (iResizePerc < 100) And (iWidth > s_Options\iResizeOnlyIfWider) ; resize requested
            fNewWidth  = Int(iWidth * (iResizePerc / 100.0))
            fNewHeight = Int(iHeight * (iResizePerc / 100.0))
            
            i = GdiPlusResizeImage(@hCurrImage, fNewWidth, fNewHeight, #InterpolationModeHighQualityBicubic, #SmoothingModeHighQuality)
            
            I2P_CHECK_GDI (i, BuildPdf_GDI)
         Else
            fNewWidth = iWidth
            fNewHeight = iHeight
         EndIf
         
         
         If s_Options\flgJpgCompression
            
            iImageType = #PDF_IMAGE_JPG
            
            Select s_Options\iJpgQuality
               Case 1 : iJpgQuality = 10
               Case 2 : iJpgQuality = 20
               Case 3 : iJpgQuality = 25
               Case 4 : iJpgQuality = 30
               Case 5 : iJpgQuality = 35
               Case 6 : iJpgQuality = 50
               Case 7 : iJpgQuality = 60
               Case 8 : iJpgQuality = 75
               Case 9 : iJpgQuality = 90
               Case 10: iJpgQuality = 100
            EndSelect
            
            i = GdiPlusSaveImageToJpgStream (hCurrImage, @*ImageBuffer, iJpgQuality)
            I2P_CHECK_GDI (i, BuildPdf_GDI)
            
         Else
            iImageType = #PDF_IMAGE_PNG
            
            i = GdiPlusSaveImageToPngStream (hCurrImage, @*ImageBuffer)
            I2P_CHECK_GDI (i, BuildPdf_GDI)
         EndIf
         
         
         i = GdipDisposeImage(hCurrImage)
         I2P_CHECK_GDI (i, BuildPdf_GDI)
         
      Else
         
         i = LoadJpgFileIntoBuffer (ImageFileName$, @*ImageBuffer)
         
         If i = #False
            LogAddFailed (#I2PDF_LOG_JPG_STORING, ImageFileName$)
            Continue ; probably the image is corrupted
         EndIf
         
         iImageType = #PDF_IMAGE_JPG
         fNewWidth = iWidth
         fNewHeight = iHeight
         
         i = GdipDisposeImage(hCurrImage)
         I2P_CHECK_GDI (i, BuildPdf_GDI)
      EndIf
      
      
      If s_Options\iPaperFormat = #PAPER_FORMAT_PAPERLESS
         
         fXpos = 0.0
         fYpos = 0.0
         
         fDestWidth = fNewWidth
         fDestHeight = fNewHeight
         
         pdf_AddPage (iOrientation, pdf_MakeCustomFormatString (fDestWidth, fDestHeight, iOrientation))
         
         I2P_CHECK_PDF (BuildPdf_PDF)
         
      Else
         
         pdf_PaperSizeToPT (PaperSize$, @fPageWidth, @fPageHeight)
         
         If s_Options\iPaperOrientation = #PAPER_LANDSCAPE
            Swap fPageWidth, fPageHeight
         EndIf
         
         If s_Options\flgDisplayHeader
            If s_Options\iPaperFormat <> #PAPER_FORMAT_PAPERLESS
               fPageHeight - pdf_MMtoPT(fCellHeightMM) * 2
            EndIf
         EndIf
         
         Repeat
            
            ; if the image is smaller then the page and enlarge to fit is not requested, simply keep the image size
            If (fNewWidth < fPageWidth) And (fNewHeight < fPageHeight) And (s_Options\flgEnlargeToFit = #False)
               fDestWidth = fNewWidth
               fDestHeight = fNewHeight
               Break
            EndIf
            
            
            ; if enlarge to fit is wanted and keeping aspect ratio is not required, simply shrink/stretch to paper size
            If (s_Options\flgKeepAspect = #False) And (s_Options\flgEnlargeToFit = #True)
               fDestWidth = fPageWidth
               fDestHeight = fPageHeight
               Break
            EndIf
            
            
            fDestWidth = fPageWidth
            fDestHeight = fNewHeight / (fNewWidth / fPageWidth)
            If fDestHeight <= fPageHeight
               Break
            EndIf
            
            
            fDestHeight = fPageHeight
            fDestWidth = fNewWidth / (fNewHeight/ fPageHeight)
            If fDestWidth <= fPageWidth
               Break
            EndIf
            
            
            HE(#False, "Internal error!")
            Break
            
         ForEver
         
         
         fXPos = (fPageWidth - fDestWidth) / 2.0
         fYPos = (fPageHeight - fDestHeight) / 2.0
         
         If s_Options\flgDisplayHeader
            If s_Options\iPaperFormat <> #PAPER_FORMAT_PAPERLESS
               fYPos + pdf_MMtoPT(fCellHeightMM)
            EndIf
         EndIf
         
         pdf_AddPage ()
         
         I2P_CHECK_PDF (BuildPdf_PDF)
      EndIf
      
      pdf_ImageFromMem (ImageFileName$, iImageType, #PDF_BPP_24, *ImageBuffer, MemorySize(*ImageBuffer), pdf_PTtoMM(fNewWidth), pdf_PTtoMM(fNewHeight), pdf_PTtoMM(fXPos), pdf_PTtoMM(fYPos), pdf_PTtoMM(fDestWidth), pdf_PTtoMM(fDestHeight))
      
      I2P_CHECK_PDF (BuildPdf_PDF)
      
      
      If s_Options\flgDisplayHeader
         
         If s_Options\iPaperFormat <> #PAPER_FORMAT_PAPERLESS
            
            Select s_Options\iHeaderDisplayWhat
               Case #HEADER_SHOW_NAME
                  Header$ = s_lsItem()\FileName$
               Case #HEADER_SHOW_FULL_NAME
                  Header$ = s_lsItem()\FullFileName$
            EndSelect
            
            Select s_Options\iHeaderDisplayAlign
               Case #HEADER_ALIGN_LEFT
                  iHeaderAlign = #PDF_ALIGN_LEFT
               Case #HEADER_ALIGN_RIGHT
                  iHeaderAlign = #PDF_ALIGN_RIGHT
               Case #HEADER_ALIGN_CENTER
                  iHeaderAlign = #PDF_ALIGN_CENTER
            EndSelect
            
            
            If s_Options\iHeaderDisplayWhere = #HEADER_TOP
               pdf_SetXY (0, 0)
            Else
               pdf_SetXY (0, pdf_PTtoMM(fPageHeight) + fCellHeightMM)
            EndIf
            
            pdf_Cell (pdf_PTtoMM(fPageWidth), fCellHeightMM, Header$, 0, iHeaderAlign)
            
         EndIf
         
      EndIf
      
      I2P_CHECK_PDF (BuildPdf_PDF)
      
      LogAddProcessed(ImageFileName$, FileSize(ImageFileName$), MemorySize(*ImageBuffer))
      
      FreeMemory(*ImageBuffer) : *ImageBuffer = 0
      
      ; update the progress bar after the file is processed
      
      SetGadgetState(#PRB_BUILDING, iProcessed)
      
      
      Repeat
         iEvent = WindowEvent()
         
         If EventWindow() = #WIN_BUILD_PDF
            
            If iEvent = #PB_Event_CloseWindow
               flgAbort = #True
            EndIf
            
            If EventGadget() = #BTN_BUILD_CANCEL
               flgAbort = #True
            EndIf
            
         EndIf
         
         Delay(10)
         
      Until iEvent = 0
      
      
      If s_Options\flgMemoryLoad
         If (qFreeMem < qLowMem) And (flgWarned = #False)
            
            flgWarned = #True
            
            DisableWin(#WIN_BUILD_PDF)
            
            If ShowOverLoadWarning() <> #IDOK
               flgAbort = #True
            EndIf
            
            EnableWin(#WIN_BUILD_PDF)
            
            PumpMessages()
            
         EndIf
      EndIf
      
      If flgAbort: Break: EndIf
      
   Next
   
   FreeGadget(#PRB_BUILDING)
   
   
   If (Not flgAbort)
      DisableGadget(#BTN_BUILD_CANCEL, 1)
      
      FreeGadget(#TXT_BUILD_PDF_1)
      
      PumpMessages()
      
      SetGadgetText(#TXT_BUILD_PDF_2, "Saving PDF to disk, please wait ...")
      
      pdf_Save (OutFile$)
      
      I2P_CHECK_PDF (BuildPdf_PDF)
   EndIf
   
   If IsWindow(#WIN_BUILD_PDF)
      Delay(100)
      CloseWindow(#WIN_BUILD_PDF)
   EndIf
   
   ; remember last directory used for saving
   If s_Options\flgRememberLastDirectory
      s_Options\LastDirectory$ = GetPathPart(OutFile$)
   EndIf
   
   iRetVal = #True
   
   Goto BuildPdf_EXIT
   
   
   BuildPdf_GDI:
   If IsWindow(#WIN_BUILD_PDF): CloseWindow(#WIN_BUILD_PDF):  EndIf
   If *ImageBuffer : FreeMemory (*ImageBuffer): EndIf
   
   If i = #GDIPLUS_OUT_OF_MEMORY
      Goto BuildPdf_GDI_OUT_OF_MEMORY
   EndIf
   
   ; GDI+ fatal error
   HE(#False, "GDI+ unexpected error (code: " + Str(i) + " )")
   
   
   BuildPdf_PDF: ; PDF error
   If IsWindow(#WIN_BUILD_PDF): CloseWindow(#WIN_BUILD_PDF):  EndIf
   If *ImageBuffer : FreeMemory (*ImageBuffer): EndIf
   
   If pdf_GetErrorCode() = #PDF_ERR_OUT_OF_MEMORY
      Goto BuildPdf_PDF_OUT_OF_MEMORY
   EndIf
   
   If pdf_GetErrorCode() = #PDF_ERR_UNSUPPORTED_PNG
      ; PDF fatal error
      HE(#False, "Internal error. I have just built an unsupported PNG format (!)")
   EndIf
   
   If pdf_GetErrorCode() = #PDF_ERR_FILE_CREATION
      CenterRequesterPositionToWindow(WindowID(#WIN_MAIN))
      MessageBox_(WindowID(#WIN_MAIN), pdf_GetErrorMessage() + #CRLF$ + "Do you have the PDF already open in the reader ?", PROGRAM_TITLE(), #MB_OK | #MB_ICONEXCLAMATION | #MB_APPLMODAL)
      iRetVal = #False
      Goto BuildPdf_EXIT
   EndIf
   
   If pdf_GetErrorCode()
      CenterRequesterPositionToWindow(WindowID(#WIN_MAIN))
      MessageBox_(WindowID(#WIN_MAIN), pdf_GetErrorMessage(), PROGRAM_TITLE(), #MB_OK | #MB_ICONEXCLAMATION | #MB_APPLMODAL)
      iRetVal = #False
      Goto BuildPdf_EXIT
   EndIf
   
   
   BuildPdf_GDI_OUT_OF_MEMORY: ; out of memory inside GDI+
   CenterRequesterPositionToWindow(WindowID(#WIN_MAIN))
   MessageBox_(WindowID(#WIN_MAIN), "Out of memory inside a GDI+ function.", PROGRAM_TITLE(), #MB_OK | #MB_ICONEXCLAMATION | #MB_APPLMODAL)
   iRetVal = #False
   Goto BuildPdf_EXIT
   
   BuildPdf_PDF_OUT_OF_MEMORY: ; out of memory inside PDF
   CenterRequesterPositionToWindow(WindowID(#WIN_MAIN))
   MessageBox_(WindowID(#WIN_MAIN), "Out of memory inside a PDF function.", PROGRAM_TITLE(), #MB_OK | #MB_ICONEXCLAMATION | #MB_APPLMODAL)
   iRetVal = #False
   Goto BuildPdf_EXIT
   
   
   BuildPdf_EXIT:
   pdf_End()
   LogClose()
   FreeFont(nFontBold)
   PurgeMemory()
   
   If (*tLogfile\iTotFiles = 0)
      DeleteFile(OutFile$) ; pdf is empty (and invalid too)
   EndIf
   
   If (*tLogfile\iMissingFiles + *tLogfile\iFailedFiles) And (iRetVal = #True)
      CenterRequesterPositionToWindow(WindowID(#WIN_MAIN))
      Message$ = "There were some errors processing the files ..." + #CRLF$ + #CRLF$
      Message$ + "Files skipped : " + Str(*tLogfile\iFailedFiles) + #CRLF$ + #CRLF$
      Message$ + "Files missing : " + Str(*tLogfile\iMissingFiles) + #CRLF$ + #CRLF$
      Message$ + "Please see the log  " + Chr(34) + LogGetFileName() + Chr(34) + #CRLF$ + #CRLF$
      Message$ + "for more info." + #CRLF$
      MessageBox_(WindowID(#WIN_MAIN), Message$, PROGRAM_TITLE(), #MB_OK | #MB_ICONEXCLAMATION | #MB_APPLMODAL)
   EndIf
   
   
   If (*tLogfile\iMissingFiles + *tLogfile\iFailedFiles = 0) And (s_Options\flgDeleteLogIfSuccessful = #True)
      LogDelete()
   EndIf
   
   EnableWin(#WIN_MAIN)
   
   ProcedureReturn iRetVal
   
EndProcedure

Procedure PurgeMemory()
   EmptyWorkingSet (GetCurrentProcess_())
EndProcedure

Procedure.i ShowOverLoadWarning()
   Protected OverloadText$, SP$ = Space(8)
   Protected i
   
   CenterRequesterPositionToWindow(WindowID(#WIN_MAIN))
   
   OverloadText$ =                "Physical memory is running low."
   
   OverloadText$ + #CR$ + #CR$ +  "You are trying to create a PDF too big to fit in the available physical memory" + SP$
   OverloadText$ + #CR$ +         "so the OS is swapping data to your hard disk and using the virtual memory."
   
   OverloadText$ + #CR$ + #CR$ +  "You can go on and let the OS do its hard work or abort the current operation"
   OverloadText$ + #CR$ +         "to restart it again with more aggressive compression options."
   OverloadText$ + #CR$ + #CR$
   OverloadText$ + #CR$ +         "Do you want to continue with the current operation ?"
   OverloadText$ + #CR$ + #CR$
   
   i = MessageBox_(WindowID(#WIN_MAIN), OverloadText$, PROGRAM_TITLE(), #MB_OKCANCEL | #MB_ICONEXCLAMATION | #MB_APPLMODAL)
   
   ProcedureReturn i
EndProcedure

Procedure ClearListView()
   ClearGadgetItems(#LIG_FILES)
   ClearItemsData()
   SetBuildButtonState()
   CloseThumbnail()
   WinResize()
   SetActiveGadget(#LIG_FILES)
   UpdateStatusBar()
EndProcedure

Procedure.i AskForExit()
   Protected iRetVal
   
   If GetWindowState(#WIN_MAIN) = #PB_Window_Minimize
      CenterRequesterPositionToWindow(0)
   Else
      CenterRequesterPositionToWindow(WindowID(#WIN_MAIN))
   EndIf
   
   iRetVal = MessageBox_(WindowID(#WIN_MAIN), "Do you really want to quit ?", PROGRAM_TITLE(), #MB_YESNO | #MB_DEFBUTTON1 | #MB_ICONQUESTION | #MB_APPLMODAL)
   
   If iRetVal = #IDYES
      ProcedureReturn #True
   Else
      ProcedureReturn #False
   EndIf
   
EndProcedure

Procedure.i OverWriteIfAlreadyThere (FileName$)
   Protected iRetVal
   
   If FileSize(FileName$) >= 0
      CenterRequesterPositionToWindow(WindowID(#WIN_MAIN))
      iRetVal = MessageBox_(WindowID(#WIN_MAIN), "Do you want to overwrite the existing file ?", PROGRAM_TITLE(), #MB_YESNO | #MB_DEFBUTTON1 | #MB_ICONQUESTION | #MB_APPLMODAL)
      If iRetVal = #IDYES
         ProcedureReturn #True
      Else
         ProcedureReturn #False
      EndIf
   EndIf
   
   ProcedureReturn #True
EndProcedure

Procedure.i GetOuterPosition (nGadget, iYdrop)
   Protected iHeaderHeight = ListIconGadget_GetHeaderHeight(nGadget)
   
   If iYdrop < iHeaderHeight
      ProcedureReturn #ITEM_TOP
   Else
      ProcedureReturn #ITEM_BOTTOM
   EndIf
EndProcedure

Procedure GetLocalCursorPosXY (hWin, *iX, *iY)
   Protected tPOINT.POINT
   
   GetCursorPos_(@tPOINT)
   
   MapWindowPoints_(#HWND_DESKTOP, hWin, @tPOINT, 1)
   
   PokeI(*iX, tPOINT\x)
   PokeI(*iY, tPOINT\y)
   
EndProcedure

Procedure.s GetLowestDirPart(FullPath$)
   Protected Path$, Dir$
   Protected i, iLen
   
   Path$ = GetPathPart(FullPath$)
   
   If Path$
      iLen = Len(Path$)
      For i = iLen To 1 Step - 1
         If Mid(Path$,i,1) = "\"
            Break
         EndIf
      Next
      
      If i = 1 ; "\"
         ProcedureReturn "\"
      EndIf
      
      If i = 3 ; "c:\*.*"
         If Mid(Path$, 2, 1) = ":"
            ProcedureReturn "\"
         EndIf
      EndIf
      
      iLen = Len(Path$)
      For i = iLen - 1 To 1 Step - 1
         If Mid(Path$,i,1) = "\"
            Break
         EndIf
      Next
      
      Dir$ = Mid(Path$, i + 1, iLen - i - 1)
   EndIf
   
   ProcedureReturn Dir$
EndProcedure

Procedure.s GetTitleFromFilename (FileName$)
   Protected RetVal$ = GetFilePart(FileName$)
   
   ProcedureReturn ReplaceString(RetVal$, "." + GetExtensionPart(RetVal$), "")
EndProcedure

Procedure.i ExitMain()
   Shared s_Options
   
   If s_Options\flgConfirmations
      ProcedureReturn AskForExit()
   EndIf
   
   ProcedureReturn #True
EndProcedure

Procedure UpdateNumFiles()
   Shared s_FontStatusBar, s_TotFilesSize, s_lsItem()
   Protected Temp$
   Protected FilesText$ = "files"
   Protected iForeColor = #Black
   Protected iNumFiles = ListSize(s_lsItem())
   Protected hStatus = StatusBarID(#STB_STATUS)
   
   If iNumFiles
      If iNumFiles = 1 : FilesText$ = "file" : EndIf
      Temp$ = FormatQuad(iNumFiles) + " " + FilesText$ + Space(2) + FormatQuadWithByteSuffix(s_TotFilesSize, #False, 0)
      StatusBarEx_SetText(hStatus, 7, Temp$, iForeColor, #SBTEXT_EX_TRANSPARENT, #SBTEXT_EX_RIGHT, FontID(s_FontStatusBar))
   Else
      StatusBarEx_SetText(hStatus, 7, "", iForeColor, #SBTEXT_EX_TRANSPARENT, #SBTEXT_EX_RIGHT, FontID(s_FontStatusBar))
   EndIf
EndProcedure

Procedure UpdateStatusBar()
   Shared s_Options, s_FontStatusBar
   Protected Temp$
   Protected iForeColor = #Black
   Protected hStatus = StatusBarID(#STB_STATUS)
   
   ; jpg / png
   If s_Options\flgJpgCompression : Temp$ = "JPG :" + RSet(Str(s_Options\iJpgQuality),2," ") : Else : Temp$ = "PNG" : EndIf
   StatusBarEx_SetText(hStatus, 0, Temp$, iForeColor, #SBTEXT_EX_TRANSPARENT, #SBTEXT_EX_CENTER, FontID(s_FontStatusBar))
   
   ; resize %
   Temp$ = Str(s_Options\iImageScaling) + "%"
   StatusBarEx_SetText(hStatus, 1, Temp$, iForeColor, #SBTEXT_EX_TRANSPARENT, #SBTEXT_EX_CENTER, FontID(s_FontStatusBar))
   
   ; paper format
   Select s_Options\iPaperFormat
      Case #PAPER_FORMAT_PAPERLESS
         Temp$ = "PAPERLESS"
      Case #PAPER_FORMAT_A2
         Temp$ = "A2"
      Case #PAPER_FORMAT_A3
         Temp$ = "A3"
      Case #PAPER_FORMAT_A4
         Temp$ = "A4"
      Case #PAPER_FORMAT_LEGAL
         Temp$ = "US LEGAL"
      Case #PAPER_FORMAT_LETTER
         Temp$ = "US LETTER"
   EndSelect
   StatusBarEx_SetText(hStatus, 2, Temp$, iForeColor, #SBTEXT_EX_TRANSPARENT, #SBTEXT_EX_CENTER, FontID(s_FontStatusBar))
   
   ; startup
   Select s_Options\iPdfStartup
      Case 1 : Temp$ = "FIT PAGE"
      Case 2 : Temp$ = "FIT WIDTH"
      Case 3 : Temp$ = "TRUE SIZE"
      Case 4 : Temp$ = "FULL SCREEN"
   EndSelect
   StatusBarEx_SetText(hStatus, 4, Temp$, iForeColor, #SBTEXT_EX_TRANSPARENT, #SBTEXT_EX_CENTER, FontID(s_FontStatusBar))
   
   ; if paperless = on set gray color
   If s_Options\iPaperFormat = #PAPER_FORMAT_PAPERLESS
      iForeColor = GetSysColor_(#COLOR_GRAYTEXT)
   Else
      iForeColor = #Black
   EndIf
   
   ; potrtrait / landscape
   If s_Options\iPaperOrientation = #PAPER_PORTRAIT : Temp$ = "PORTRAIT" : Else : Temp$ = "LANDSCAPE" : EndIf
   StatusBarEx_SetText(hStatus, 3, Temp$, iForeColor, #SBTEXT_EX_TRANSPARENT, #SBTEXT_EX_CENTER, FontID(s_FontStatusBar))
   
   ; enlarge to fit
   If s_Options\flgEnlargeToFit : Temp$ = "ENLARGE" : Else : Temp$ = "" : EndIf
   StatusBarEx_SetText(hStatus, 5, Temp$, iForeColor, #SBTEXT_EX_TRANSPARENT, #SBTEXT_EX_CENTER, FontID(s_FontStatusBar))
   
   ; aspect ratio
   If s_Options\flgKeepAspect : Temp$ = "KEEP RATIO" : Else : Temp$ = "" : EndIf
   If s_Options\flgEnlargeToFit And s_Options\iPaperFormat <> #PAPER_FORMAT_PAPERLESS : iForeColor = #Black : Else : iForeColor = GetSysColor_(#COLOR_GRAYTEXT) : EndIf
   StatusBarEx_SetText(hStatus, 6, Temp$, iForeColor, #SBTEXT_EX_TRANSPARENT, #SBTEXT_EX_CENTER, FontID(s_FontStatusBar))
   
   ; num files
   UpdateNumFiles()
   
EndProcedure


Procedure ExpandDirectory (RootDirectory$, List lsFiles$(), flgRecursive)
   
   Protected iExDir, FileName$, FullFileName$, CurrDir$
   Static iCount
   
   iExDir = ExamineDirectory(#PB_Any, RootDirectory$, "*.*")
   
   If iExDir
      While NextDirectoryEntry(iExDir)
         If DirectoryEntryType(iExDir) = #PB_DirectoryEntry_File
            FileName$ = DirectoryEntryName(iExDir)
            FullFileName$ = RootDirectory$ + FileName$
            
            AddElement(lsFiles$())
            lsFiles$() = FullFileName$
            
            iCount + 1
         Else
            If flgRecursive = 1
               CurrDir$ = DirectoryEntryName(iExDir)
               If CurrDir$ <> "." And CurrDir$ <> ".."
                  ExpandDirectory (RootDirectory$ + CurrDir$ + "\", lsFiles$(), flgRecursive)
               EndIf
            EndIf
         EndIf
      Wend
      
      FinishDirectory(iExDir)
   EndIf
   
EndProcedure

Procedure.i ExpandDirectories (List lsFiles$(), flgRecursive)
   Protected CurrFile$
   
   ForEach lsFiles$()
      CurrFile$ = lsFiles$()
      If FileSize(CurrFile$) = -2 ; directory
         DeleteElement(lsFiles$())
         ExpandDirectory (CurrFile$ + "\", lsFiles$(), flgRecursive)
      EndIf
   Next
   
   ProcedureReturn ListSize(lsFiles$())
EndProcedure


Procedure.i StringGetWidth (nWindow, nFont, Text$)
   
   Protected iWidth
   
   StartDrawing(WindowOutput(nWindow))
   
   DrawingFont(nFont)
   
   iWidth = TextWidth(Text$)
   
   StopDrawing()
   
   ProcedureReturn iWidth
   
EndProcedure

Procedure Main(*tLogfile.T_LOGFILE)
   Shared s_Options, s_FontStatusBar, s_ItemDragged, s_UniqueID
   Protected tWinMain.T_WINDOW_RECT
   Protected iEvent, iRetVal, iSkipped, iItemCurrent, iHowManyFilesDropped, flgExitLoop
   Protected FilesDropped$, PdfOut$, PdfDefName$
   Protected flgSettingsConfirmed, flgAskAgain
   Protected iXdrop, iYdrop
   Protected flgCanceled
   Protected NewList lsFilesDropped$(), NewList lsFilesSkipped$()
   
   LoadSettings()
   
   s_UniqueID = 0
   
   tWinMain\X = 100
   tWinMain\Y = 100
   tWinMain\W = #WIN_WIDTH
   tWinMain\H = #WIN_HEIGHT
   
   If s_Options\flgRememberWinPos
      tWinMain\X = s_Options\tWinPos\X
      tWinMain\Y = s_Options\tWinPos\Y
      tWinMain\W = s_Options\tWinPos\W
      tWinMain\H = s_Options\tWinPos\H
   EndIf
   
   HE(OpenWindow(#WIN_MAIN, tWinMain\X, tWinMain\Y, tWinMain\W, tWinMain\H, PROGRAM_TITLE(), #PB_Window_SizeGadget|#PB_Window_MinimizeGadget|#PB_Window_MaximizeGadget|#PB_Window_Invisible), "Open window failed for #WIN_MAIN.")
   
   ListIconGadget(#LIG_FILES, 15, 15, 0, 0, "Filename", 1, #PB_ListIcon_CheckBoxes | #PB_ListIcon_FullRowSelect | #PB_ListIcon_AlwaysShowSelection)
   
   ; *WORKAROUND* remove the double buffer style added by PB 4.40
   SendMessage_(GadgetID(#LIG_FILES), #LVM_SETEXTENDEDLISTVIEWSTYLE, #LVS_EX_DOUBLEBUFFER, 0)
   
   SmartWindowRefresh(#WIN_MAIN, #True)
   
   AddGadgetColumn(#LIG_FILES, 1, "Resolution", #RESOLUTION_COLUMN_WIDTH)
   AddGadgetColumn(#LIG_FILES, 2, "Size", #FILESIZE_COLUMN_WIDTH)
   
   ListIconGadget_SetColumnAlignment(#LIG_FILES, 1, #LVCFMT_RIGHT)
   ListIconGadget_SetColumnAlignment(#LIG_FILES, 2, #LVCFMT_RIGHT)
   
   TextGadget(#TXT_JPG_STORING_MODE, 0,0, 140, 20, "", #PB_Text_Right)
   
   ButtonGadget(#BTN_BUILD_PDF, 15, 0, 200, 38, "Build PDF")
   
   CreateMenu(#MNU_WIN_MAIN, WindowID(#WIN_MAIN))
   
   MenuTitle("Files")
   
   MenuItem(#MNI_SORT_NOW, "Sort now")
   MenuItem(#MNI_SELECT_ALL, "Select all")
   MenuItem(#MNI_DESELECT_ALL, "Deselect all")
   MenuItem(#MNI_CLEAR, "Clear")
   MenuItem(#MNI_QUIT, "Quit")
   
   
   MenuTitle("Options")
   
   MenuItem(#MNI_STAYONTOP, "Stay on top")
   MenuItem(#MNI_REMEMBER_WIN_POS, "Remember window position")
   MenuItem(#MNI_SHOW_THUMBS, "Show thumbnail")
   
   MenuBar()
   MenuItem(#MNI_SETTINGS, "More options ...")
   
   
   MenuTitle("Help")
   MenuItem(#MNI_HELP, "Help" + Chr(9) + "F1")
   MenuItem(#MNI_ABOUT, "About")
   
   CreateStatusBar(#STB_STATUS, WindowID(#WIN_MAIN))
   
   s_FontStatusBar = LoadFont (#PB_Any, Control_GetFontName (StatusBarID(#STB_STATUS)), 8)
   
   AddStatusBarField(StringGetWidth (#WIN_MAIN, s_FontStatusBar, "JPG :10")+4) ; JPG :10
   AddStatusBarField(StringGetWidth (#WIN_MAIN, s_FontStatusBar, "100%")+4)    ; 100%
   AddStatusBarField(StringGetWidth (#WIN_MAIN, s_FontStatusBar, "PAPERLESS")) ; PAPERLESS
   AddStatusBarField(StringGetWidth (#WIN_MAIN, s_FontStatusBar, "LANDSCAPE")) ; LANDSCAPE
   AddStatusBarField(StringGetWidth (#WIN_MAIN, s_FontStatusBar, "FULL SCREEN")) ; FULL SCREEN
   AddStatusBarField(StringGetWidth (#WIN_MAIN, s_FontStatusBar, "ENLARGE"))     ; ENLARGE
   AddStatusBarField(StringGetWidth (#WIN_MAIN, s_FontStatusBar, "KEEP RATIO"))  ; KEEP RATIO
   
   AddStatusBarField(#PB_Ignore)
   
   ; this call enable you to trap click events in the statusbar
   StatusBarEx_GrabClickEvents (StatusBarID(#STB_STATUS), @StatusBar_ClickEvents_CallBack())
   
   UpdateStatusBar()
   
   AddKeyboardShortcut(#WIN_MAIN, #PB_Shortcut_F1, #MNI_HELP)
   AddKeyboardShortcut(#WIN_MAIN, #PB_Shortcut_Delete, #MNI_POP_DELETE)
   
   SetMenuItemState(#MNU_WIN_MAIN, #MNI_STAYONTOP, s_Options\flgStayOnTop)
   SetMenuItemState(#MNU_WIN_MAIN, #MNI_REMEMBER_WIN_POS, s_Options\flgRememberWinPos)
   
   SetMenuItemState(#MNU_WIN_MAIN, #MNI_SHOW_THUMBS, s_Options\flgShowThumbs)
   DisableMenuItem(#MNU_WIN_MAIN, #MNI_SHOW_THUMBS, Bool(s_Options\flgPreprocessing = #False))
   
   SetBuildButtonState()
   
   StickyWindow (#WIN_MAIN, s_Options\flgStayOnTop)
   
   ; enable drag and drop for the ListIconGadget
   EnableGadgetDrop(#LIG_FILES, #PB_Drop_Files, #PB_Drag_Copy)
   
   ; callback for the main window
   SetWindowCallback(@CallBack_WinMain(), #WIN_MAIN)
   
   ; set new winproc for the ListIconGadget
   SubclassWindow (GadgetID(#LIG_FILES), @CallBack_ListView())
   
   ; ... and for the ListIconGadget's header
   SubclassWindow (SendMessage_(GadgetID(#LIG_FILES), #LVM_GETHEADER, #Null, #Null), @CallBack_ListViewHdr())
   
   ; limit window resizing
   WindowBounds(#WIN_MAIN, #WIN_MIN_WIDTH, #WIN_MIN_HEIGHT, #PB_Ignore, #PB_Ignore)
   
   ShowWin (#WIN_MAIN)
   
   ResizeWindow(#WIN_MAIN, #PB_Ignore, #PB_Ignore, tWinMain\W, tWinMain\H)
   
   
   flgExitLoop = #False
   
   Repeat
      
      iEvent = WaitWindowEvent()
      
      Select iEvent
            
         Case #PB_Event_Gadget
            
            Select EventGadget()
                  
               Case #LIG_FILES
                  
                  If EventType() = #PB_EventType_DragStart
                     
                     s_ItemDragged = SendMessage_(GadgetID(#LIG_FILES), #LVM_GETHOTITEM,0,0)
                     
                     If s_ItemDragged >= 0
                        
                        SetGadgetState(#LIG_FILES, s_ItemDragged)
                        
                        EnableGadgetDrop(#LIG_FILES, #PB_Drop_Text, #PB_Drag_Move)  ; enable the drop for the local drag
                        
                        SetDragCallback(@CallBack_DragMain())
                        
                        CreateDragCursor()
                        
                        iRetVal = DragText(Str(s_ItemDragged), #PB_Drag_Move) ; start the drag storing the index of the item as data
                        
                        If iRetVal = #PB_Drag_None
                           CloseThumbnail()
                        EndIf
                        
                        DestroyDragCursor()
                        
                        EnableGadgetDrop(#LIG_FILES, #PB_Drop_Files , #PB_Drag_Copy) ; enable back the drop for the files
                        
                     EndIf
                     
                  EndIf
                  
                  If EventType() = #PB_EventType_LeftClick
                     iItemCurrent = GetGadgetState(#LIG_FILES)
                     
                     If iItemCurrent <> -1
                        SetItemStatus (iItemCurrent)
                     EndIf
                  EndIf
                  
                  If EventType() = #PB_EventType_Change
                     iItemCurrent = GetGadgetState(#LIG_FILES)
                     
                     If iItemCurrent <> -1
                        SendMessage_(WindowID(#WIN_MAIN), #WM_SHOW_THUMB, iItemCurrent, 0)
                        SetActiveGadget(#LIG_FILES)
                     EndIf
                  EndIf
                  
                  
               Case #BTN_BUILD_PDF
                  flgSettingsConfirmed = #True
                  
                  If s_Options\flgShowOptions
                     flgSettingsConfirmed = ShowSettingsPanel()
                  EndIf
                  
                  If flgSettingsConfirmed
                     UpdateStatusBar()
                     
                     Repeat
                        flgAskAgain = #False
                        
                        CenterRequesterPositionToWindow(WindowID(#WIN_MAIN))
                        
                        PdfOut$ = AskPdfName(PdfDefName$)
                        
                        If Len(PdfOut$)
                           
                           If s_Options\flgConfirmations
                              flgAskAgain = Bool(Not OverWriteIfAlreadyThere(PdfOut$))
                           EndIf
                           
                           If flgAskAgain = #False
                              
                              If s_Options\flgDocTitleFromFileName
                                 s_Options\DocTitle$ = GetTitleFromFilename(PdfOut$)
                              EndIf
                              
                              iRetVal = BuildPdf(PdfOut$, *tLogfile)
                              
                              If iRetVal
                                 If s_Options\flgOpenPdf
                                    If FileSize(PdfOut$) > 0
                                       RunProgram(PdfOut$)
                                    EndIf
                                 EndIf
                                 
                                 If s_Options\flgOpenDir
                                    If FileSize(PdfOut$) > 0
                                       RunProgram(GetPathPart(PdfOut$))
                                    EndIf
                                 EndIf
                                 
                              EndIf
                              
                           EndIf
                           
                        EndIf
                        
                     Until flgAskAgain = #False
                     
                  EndIf
                  
            EndSelect
            
         Case #PB_Event_GadgetDrop
            
            Select EventGadget()
                  
               Case #LIG_FILES
                  
                  Select EventDropType()
                        
                     Case #PB_Drop_Text
                        
                        iItemCurrent = GetGadgetState(#LIG_FILES)
                        
                        If iItemCurrent <> -1
                           
                           ItemDropped (Val(EventDropText()), iItemCurrent)
                           
                           ; select the new current item
                           
                           SetGadgetState(#LIG_FILES, -1)
                           
                           If Val(EventDropText()) > iItemCurrent
                              SetGadgetState(#LIG_FILES, iItemCurrent)
                           Else
                              SetGadgetState(#LIG_FILES, iItemCurrent - 1)
                           EndIf
                           
                           SetActiveGadget(#LIG_FILES)
                           
                        Else
                           GetLocalCursorPosXY (GadgetID(#LIG_FILES), @iXdrop, @iYdrop)
                           
                           ItemDroppedOuter (Val(EventDropText()), GetOuterPosition (#LIG_FILES, iYdrop))
                           
                           SetGadgetState(#LIG_FILES, -1)
                           SetGadgetState(#LIG_FILES, CountGadgetItems(#LIG_FILES) - 1)
                           SetActiveGadget(#LIG_FILES)
                        EndIf
                        
                     Case #PB_Drop_Files
                        FilesDropped$ = EventDropFiles() ; list of dropped files separated by chr(10)
                        
                        ClearList(lsFilesDropped$())
                        
                        iHowManyFilesDropped = SplitStringToWords(FilesDropped$ , #LF$, lsFilesDropped$())
                        
                        If iHowManyFilesDropped
                           
                           ; check if we have to keep previous data
                           If s_Options\flgClearBeforeDrop
                              ClearItemsData()
                              ClearGadgetItems(#LIG_FILES)
                              SetBuildButtonState()
                           EndIf
                           
                           FirstElement(lsFilesDropped$())
                           
                           If s_UniqueID = 0 And iHowManyFilesDropped = 1
                              ; if only a single directory was dropped
                              If FileSize(lsFilesDropped$()) = -2 ; directory
                                 PdfDefName$ = GetLowestDirPart(lsFilesDropped$() + "\") + ".pdf"
                              Else
                                 ; if only a single file was dropped
                                 PdfDefName$ = GetTitleFromFilename(lsFilesDropped$()) + ".pdf"
                              EndIf
                           Else
                              PdfDefName$ = ""
                           EndIf
                           
                           ExpandDirectories(lsFilesDropped$(), s_Options\flgEnableSubFolders)
                           
                           If s_Options\flgPreprocessing
                              iRetVal = ProcesFilesDropped(lsFilesDropped$(), lsFilesSkipped$(), @flgCanceled)
                           Else
                              iRetVal = AddFilesDropped (lsFilesDropped$())
                           EndIf
                           
                           If iRetVal
                              s_UniqueID + iRetVal
                              SetBuildButtonState()
                              
                              iSkipped = ListSize(lsFilesSkipped$())
                              
                              If iSkipped And (Not flgCanceled)
                                 ShowSkippedFiles (lsFilesSkipped$())
                                 ClearList(lsFilesSkipped$())
                              EndIf
                              
                              SetGadgetState(#LIG_FILES, 0)
                              SetActiveGadget(#LIG_FILES)
                           Else
                              SetGadgetState(#LIG_FILES, -1)
                              CenterRequesterPositionToWindow(WindowID(#WIN_MAIN))
                              MessageBox_(WindowID(#WIN_MAIN), "No files added, format not supported.", PROGRAM_TITLE(), #MB_OK | #MB_ICONEXCLAMATION | #MB_APPLMODAL)
                              SetGadgetState(#LIG_FILES, 0)
                              SetActiveGadget(#LIG_FILES)
                           EndIf
                           
                        EndIf
                        
                        UpdateStatusBar()
                  EndSelect
            EndSelect
            
         Case #PB_Event_Menu
            
            Select EventMenu()
               Case #MNI_HELP
                  OpenHelp(#PROGRAM_NAME$ + ".chm", "")
                  
               Case #MNI_QUIT
                  flgExitLoop = ExitMain()
                  
               Case #MNI_CLEAR
                  ClearListView()
                  
               Case #MNI_SORT_NOW
                  
                  SortListView()
                  
                  RefreshListView()
                  
                  SetGadgetState(#LIG_FILES, 0)
                  
               Case #MNI_REMEMBER_WIN_POS
                  s_Options\flgRememberWinPos = Bool(Not s_Options\flgRememberWinPos)
                  SetMenuItemState(#MNU_WIN_MAIN, #MNI_REMEMBER_WIN_POS, s_Options\flgRememberWinPos)
                  
               Case #MNI_STAYONTOP
                  
                  s_Options\flgStayOnTop = Bool (Not s_Options\flgStayOnTop)
                  SetMenuItemState(#MNU_WIN_MAIN, #MNI_STAYONTOP, s_Options\flgStayOnTop)
                  StickyWindow (#WIN_MAIN, s_Options\flgStayOnTop)
                  
               Case #MNI_SETTINGS
                  
                  ShowSettingsPanel()
                  
                  InvalidateRect_(GadgetID(#LIG_FILES), 0, 1) ; needed for 64 bit version ...
                  
                  UpdateStatusBar()
                  
                  StickyWindow (#WIN_MAIN, s_Options\flgStayOnTop)
                  
                  SetActiveGadget(#LIG_FILES)
                  
               Case #MNI_ABOUT
                  
                  ShowAbout()
                  SetActiveGadget(#LIG_FILES)
                  StickyWindow (#WIN_MAIN, s_Options\flgStayOnTop)
                  
               Case #MNI_POP_DELETE
                  
                  If GetGadgetState(#LIG_FILES) > -1
                     If CountCheckedItems(#LIG_FILES)
                        DeleteCheckedItems (#LIG_FILES)
                     Else
                        DeleteItem(GetGadgetState(#LIG_FILES))
                     EndIf
                     
                     SetActiveGadget(#LIG_FILES)
                     
                     WinResize()
                     
                     SetBuildButtonState()
                  EndIf
                  
                  
               Case #MNI_POP_MOVE_TO_TOP
                  iItemCurrent = GetGadgetState(#LIG_FILES)
                  
                  If CountCheckedItems(#LIG_FILES)
                     MoveCheckedItems (#LIG_FILES, #ITEM_TOP)
                  Else
                     MoveCheckedItem (iItemCurrent, #ITEM_TOP)
                  EndIf
                  
               Case #MNI_POP_MOVE_TO_BOTTOM
                  iItemCurrent = GetGadgetState(#LIG_FILES)
                  
                  If CountCheckedItems(#LIG_FILES)
                     MoveCheckedItems (#LIG_FILES, #ITEM_BOTTOM)
                  Else
                     MoveCheckedItem (iItemCurrent, #ITEM_BOTTOM)
                  EndIf
                  
               Case #MNI_SHOW_THUMBS
                  
                  s_Options\flgShowThumbs = Bool(Not s_Options\flgShowThumbs)
                  SetMenuItemState(#MNU_WIN_MAIN, #MNI_SHOW_THUMBS, s_Options\flgShowThumbs)
                  
                  If s_Options\flgShowThumbs
                     iItemCurrent = GetGadgetState(#LIG_FILES)
                     
                     If iItemCurrent <> -1
                        SendMessage_(WindowID(#WIN_MAIN), #WM_SHOW_THUMB, iItemCurrent, 0)
                        SetActiveGadget(#LIG_FILES)
                     EndIf
                  Else
                     CloseThumbnail()
                  EndIf
                  
               Case #MNI_POP_CLOSE
                  ; nothing to do
                  
               Case #MNI_SELECT_ALL
                  CheckAllItems(#LIG_FILES)
                  
               Case #MNI_DESELECT_ALL
                  UncheckAllItems(#LIG_FILES)
                  
            EndSelect
            
         Case #PB_Event_SizeWindow
            Select EventWindow()
               Case #WIN_MAIN : WinResize()
            EndSelect
            
         Case #PB_Event_CloseWindow
            
            Select EventWindow()
                  
               Case #WIN_MAIN
                  flgExitLoop = ExitMain()
                  
               Case #WIN_THUMB
                  s_Options\flgShowThumbs = #False
                  SetMenuItemState(#MNU_WIN_MAIN, #MNI_SHOW_THUMBS, s_Options\flgShowThumbs)
                  CloseThumbnail()
            EndSelect
            
            
      EndSelect ; iEvent
      
   Until flgExitLoop
   
   
   If SaveSettings() = #False
      CenterRequesterPositionToWindow(WindowID(#WIN_MAIN))
      MessageBox_(WindowID(#WIN_MAIN), "Unable to save settings on exit.", PROGRAM_TITLE(), #MB_OK | #MB_ICONEXCLAMATION | #MB_APPLMODAL)
   EndIf
   
   FreeFont(s_FontStatusBar)
   
   ClearItemsData()
   
   FreeImageEx(#IMA_THUMB)
   
   CloseWindow(#WIN_MAIN)
EndProcedure


Init()
Main(s_tLogfile)
Close()

; IDE Options = PureBasic 6.03 LTS (Windows - x64)
; Folding = ------------------
; Optimizer
; EnableXP
; EnableUser
; UseIcon = ..\res\acrobat.ico
; Executable = ..\out64\i2pdf.exe
; CPU = 1
; CompileSourceDirectory
; EnablePurifier
; EnableExeConstant